#ifndef GLBASEBREEDER_H_
#define GLBASEBREEDER_H_
#include "GLConstants.h"

class GLBaseCrossover;
class GLBaseOrganismStorage;
class GLBasePopulation;
/**
 * Base breeder class -- takes some container with organisms and
 * fills the new container with offsprings using certain crossover.
 */
class GLBaseBreeder
{
public:
  /**
   * Simple constructor. Assigns the crossover which will be used for
   * breeding.
   */
  GLBaseBreeder(GLBaseCrossover* crossover);

  /**
   * Destructor, destroys everything, including crossover.
   */
  virtual ~GLBaseBreeder();

  /**
   * Breeds organisms -- produces the offsprings from parents based
   * on certain criteria using the crossover owned by this breeder.
   * @param population population to which parents belong -- mainly to
   *    be able to check the uniqueness of the new offsprings.
   * @param potential_parents organisms to be bred.
   * @param offsprings storage place to store the children.
   * @param fill when true adds the organisms to the storage.
   * @param twins when true all the offsprings will be checked for
   *    uniqueness with the parent's population before being added to the
   *    storage of offsprings.
   * @param size_limits limit number of produced offsprings. When < 0
   *    all the offsprings generated by the algorithm will be returned.
   * @return number of offsprings produced.
   */
  virtual int breedOrganisms(GLBasePopulation* population,
                             GLBaseOrganismStorage* potential_parents,
                             GLBaseOrganismStorage* offsprings,
                             GLConstants::TStorageFilling fill =
                                 GLConstants::APPEND,
                             GLConstants::TTwinsFlags twins =
                                 GLConstants::NO_TWINS,
                             int size_limits = -1) = 0;

  /**
   * Returns the crossover which is used by the breeder.
   * @return crossover which this breeder uses.
   */
  GLBaseCrossover* getCrossover()
  { return m_crossover; }

  /**
   * Returns the crossover which is used by the breeder.
   * @return crossover which this breeder uses.
   */
  const GLBaseCrossover* getCrossover() const
  { return m_crossover; }
  
  /**
   * Resets breeder (e.g. sets counter, if any, to zero etc.) and its
   * crossover.
   */
  virtual void reset();
private:
  GLBaseCrossover *m_crossover;
};

#endif /* GLBASEBREEDER_H_ */
