#ifndef GLBASEFITNESS_H_
#define GLBASEFITNESS_H_
#include <NLInterfaceOutput.h>

#include "GLLogicError.h"
/**
 * Base class for fitness value storage. The main purpose of this
 * class to unify organisms for multiple and single objectives.
 */
class GLBaseFitness : public nlutils::NLInterfaceOutput
{
public:
  /**
   * Simple constructor.
   */
  GLBaseFitness() {};

  /**
   * Simple destructor
   */
  virtual ~GLBaseFitness() {};

  /**
   * Method to assign the value of fitness function to another one. Unlike
   * copying constructor it may be used to transform the family of fitness
   * classes to 'common denominator'. The resulting fitness not necessarily
   * should be of the same type as the current one. The purpose of this
   * method is to make the collection of statistics for multiple runs easier
   * and more general.
   * @param result fitness, which has the same value as current one, but 
   *    may be transformed to another type. E.g. this function may be
   *    called to transform GLFitness<int> to GLFitness<double>.
   */
  virtual void assignTo(GLBaseFitness& result) const
    {
      throw GLLogicError("GLBaseFitness:assignTo()\n"
                         "Is not implemented for this class\n");
    }

  /**
   * Returns the copy of the fittness class.
   * @return the copy of this class.
   */
  virtual GLBaseFitness* copy() const = 0;

  /**
   * Returns true if 2 fitness values are equal.
   * @param fitness pointer to fitness to make comparison with.
   */
  virtual bool isEqual(const GLBaseFitness* fitness) const = 0;

  /**
   * Returns true if this fitness is less then the one passed as
   * a parameter.
   * @param fitness pointer to fitness to make a comparison with.
   */
  virtual bool isLess(const GLBaseFitness* fitness) const = 0;

  /**
   * Class for use with stl::set etc.
   */
  class CompareFitness
  {
  public:
    /**
     * Returns true if left operand is < then the right one.
     * @returns true if left operand is < then the right one.
     */
    bool operator()(const GLBaseFitness* left,
                    const GLBaseFitness* right) const
        { return left->isLess(right); }
  };//class CompareOrganismsSoft

};

/**
 * Performs the comparison of 2 fitness functions.
 * @return true if left is less then right.
 */
inline bool operator<(const GLBaseFitness& left,
                 const GLBaseFitness& right)
  { return left.isLess(&right); }

/**
 * Performs the soft comparison of 2 fitness functions.
 * @return true if left is equal to right.
 */
inline bool operator==(const GLBaseFitness& left,
                 const GLBaseFitness& right)
  { return left.isEqual(&right); }

#endif /*GLBASEFITNESS_H_*/
