#include <iostream>
#include <stdlib.h>
#include <NLstl_utils.h>

#include "GLBaseBreeder.h"
#include "GLBaseCrossover.h"
#include "GLBaseEvaluator.h"
#include "GLBaseGaStatus.h"
#include "GLBaseGaStopper.h"
#include "GLBaseGeneticAlgorithm.h"
#include "GLBaseMutator.h"
#include "GLBasePopulation.h"
#include "GLBaseRanker.h"
#include "GLBaseSelectionist.h"
#include "GLBaseVariator.h"
#include "GLFactory.h"
#include "GLParametersGeneticAlgorithm.h"
#include "gl_utils.h"

using namespace std;
/**
  * Constructor. Creates the Genetic algorithm.
  * @param parameters genetic algorithm's parameters.
  * @param factory factory to create GA elements (mutator, crossover etc.).
 */
GLBaseGeneticAlgorithm::GLBaseGeneticAlgorithm(
    const GLParametersGeneticAlgorithm& parameters,
    GLFactory* factory)
  {
    m_parameters = nlutils::transform_pointer<GLParametersGeneticAlgorithm,
                      GLBaseParameters>(parameters.copy());
    initialise(factory);
    m_evaluator = NULL;
  }//constructor

GLBaseGeneticAlgorithm::~GLBaseGeneticAlgorithm()
  {
    if (m_breeder != NULL)      delete m_breeder;
    if (m_parameters != NULL)   delete m_parameters;
    if (m_selectionist != NULL) delete m_selectionist;
    if (m_variator != NULL)     delete m_variator;
  }//destructor

/**
 * Initialises GA -- creates crossover, mutator etc... Made private as
 * it is called in constructor and should not be virtual.
 * @param factory factory to create GA elements (mutator, crossover etc.).
 */
void GLBaseGeneticAlgorithm::initialise(GLFactory *factory)
  {
    //create necessary elements of the GA.
    //First the temporary ones, which will be passed as a parameters for
    //main elements
    GLBaseCrossover* crossover = factory->createCrossover(
              m_parameters->m_crossover_type,
              m_parameters->m_crossover_params,
              this);
    GLBaseMutator* mutator = factory->createMutator(
              m_parameters->m_mutator_type,
              m_parameters->m_mutation_rate,
              m_parameters->m_mutator_params,
              this);
    GLBaseRanker* ranker = factory->createRanker(
              m_parameters->m_ranker_type,
              m_parameters->m_ranker_params,
              this);
    //then the main elements.
    m_breeder = factory->createBreeder(
              m_parameters->m_breeder_type,
              crossover,
              m_parameters->m_breeder_params,
              this);

    m_selectionist = factory->createSelectionist(
              m_parameters->m_selectionist_type,
              ranker,
              m_parameters->m_selectionist_params,
              this);

    m_variator = factory->createVariator(
              m_parameters->m_variator_type,
              mutator,
              m_parameters->m_variator_params,
              this);
  }//initialise


/**
 * Resets the GA after run, so it is possible to make another run
 * with the same class instance. It should delete organisms from 
 * population, reset all the GA's elements (breeder, selectionist and
 *    variator) and set the status to GA_STATE_INITIALISED.
 * Be careful not to reset the GA while it is running.
 */
void GLBaseGeneticAlgorithm::reset()
  {
    m_breeder->reset();
    m_selectionist->reset();
    m_variator->reset();
  }//reset
/**
 * Method to determine if the criteria for stopping is satisfied.
 * Always return false if
 *      getStatus()->m_current_state != GLConstants::GA_STATE_RUNNING.
 * Always returns true if the stoppers list is empty.
 * @param stoppers list of stop criteria.
 * @param bool_operator if == AND_OPERATOR then return true when ALL
 *    stoppers are satisfied, when == OR_OPERATOR return true if
 *    at least ONE of the stoppers is satisfied.
 * @return true if GA is to stop and false otherwise.
 */
bool GLBaseGeneticAlgorithm::toFinish(const TListOfStoppers& stoppers,
                                      bool  bool_operator)
  {
    if (stoppers.empty()) return true;
    //take the current status of the GA
    const GLBaseGaStatus* status = getStatus();
    if (status->m_current_state != GLConstants::GA_STATE_RUNNING)
      return false;

    //loop through all stoppers
    bool stopper_status = false;
    TListOfStoppers::const_iterator it = stoppers.begin();
    while (it != stoppers.end())
      {
        stopper_status = (*it)->stop(status);
        //return false if AND_OPERATOR chosen and stopper is false.
        if ((bool_operator == GLConstants::AND_OPERATOR) &&
            (!stopper_status)) return false;
        //return true if OR operator chosen and stopper is true
        if ((bool_operator == GLConstants::OR_OPERATOR) &&
            (stopper_status)) return true;
        ++it;
      }

    //reached the end -- either all the stoppers showed false or all of them
    //showed true -- depending on the bool_operator status.
    return bool_operator == GLConstants::AND_OPERATOR;
  }//toFinish
