#ifndef GLBASEGENETICALGORITHM_H_
#define GLBASEGENETICALGORITHM_H_

#include "GLBaseBreeder.h"
#include "GLBaseSelectionist.h"
#include "GLBaseVariator.h"
#include "GLBaseGaStopper.h"
#include "GLConstants.h"
class GLBaseCrossover;
class GLBaseEvaluator;
class GLBaseGaStatus;
class GLBaseMutator;
class GLBaseOrganismInitialiser;
class GLBasePopulation;
class GLBaseRanker;
class GLFactory;

#include "GLParametersGeneticAlgorithm.h"

/**
 * Prototype of the genetic algorithm class.
 */
class GLBaseGeneticAlgorithm
{
public:
  /**
   * Constructor. Creates the genetic algorithm.
   * @param parameters genetic algorithm's parameters. They are copied
   *    within the GA so any further changes to the exemplar passed to
   *    constructor will not affect the GA behaviour.
   * @param factory factory to create GA elements (mutator, crossover etc.).
   */
  GLBaseGeneticAlgorithm(const GLParametersGeneticAlgorithm& parameters,
          GLFactory* factory);

  /**
   * Destructor. Destroys everything owned by the GA.
   */
  virtual ~GLBaseGeneticAlgorithm();

  /**
   * Returns the current breeder used by GA.
   * @return breeder currently in use by GA.
   */
  GLBaseBreeder* getBreeder()
  { return m_breeder; }

  /**
   * Returns the current breeder used by GA.
   * @return breeder currently in use by GA.
   */
  const GLBaseBreeder* getBreeder() const
  { return m_breeder; }

  /**
   * Returns the GA's crossover.
   * @return crossover used by this GA.
   */
  GLBaseCrossover* getCrossover()
  { return getBreeder()->getCrossover(); }

  /**
   * Returns the GA's crossover.
   * @return crossover used by this GA.
   */
  const GLBaseCrossover* getCrossover() const
  { return getBreeder()->getCrossover(); }

  /**
   * Returns the evaluator currently in use by the GA.
   */
  GLBaseEvaluator* getEvaluator() const
  { return m_evaluator; }

  /**
   * Returns the mutator currently in use by GA.
   * @return mutator which is currently used by this GA.
   */
  GLBaseMutator* getMutator()
  { return getVariator()->getMutator(); }

  /**
   * Returns the mutator currently in use by GA.
   * @return mutator which is currently used by this GA.
   */
  const GLBaseMutator* getMutator() const
  { return getVariator()->getMutator(); }

  /**
   * Returns the GA's parameters.
   * @return current ga's parameters.
   */
  const GLParametersGeneticAlgorithm* getParameters() const
  { return m_parameters; }
  /**
   * Returns the ranker which is currently in use.
   * @return ranker currently used by this GA.
   */
  GLBaseRanker* getRanker()
  { return getSelectionist()->getRanker(); }

  /**
   * Returns the ranker which is currently in use.
   * @return ranker currently used by this GA.
   */
  const GLBaseRanker* getRanker() const
  { return getSelectionist()->getRanker(); }

  /**
   * Returns the selectionist which is currently in use by GA.
   * @return the selectionist currently in use by GA.
   */
  GLBaseSelectionist* getSelectionist()
  { return m_selectionist; }

  /**
   * Returns the selectionist which is currently in use by GA.
   * @return the selectionist currently in use by GA.
   */
  const GLBaseSelectionist* getSelectionist() const
  { return m_selectionist; }

  /**
   * Returns the current status of the genetic algorithm.
   * @return pointer to the structure with the current status of the GA.
   */
  virtual const GLBaseGaStatus* getStatus() const = 0;

  /**
   * Returns the variator which is currently in use by GA.
   * @return variator currently in use by GA.
   */
  GLBaseVariator* getVariator()
  { return m_variator; }

  /**
   * Returns the variator which is currently in use by GA.
   * @return variator currently in use by GA.
   */
  const GLBaseVariator* getVariator() const
  { return m_variator; }

  /**
   * Resets the GA after run, so it is possible to make another run
   * with the same class instance. It should delete organisms from 
   * population, reset all the GA's elements (breeder, selectionist and
   *    variator) and set the status to GA_STATE_INITIALISED.
   * Be careful not to reset the GA while it is running.
   */
  virtual void reset();
  
  /**
   * Runs the GA.
   * @param initialiser organism initialiser -- will determine which
   *    type of organism will be used for GA. Ideally it should allow to
   *    use the same exemplar of GA for solving entirely different
   *    problems by changing only initialiser and evaluator.
   * @param evaluator class for evaluating the fitness function.
   * @param stoppers list of stoppers which determine when the GA is to stop.
   * @param bool_operator if == AND_OPERATOR then GA stops when ALL stoppers
   *    indicate that, when == OR_OPERATOR (default) then it happens when
   *    at least ONE of the stoppers is satisfied.
   * @return pointer to the structure with the GA status -- all the
   *    information necessary for post-processing the results, e.g.
   *    best ever organism in GA etc.
   */
  virtual const GLBaseGaStatus* runGA(
          GLBaseOrganismInitialiser*  initialiser,
                     GLBaseEvaluator* evaluator,
               const TListOfStoppers& stoppers,
                    bool  bool_operator = GLConstants::OR_OPERATOR) = 0;

protected:
  /**
   * Method to determine if the criteria for stopping is satisfied.
   * Always return false if
   *      getStatus()->m_current_state != GLConstants::GA_STATE_RUNNING.
   * Always returns true if the stoppers list is empty.
   * @param stoppers list of stop criteria.
   * @param bool_operator if == AND_OPERATOR then return true when ALL
   *    stoppers are satisfied, when == OR_OPERATOR return true if
   *    at least ONE of the stoppers is satisfied.
   * @return true if GA is to stop and false otherwise.
   */
  virtual bool toFinish(const TListOfStoppers& stoppers,
                        bool  bool_operator);
private:
  /**
   * Initialises GA -- creates crossover, mutator etc... Made private as
   * it is called in constructor and should not be virtual.
   * @param factory factory to create GA elements (mutator, crosso
   */
  void initialise(GLFactory* factory);
protected:
  ///Evaluator currently in use -- made  protected, as children will
  ///need to access it directly.
  GLBaseEvaluator*              m_evaluator;
private:
  GLBaseBreeder*                m_breeder;
  GLParametersGeneticAlgorithm* m_parameters;
  GLBaseSelectionist*           m_selectionist;
  GLBaseVariator*               m_variator;
};

#endif /* GLBASEGENETICALGORITHM_H_ */
