#ifndef GLBASEMUTATOR_H_
#define GLBASEMUTATOR_H_

class GLBaseOrganism;

/**
 * Abstract base class for all the mutators in GenLib.
 */
class GLBaseMutator
{
public:
  /**
   * Creates the base mutator.
   * @param mutation_rate rate of the mutation, interpretation is upto
   *    concrete implementation of the mutator. Normally one would expect
   *    to keep it in (0; 1] interval.
   * @param check_range if true, check is made that mutation rate is
   *    > 0 and < 1.
  */
  GLBaseMutator(double mutation_rate = 0.5,
                bool check_range = false)
    { setMutationRate(mutation_rate, check_range); }
  
  /**
   * Copying constructor.
   * @param mutator source for copying.
   */
  GLBaseMutator(const GLBaseMutator& mutator);
  
  /**
   * Destructor, destroys everything.
   */
  virtual ~GLBaseMutator(){};
  
  /**
   * Creates copy of the current exemplar of the mutator.
   * Should be overloaded in childrens.
   * @return copy of the current exemplar of the mutator.
   */
  virtual GLBaseMutator* copy() const = 0;
  
  /**
   * Returns the mutation rate. Its interpretation depends on concrete
   * implementation of the mutator.
   * @return the mutatio rate.
   */
  double getMutationRate() const
    { return m_mutation_rate; }
  
  /**
   * Performs the mutation on the organism. If passed organism
   * was changed (mutated) returns true, otherwise returns false.
   * @param organism organism to be mutated.
   * @return true if organism was mutated and false otherwise.
   */
  virtual bool mutateOrganism(GLBaseOrganism* organism) = 0;
 
  /**
   * Resets the mutator (e.g. sets counter, if any, to zero etc.).
   */
  virtual void reset();
  
 /**
  * Sets the new mutation rate for the mutator. 
  * @param mutation_rate rate of the mutation, interpretation is upto
  *    concrete implementation of the mutator. Normally one would expect
  *    to keep it in (0; 1) interval.
  * @param checkRange if true, check is made that mutation rate is
  *    > 0 and < 1.
 */
  void setMutationRate(double mutation_rate, bool checkRange = false);
  
private:
  double m_mutation_rate; //mutation rate -- interpretation of this value
                          //is upto concrete mutator implementation.
	
};

#endif /*GLBASEMUTATOR_H_*/
