#include "GLBaseOrganism.h"
#include "GLUtils.h"
#include "GLLogicError.h"

/**
  * Simple constructor, create instance of the organism.
  */
GLBaseOrganism::GLBaseOrganism()
{
   m_is_evaluated = false;
   m_locked_in_memory = true;
   m_tag = 0;
   m_fitness = NULL;
}//Constructor

/**
 * Copying constructor.
 * @param source source organism.
 */ 
GLBaseOrganism::GLBaseOrganism(const GLBaseOrganism& source)
  {
    m_is_evaluated = source.m_is_evaluated;
    if (m_is_evaluated) 
      {
        m_fitness = source.m_fitness->copy();
      }
    m_tag = source.m_tag;
  }//Constructor

/**
 * Destructor, destoys fitness function value if was evaluated.
 */
GLBaseOrganism::~GLBaseOrganism()
{
   if (m_fitness != NULL) delete m_fitness;
}//Destructor

/**
 * Checks if the passed pointer is compatible with the current class.
 * If not will throw GLLogicError. Normally this class should be 
 * overloaded in parents.
 * @param source class to check the compatibility with the current.
 * @message some text to be used in exception. E.g. where this function
 *    was called from.
 */
void GLBaseOrganism::assertCompatibility(const GLBaseOrganism* source,
                                const string& message) const
  {
    if (dynamic_cast<const GLBaseOrganism*>(source) == NULL)
      {
        char errormsg[1000];
        sprintf(errormsg, "\nGLBaseOrganism::assertCompatibility\n"
                          "%s\n"
                          "Passed pointer is either NULL or not compatible "
                          "with this class\n",
                          message.c_str());
        throw GLLogicError(errormsg);
      }
  }//assertCompatibility

/**
 * Evaluates the fitness of the organism. It is assumed, that 
 * implementations of the organism will then store the evaluated
 * value somewhere within.
 * @param evaluator the class for evaluating the fitness of the organism.
 * @return the value of the fitness function corresponding to this organism.
 */
const GLBaseFitness* GLBaseOrganism::evaluate(
        GLBaseEvaluator *evaluator) 
{ 
  setFitnessValue(evaluator->evaluateOrganism(this));
  return m_fitness;
}//evaluate


/**
 * Resets the fitness function -- sets the isEvaluated flag to false and
 * deletes the class which stores the fitness value of the organism.
 * @param flag new value of the flag to be set.
 */
void GLBaseOrganism::resetFitness()
{
  if (m_fitness != NULL) 
    {
      delete m_fitness;
    }
  m_is_evaluated = false; 
}//resetFitness

/**
 * Sets the new fitness value, destroying the previous one.
 * If fitness is == NULL that isEvaluated flag will be set to false,
 * otherwise to true.
 * @param fitness new fitness value of the organism.
 */
void GLBaseOrganism::setFitnessValue(const GLBaseFitness* fitness)
{
  if (m_fitness != NULL)
    {
      delete m_fitness;
    }
  m_fitness = fitness;
  m_is_evaluated = fitness != NULL;
}//setFitnessValue

/*-------------------------- INTERNAL CLASSES ---------------------*/

  /**
   * Returns true if left operand is < then the right one.
   * @param left left operand in 'less' expression.
   * @param right right operand in 'less' expression.
   * @returns true if left operand is < then the right one.
   */
bool GLBaseOrganism::CompareFitnessLess::operator()(
                  const GLBaseOrganism* left,
                  const GLBaseOrganism* right) const
{ 
  GLUtils::assertEvaluated(left, 
      "'left' from "
      "GLBaseOrganism::CompareOrganismsByFitness::operator()");
  GLUtils::assertEvaluated(right, 
      "'right' from "
      "GLBaseOrganism::CompareOrganismsByFitness::operator()");
  return left->getFitness()->isLess(right->getFitness()); 
}//operator()


