#ifndef GLBASEORGANISM_H_
#define GLBASEORGANISM_H_
#include <ostream>
#include <set>
#include <utility>
#include <vector>

#include <NLInterfaceOutput.h>

#include "GLBaseEvaluator.h"
#include "GLBaseFitness.h"

using namespace std;


/**
 * Base class for all organisms.
 */
class GLBaseOrganism: public nlutils::NLInterfaceOutput
{
public:
 /**
  * Simple constructor, create instance of the organism.
  */
  GLBaseOrganism();
  
  /**
   * Copying constructor.
   * @param source source organism.
   */ 
  GLBaseOrganism(const GLBaseOrganism& source);

  /**
   * Destructor, destoys fitness function value if was evaluated.
   */
  virtual ~GLBaseOrganism();

  /**
   * Checks if the passed pointer is compatible with the current class.
   * If not will throw GLLogicError. Normally this class should be 
   * overloaded in parents.
   * @param source class to check the compatibility with the current.
   * @param message some text to be used in exception. E.g. where this function
   *    was called from.
   */
  virtual void assertCompatibility(const GLBaseOrganism* source,
                                  const string& message) const;   

  /**
   * Copies the genetic information from the source organism
   * to the current one (e.g. chromosome, type or the organism,
   * bounds etc.) leaving out fitness information and similar 
   * not-internal data. Normally will reset fitness value.
   * @param source source organism.
   */
  virtual void copyGeneticData(const GLBaseOrganism *source) 
    { resetFitness(); };
  
  /**
   * Evaluates the fitness of the organism. It is assumed, that 
   * implementations of the organism will then store the evaluated
   * value somewhere within.
   * @param evaluator the class for evaluating the fitness of the organism.
   * @return the value of the fitness function corresponding to this 
   *         organism.
   */
  virtual const GLBaseFitness* evaluate(GLBaseEvaluator *evaluator);
     
  /**
   * Returns the pointer to the object with the fitness value or,
   * if organism is not evaluated, to the NULL pointer.
   */
  const GLBaseFitness* getFitness() const
  { return m_fitness; }
  
  /**
    * Returns the size of the organism.
    * @return the size of the organism.
   */
  virtual int getSize() const = 0;
  
  /**
   * Return the organism's tag.
   * @return the value of the organism's tag.
   */
  int getTag() const
      { return m_tag; }
  
  /**
   * Inverts the current state of the memory lock.
   * @return the resulting state of the memory lock.
   */
  bool invertMemoryLock()
    { return m_locked_in_memory = !(m_locked_in_memory); }
  /**
   * Returns true if 2 organisms are equal.
   * The comparison is NOT based on fittness (at least not fully).
   * @param organism another organism to be compared with.
   * @param soft when true soft comparison is done -- only
   *            gene's value. When false compares all others
   *            values -- fitness fucntion etc. 
   *            Tag's value are never compared.
   */
  virtual bool isEqual(const GLBaseOrganism *organism, 
               bool soft = true) const = 0;
  
  /**
   * Returns true if the fitness function for this organism was calculated.
   * @return true if the fitness function for this organism was calculated
   *    and false otherwise. It is assumed, when true, the value of the
   *    fitness function can be obtained from the organism.
   */
  bool isEvaluated() const
  { return m_is_evaluated; }
  
  /**
   * Returns true if this organism is less than one passed as a parameter.
   * The comparison is NOT based on fittness (at least not fully).
   * @param organism another organism to be compared with.
   * @param soft when true soft comparison is done -- based only on
   *            gene's value. When false compares all others
   *            values -- fitness fucntion etc. 
   *            Tag's value are never compared.
   */
  virtual bool isLess(const GLBaseOrganism *organism, 
               bool soft = true) const = 0;

  /**
   * Indicates, if the instance of this class is 'locked' in memory -- i.e.
   * if the organism was dynamically allocated, should it be deleted (false)
   * or not (true).
   * As normally in GenLib pointers to the organisms should be copied, instead
   * of real organisms (to minimise the overheads) this method may be used to
   * mark those organism which can be deleted safely.
   * @return true if organism should not be deleted (e.g. another pointer
   *    on it exists) and false if it can be done.
   */
  bool isLockedInMemory() const
  { return m_locked_in_memory; }

  /**
   * Lock the organism in memory -- indicates to container of pointers to
   * organisms that this organism should not be deleted.
   */
  void lockInMemory() 
    { m_locked_in_memory = true; } 
  
  /**
   * Creates the copy of the current organism. Do not forget to delete it,
   * once not needed anymore.
   * this->isEqual(makeClone(true), true) == true;
   * this->isEqual(makeClone(true), false) == ?;
   * this->isEqual(makeClone(false), false) == true; 
   * @param soft when true only genetic information (chromosome) is copied
   *            to the clone.
   * @return the copy of the current organsim.
   */
  virtual GLBaseOrganism* makeClone(bool soft = true) const = 0;
  
  /**
   * Resets the fitness function -- sets the isEvaluated flag to false and
   * deletes the class which stores the fitness value of the organism.
   */
  virtual void resetFitness(); 

  /**
   * Sets the new fitness value, destroying the previous one.
   * If fitness is == NULL that isEvaluated flag will be set to false,
   * otherwise to true.
   * @param fitness new fitness value of the organism.
   */
  virtual void setFitnessValue(const GLBaseFitness* fitness);

  /**
   * Sets value to the memory lock (see isLockedInMemory() method).
   * @param new_lock -- new value for the memory lock. False means this
   *    object may be deleted from memory, true -- otherwise.
   */
  void setMemoryLock(bool new_lock)
    { m_locked_in_memory = new_lock; } 
  
  /**
   * Tag the organism with the new value. Note, that method is const, but
   * as the m_tag is mutable the tag of the organism may be changed.
   * @param new_tag new value to be assiged to tag.
   */
  void setTag(int new_tag)
  { m_tag = new_tag; }
  
  /**
   * Unlock the organism in memory -- allows it to be destoyed.
   */
  void unlockInMemory() 
    { m_locked_in_memory = false; }
  
public:

  /**
   * Class for use with stl::set etc. 
   */  
  class CompareLess
  {
  public:
    /**
     * Determines wether the comparison class will do 'soft' comparison,
     * i.e. genetic material only, or 'hard' including, possibly, fittness.
     */
    CompareLess(bool soft_flag = true):tm_soft(soft_flag) {}
    /**
     * Returns true if left operand is < then the right one.
     * @returns true if left operand is < then the right one.
     */
    bool operator()(const GLBaseOrganism* left,
                    const GLBaseOrganism* right) const
        { return left->isLess(right, tm_soft); }
  private:
    bool tm_soft;
  };//class CompareLess

  /**
   * Class for use with stl algorithms line find etc. 
   */  
  class CompareEqual
  {
  public:
    ///type to satisfy STL requirements.
    typedef const GLBaseOrganism* first_argument_type;
    ///type to satisfy STL requirements.
    typedef const GLBaseOrganism* second_argument_type;
    ///type to satisfy STL requirements.
    typedef bool result_type;
    /**
     * Determines wether the comparison class will do 'soft' comparison,
     * i.e. genetic material only, or 'hard' including, possibly, fittness.
     */
    CompareEqual(bool soft_flag = true):tm_soft(soft_flag) {}
    /**
     * Returns true if left operand is == then the right one.
     * @returns true if left operand is == then the right one.
     */
    result_type operator()(first_argument_type left,
                           first_argument_type right) const
        { return left->isEqual(right, tm_soft); }
  private:
    bool tm_soft;
  };//class CompareEqual
  
  /**
   * Class for use with stl, based on comparisons of organisms by
   * fitness value.
   */
  class CompareFitnessLess
  {
  public:
    /**
     * Returns true if left operand is < then the right one.
     * @returns true if left operand is < then the right one.
     */
    bool operator()(const GLBaseOrganism* left,
                    const GLBaseOrganism* right) const;
  };//class CompareFitnessLess

  /**
   * Class to be used with STL containers -- clones organism.
   */
  class Cloner
  {
  public:
    /**
     * Constructor. By default set cloning to soft mode.
     * @param soft_clone if true this class will perform 'soft' cloning
     *          (only genetic information), what false -- all the class
     *          data will be copied.
     */
    Cloner(bool soft_clone = true): tm_soft_clone(soft_clone){}
    
    /**
     * Does the cloning.
     * @param organism organism to be cloned.
     * @return clone of the organism.
     */
    GLBaseOrganism* operator()(const GLBaseOrganism* organism)
    { return organism->makeClone(tm_soft_clone); }
  private:
    bool tm_soft_clone; //when true soft cloning is used.
  };  //Cloner class
  
private:
  bool m_is_evaluated;  //set to true if the organism was evaluated and
                        //can return the fitness value.
  const GLBaseFitness* m_fitness; //place to store the fitness of the 
                                  //organism.
  int m_tag;             //can be used to tag organisms. This value never 
                         //saved and should be used in short transactions
                         //only.
  bool m_locked_in_memory;  //when true the organism should not be deleted from
                            //the memory.
};

/**
 * Performs the soft comparison of 2 classes.
 * @return true if left is less then right in the 'soft' sence.
 */
inline bool operator<(const GLBaseOrganism& left, 
                 const GLBaseOrganism& right)
  { return left.isLess(&right); }

/**
 * Performs the soft comparison of 2 classes.
 * @return true if left is equal to right in the 'soft' sence.
 */
inline bool operator==(const GLBaseOrganism& left, 
                 const GLBaseOrganism& right)
  { return left.isEqual(&right); }


/**
 * Type for set of pointer to the organisms.
 */
typedef set<GLBaseOrganism*, 
            GLBaseOrganism::CompareLess> 
                                    TGLSetOfOrganisms;

/**
 * Type for set of constant pointer to the organisms.
 */
typedef set<const GLBaseOrganism*, 
            GLBaseOrganism::CompareLess>
                                    TGLSetOfOrganismsConst;

/**
 * Type for vector of pointer to the organisms.
 */
typedef vector<GLBaseOrganism*> TGLVectorOfOrganisms;

/**
 * Type for vector of constant pointer to the organisms.
 */
typedef vector<const GLBaseOrganism*> TGLVectorOfOrganismsConst;

#endif /*GLBASEORGANISM_H_*/
