#include "GLBaseOrganism.h"
#include "GLBaseOrganismStorage.h"

/**
 * Simple constructor. Sets the memory policy to GLConstants::DO_NOT_FREE
*/
GLBaseOrganismStorage::GLBaseOrganismStorage()
{
  m_free_memory = GLConstants::DO_NOT_FREE;
}//Constructore

/**
 * Destructor. Deletes the pointers, if necessary.
 */
GLBaseOrganismStorage::~GLBaseOrganismStorage()
{
}

/**
 * Frees memory allocated to all organisms in storage (but do not change
 * the SIZE of the storage, so should be used with care) starting from
 * iterator. If iterator == NULL then deletes all elements from the start.
 * @param start iterator to start from.
 */
void GLBaseOrganismStorage::deleteAllOrganisms(iterator* start)
  {
    //removes the locks for organisims.
    setMemoryLockForAllOrganisms(false, start);
    //delete organisms
    deleteOrganisms(start);
  }//deleteAllOrganisms

/**
 * Deletes organisms which are 'unlocked' in memory i.e. those, for which
 * method isLockedInMemory() returns false, starting from iterator. 
 * If iterator == NULL then starts from the beginning.
 * @param start iterator to start from.
 */
void GLBaseOrganismStorage::deleteOrganisms(iterator* start)
  {
    //determine the starting point.
    iterator *it = start;
    if (it == NULL)
      it = getIterator();
    //loop until end
    for(; !it->isEnd(); it->moveNext())
      {
        //skip NULL elements
        if (it->getElement() == NULL) continue;
        //skip those which are not tagged as REMOVE
        if (it->getElement()->isLockedInMemory()) continue;
        //delete element
        delete it->getElement();
        it->getElement() = NULL;
      }
    //delete iterator if it was created.
    if (start == NULL) delete it;
  }//deleteMarkedOrganisms

/**
 * Transfers the organisms from the vector to the storage. Note, that
 * vector cannot be vector of constants. Base class provides implementation
 * which for efficiency reasons can be overloaded in concrete classes.
 * @param vector vector with organism to be placed in storage.
 * @param fill if APPEND the elements will be added to the storage (only
 *    those which are not in storage already), when REPLACE the storage
 *    will be emptied before adding the content of the vector.
 * @param size how many organisms from vector to be transferred. When < 0
 *     or >- vector.size() will use all the elements from the vector.
 * @return how many elements were actually transferred (e.g. excluding
 *    those, which were in storage already).
 */
int GLBaseOrganismStorage::fillFromVector(
                           const vector<GLBaseOrganism*>& vector,
                           GLConstants::TStorageFilling fill,
                           int size)
  {
    //clean the storage if necessary
    if (fill == GLConstants::REPLACE) emptyStorage();

    //determine, how many elements we are going to transfer
    int to_transfer = size;
    if ((size < 0) || (size > (int)vector.size()))
      {
        to_transfer = vector.size();
      }
    int counter = 0; //number of elements actually transferred.

    for(int i = 0; i < to_transfer; i++)
      {
        //add organisms one-by-one
        if (addOrganism(vector[i]) == true)
        {
          counter++;
        }
        else
        {
          delete vector[i];
        }
      }
    return counter;
  }//fillFromVector

/**
 * Checks, if the organism is already in storage (compared not pointers,
 * but the content of the organisms). The base implementation uses
 * simple sequential search using iterator. However, depending on
 * implementation of the storage in concrete class it may be replaced
 * with the more efficient verision.
 * @param organism organism to be found.
 * @param soft when true comparison is made in the 'soft' sense.
 * @return pointer on found organism in the storage or NULL if nothing
 *    was found.
 */
const GLBaseOrganism* GLBaseOrganismStorage::findOrganism(
                                     const GLBaseOrganism* organism,
                                     bool soft) const
  {
     const_iterator* it = getIteratorConst();
     //loop though all elements in storage
     for (; !it->isEnd(); it->moveNext())
       {
         if (it->getElement()->isEqual(organism, soft))
            break;
       }
     //if something was found - keep it, if not -- set NULL.
     const GLBaseOrganism* found = it->isEnd() ? NULL : it->getElement();
     delete it;
     return found;
  }//findOrganism

/**
 * Provides the vector of pointers to the organisms in storage starting
 * from the first one. The base version uses iterator to fill the
 * vector, however, for real implementation of storage which uses stl
 * containers more efficient approach should be possible.
 * @param container vector of pointers to the organisms from storage
 *                    (output value).
 * @param fill when APPEND the new organisms are added to the container,
 *    when REPLACE -- the container is cleaned before adding the
 *    organisms.
 * @param size number of organisms from storage to be returned in vector.
 *            if < 0 or > getSize() then returns all the organisms.
 * @param copy_clones when true the pointers to the clones of the organisms
 *    will be copied, otherwise -- pointers to the original organisms.
 * @return number of elements transferred to container.
 *
 */
int GLBaseOrganismStorage::getOrganismsVector(
                       vector<const GLBaseOrganism*>& container,
                       GLConstants::TStorageFilling fill,
                       int size,
                       bool copy_clones) const
{
  //empty container if not appended.
  if(fill == GLConstants::REPLACE) container.clear();
  //resize accordingly
  int to_transfer = size;
  if ((size == -1) || (size >= getSize()))
      to_transfer = getSize();
  int container_start = container.size();
  container.resize(container.size() + to_transfer);

  //set both iterators (for container and storage) to the start
  vector<const GLBaseOrganism*>::iterator it_cont =
          container.begin() + container_start;
  const_iterator* it = getIteratorConst();
  for(int i = 0; i < to_transfer; i++)
    {
      *it_cont = copy_clones ?
                 it->getElement()->makeClone(false) :
                 it->getElement();
      ++it_cont;
      it->moveNext();
    }
  delete it;
  return to_transfer;
 }//getOrganismsVector

/**
 * Provides the vector of pointers to the organisms in storage starting
 * from the first one. The base version uses iterator to fill the
 * vector, however, for real implementation of storage which uses stl
 * containers more efficient approach should be possible.
 * @param container vector of pointers to the organisms from storage
 *                    (output value).
 * @param fill when APPEND the new organisms are added to the container,
 *    when REPLACE -- the container is cleaned before adding the
 *    organisms.
 * @param size number of organisms from storage to be returned in vector.
 *            if < 0 or > getSize() then returns all the organisms.
 * @param copy_clones when true the pointers to the clones of the organisms
 *    will be copied, otherwise -- pointers to the original organisms.
 * @return number of elements transferred to container.
 *
 */
int GLBaseOrganismStorage::getOrganismsVector(
                       vector<GLBaseOrganism*>& container,
                       GLConstants::TStorageFilling fill,
                       int size,
                       bool copy_clones)
{
  //empty container if not appended.
  if(fill == GLConstants::REPLACE) container.clear();
  //resize accordingly
  int to_transfer = size;
  if ((size == -1) || (size >= getSize()))
      to_transfer = getSize();
  int container_start = container.size();
  container.resize(container.size() + to_transfer);

  //set both iterators (for container and storage) to the start
  vector<GLBaseOrganism*>::iterator it_cont =
          container.begin() + container_start;
  iterator* it = getIterator();
  for(int i = 0; i < to_transfer; i++)
    {
      *it_cont = copy_clones ?
                 it->getElement()->makeClone(false) :
                 it->getElement();
      ++it_cont;
      it->moveNext();
    }
  delete it;
  return to_transfer;
}//getOrganismsVector


/**
 * Sets the memory lock (true locked, false unlocked) for all organisms in 
 * the storage starting from the iterator until end. If iterator == NULL
 * then starts from the beginning.
 * @param new_lock the new value for memory lock flag for all organisms.
 * @param start the iterator on the first organism starting from which
 *    the locks will be changed.
 */
void GLBaseOrganismStorage::setMemoryLockForAllOrganisms(
                      bool new_lock, 
                      iterator* start)
  {
    //determine the starting point.
    iterator *it = start;
    if (it == NULL)
      it = getIterator();
    //loop until end
    for(; !it->isEnd(); it->moveNext())
      {
        //skip NULL elements
        if (it->getElement() == NULL) continue;
        //change the lock
        it->getElement()->setMemoryLock(new_lock);
      }
    if (start == NULL) delete it;
  }//setMemoryLockForAllOrganisms
