#ifndef GLBASEORGANISMSTORAGE_H_
#define GLBASEORGANISMSTORAGE_H_
#include <vector>
#include "GLConstants.h"
#include "GLIterators.h"
using namespace std;
class GLBaseOrganism;
/**
 * Class for storing pointers to the organisms within the population.
 * Should provide basic sequential access to the organisms via iterator.
 *
 */
class GLBaseOrganismStorage
{
public:
  /**
   * Prototype for the ordinary iterator.
   */
  typedef GLBaseBidirectOrganismIterator iterator;
  /**
   * Prototype for constant iterator.
   */
  typedef GLBaseBidirectOrganismIteratorConst const_iterator;

  /**
   * Simple constructor. Sets the memory policy to GLConstants::DO_NOT_FREE
   */
  GLBaseOrganismStorage();

  /**
   * Destructor. Deletes the pointers, if necessary.
   */
  virtual ~GLBaseOrganismStorage();

  /**
   * Adds the new organism to storage. If the 'no duplication' flag is set and
   * organism is already in the storage -- do not add it.
   * @param organism new organism to be added to storage.
   * @param twins when set to NO_TWINS, check that this organism is not in
   *    the storage already.
   * @return true if organism was successfully added and false otherwise.
   */
  virtual bool addOrganism(GLBaseOrganism* organism,
                           GLConstants::TTwinsFlags twins =
                             GLConstants::NO_TWINS) = 0;

  /**
   * Frees memory allocated to all organisms in storage (but do not change
   * the SIZE of the storage, so should be used with care) starting from
   * iterator. If iterator == NULL then deletes all elements from the start.
   * @param start iterator to start from.
   */
  virtual void deleteAllOrganisms(iterator* start = NULL);

  /**
   * Deletes organisms which are 'unlocked' in memory i.e. those, for which
   * method isLockedInMemory() returns false, starting from iterator.
   * If iterator == NULL then starts from the beginning.
   * @param start iterator to start from.
   */
  virtual void deleteOrganisms(iterator* start = NULL);

  /**
   * Removes all the organisms from the storage. May free memory depending
   * on memory policy. After this function is called the size of the storage
   * is 0.
   */
  virtual void emptyStorage() = 0;

  /**
   * Transfers the organisms from the vector to the storage. Note, that
   * vector cannot be vector of constants. Base class provides implementation
   * which for efficiency reasons can be overloaded in concrete classes.
   * @param vector vector with organism to be placed in storage.
   * @param fill if APPEND the elements will be added to the storage (only
   *    those which are not in storage already), when REPLACE the storage
   *    will be emptied before adding the content of the vector.
   * @param size how many organisms from vector to be transferred. When < 0
   *     or >- vector.size() will use all the elements from the vector.
   * @return how many elements were actually transferred (e.g. excluding
   *    those, which were in storage already).
   */
  virtual int fillFromVector(const vector<GLBaseOrganism*>& vector,
                             GLConstants::TStorageFilling fill =
                               GLConstants::APPEND,
                             int size = -1);

  /**
   * Checks, if the organism is already in storage (compared not pointers,
   * but the content of the organisms). The base implementation uses
   * simple sequential search using iterator. However, depending on
   * implementation of the storage in concrete class it may be replaced
   * with the more efficient verision.
   * @param organism organism to be found.
   * @param soft when true comparison is made in the 'soft' sense.
   * @return pointer on found organism in the storage or NULL if nothing
   *    was found.
   */
  virtual const GLBaseOrganism* findOrganism(
                                    const GLBaseOrganism* organism,
                                    bool soft = true) const;

  /**
   * Returns the pointer to iterator at the first element of the storage.
   * @return pointer to iterator at the first element of the storage.
   */
  virtual iterator* getIterator() = 0;

  /**
   * Returns the pointer to constant iterator at the first element of
   * the storage.
   * @return pointer to constant iterator at the first element of
   * the storage.
   */
  virtual const_iterator* getIteratorConst() const = 0;

  /**
   * Returns the policy on freeing memory.
   * @return the policy on freeing memory.
   */
  GLConstants::TMemoryPolicy getMemoryPolicy() const
    { return m_free_memory; }

  /**
   * Provides the vector of pointers to the organisms in storage starting
   * from the first one. The base version uses iterator to fill the
   * vector, however, for real implementation of storage which uses stl
   * containers more efficient approach should be possible.
   * @param container vector of pointers to the organisms from storage
   *                    (output value).
   * @param fill when APPEND the new organisms are added to the container,
   *    when REPLACE -- the container is cleaned before adding the
   *    organisms.
   * @param size number of organisms from storage to be returned in vector.
   *            if < 0 or > getSize() then returns all the organisms.
   * @param copy_clones when true the pointers to the clones of the organisms
   *    will be copied, otherwise -- pointers to the original organisms.
   * @return number of elements transferred to container.
   *
   */
  virtual int
      getOrganismsVector(vector<const GLBaseOrganism*>& container,
                         GLConstants::TStorageFilling fill =
                           GLConstants::APPEND,
                         int size = -1,
                         bool copy_clones = false) const;

  /**
   * Provides the vector of pointers to the organisms in storage starting
   * from the first one. The base version uses iterator to fill the
   * vector, however, for real implementation of storage which uses stl
   * containers more efficient approach should be possible.
   * @param container vector of pointers to the organisms from storage
   *                    (output value).
   * @param fill when APPEND the new organisms are added to the container,
   *    when REPLACE -- the container is cleaned before adding the
   *    organisms.
   * @param size number of organisms from storage to be returned in vector.
   *            if < 0 or > getSize() then returns all the organisms.
   * @param copy_clones when true the pointers to the clones of the organisms
   *    will be copied, otherwise -- pointers to the original organisms.
   * @return number of elements transferred to container.
   *
   */
  virtual int
      getOrganismsVector(vector<GLBaseOrganism*>& container,
                         GLConstants::TStorageFilling fill =
                           GLConstants::APPEND,
                         int size = -1,
                         bool copy_clones = false);

  /**
   * Returns the number of elements in storage.
   * @return number of elements in storage.
   */
  virtual int getSize() const = 0;

  /**
   * Adds organisms from another storage to the current one.
   * @param storage another organism storage.
   * @param twins when equal to NO_TWINS, check that this organism is not in
   *    the storage already.
   */
  virtual void merge(const GLBaseOrganismStorage* storage,
                     GLConstants::TTwinsFlags twins =
                       GLConstants::NO_TWINS) = 0;

  /**
   * Sets the new policy on freeing memory.
   * @param new_policy new memory handling policy.
   */
  void setMemoryPolicy(GLConstants::TMemoryPolicy new_policy)
    { m_free_memory = new_policy; }

  /**
   * Decreases the size of the storage. If the size of storage was less, than
   * new size, that nothing will be done. The memory allocated for
   * organisms will be dealt with according to memory policy.
   * @param new_size new number of organisms in the storage.
   * @return the number of organisms in storage after operation.
   */
  virtual int trimSize(unsigned int new_size) = 0;
  
  /**
   * Sets the memory lock (true locked, false unlocked) for all organisms in 
   * the storage starting from the iterator until end. If iterator == NULL
   * then starts from the beginning.
   * @param new_lock the new value for memory lock flag for all organisms.
   * @param start the iterator on the first organism starting from which
   *    the locks will be changed.
   */
  virtual void setMemoryLockForAllOrganisms(bool new_lock, 
                                           iterator* start = NULL);

private:

  //determines what to do with the memory allocated to the organisms
  //once the storage is destroyed.
  GLConstants::TMemoryPolicy m_free_memory;
};

#endif /*GLBASEORGANISMSTORAGE_H_*/
