#include <iostream>

#include "GLBaseOrganism.h"
#include "GLBaseOrganismStorage.h"
#include "GLBasePopulation.h"
#include "GLBaseRanker.h"
#include "GLRuntimeError.h"
#include "GLUtils.h"

/**
 * Constructor, creates the population of given size and initialising it,
 * if necessary.
 * @param storage main storage container for keeping the organisms.
 */
GLBasePopulation::GLBasePopulation(GLBaseOrganismStorage *storage)
  {
    if (storage == NULL)
      {
        char errormsg[1000];
        sprintf(errormsg, "\nGLBasePopulation::GLBasePopulation\n"
                          "NULL pointer provided to constructor\n");
        throw GLRuntimeError(errormsg);
      }
    m_main_storage = storage;
  }//Constructor

/**
 * Destroys everything worth destroying.
 */
GLBasePopulation::~GLBasePopulation()
{
  if (m_main_storage != NULL)
    {
#if DEBUG > 2      
      cout << m_main_storage->getSize() << " deleted from main storage\n";
#endif
      m_main_storage->deleteAllOrganisms();
      delete m_main_storage;
    }
}

/**
 * Creates new generation from the existing pool of organism. The
 * general algorithm is following:
 *    1) Create offsprings by using the crossover operator.
 *    2) Create mutants by applying mutator.
 *    3) Evaluate all or new only organisms.
 *    4) Keep the fittest.
 * @param breeder class for producing the offsprings.
 * @param variator class to mutate the organisms in container.
 * @param evaluator class for evaluating the organisms.
 * @param selectionist class to select those to be included in the next
 *            generation.
 * @param best_organisms best organism(s) in newly created population.
 *            The previous content of this vector will be lost.
 * @param max_offsprings maximum number of offsprings generated.
 *            if < 0 -- number is limited only by the breeding algorithm
 *            used.
 * @param max_mutants maximum number of mutants generated.
 *            if < 0 -- number is limited only by the mutation algorithm
 *            used.
 * @param mutation_policy determines whether mutator to be applied to 
 *            children and / or to parents.
 * @param new_size the size of the new population. If < 0 its size will be
 *            equal to the size of the current population.
 * @return number of new organisms in the new generation.
 */
int GLBasePopulation::createNewGeneration(
                                 GLBaseBreeder* breeder,
                                 GLBaseVariator* variator,
                                 GLBaseEvaluator* evaluator,
                                 GLBaseSelectionist* selectionist,
                                 vector<const GLBaseOrganism*>& 
                                                         best_organisms,
                                 int max_offsprings,
                                 int max_mutants,
                                 int mutation_policy,
                                 int new_size)
  {
    //create offsprints.
#if DEBUG > 2    
    int children =
#endif      
    breed(breeder, max_offsprings);
    //mutate the population.
#if DEBUG > 2    
    int mutated = 
#endif    
    mutate(variator, max_mutants, mutation_policy);
    //evaluate all new organisms.
    evaluateAll(evaluator);
    //create new generation using the fitness value for all organisms.
    int new_organisms = keepFittest(selectionist, best_organisms, new_size);
    
#if DEBUG > 2
    std::cout << children << " children were born\n";
    std::cout << mutated << " mutants was produced\n";
    std::cout << new_organisms << " new organisms appeared in population\n";
#endif
    //Clean up and prepare for the next loop.
    prepareForNextGeneration();
    return new_organisms;
  }//createNewGeneration

/**
 * DEBUG METHOD. Evaluates the organisms in population and prints
 * them in ranked order. Note, that this debug method may change the
 * organisms' states by evaluating them.
 * @param out output stream.
 * @param evaluator evaluator for evaluating elements.
 * @param ranker operator to arrange the organisms according to their rank.
 */
void GLBasePopulation::printRankedPopulation(
                                   ostream& out,
                                   GLBaseEvaluator* evaluator,
                                   GLBaseRanker *ranker)
  {
    //evaluate all
    evaluateAll(evaluator);
    TGLVectorOfOrganisms organisms;
    m_main_storage->getOrganismsVector(organisms);
    ranker->rankOrganisms(organisms);
    GLUtils::printVectorOfOrganisms(out, organisms);
  }//printRankedPopulation
