#ifndef GLBASEPOPULATION_H_
#define GLBASEPOPULATION_H_
#include "GLBaseOrganismInitialiser.h"
#include "GLBaseOrganismStorage.h"
#include "GLConstants.h"

class GLBaseBreeder;
class GLBaseEvaluator;
class GLBaseVariator;
class GLBaseRanker;
class GLBaseSelectionist;

/**
 * Base class for keeping and maintaining the population of organisms.
 * In general, population is supposed to delete all the organisms in
 * destructor, so please, take care to save organisms you need somewhere else.
 */
class GLBasePopulation
{
public:
  /**
   * Constructor, creates the population of given size and initialising it,
   * if necessary.
   * @param storage main storage container for keeping the organisms.
   */
  GLBasePopulation(GLBaseOrganismStorage* storage);

  /**
   * Destroys everything worth destroying.
   */
  virtual ~GLBasePopulation();

  /**
   * Adds new organism to population. If the 'allow_duplicates' it false
   * and organism is already in the population -- do not add it.
   * @param organism new organism to be added to storage.
   * @param twins when NO_TWINS, check that this organism is not in
   *    the storage already.
   * @return true if organism was succesfully added and false otherwise.
   */
  bool addOrganism(GLBaseOrganism* organism,
                   GLConstants::TTwinsFlags twins =
                     GLConstants::NO_TWINS)
     { return m_main_storage->addOrganism(organism, twins); }

  /**
   * Create new organisms in population by applying crossover.
   * @param breeder class for producing the offsprings.
   * @param max_offsprings maximum number of children generated.
   *            if < 0 -- number is limited only by the crossover operator
   *            used.
   * @return number of newly created organisms.
   */
  virtual int breed(GLBaseBreeder *breeder,
                    int max_offsprings = -1) = 0;

  /**
   * Creates new generation from the existing pool of organism. The
   * general algorithm is following:
   *    1) Create offsprings by using the crossover operator.
   *    2) Create mutants by applying mutator.
   *    3) Evaluate all or new only organisms.
   *    4) Keep the fittest.
   * @param breeder class for producing the offsprings.
   * @param variator class to mutate the organisms in container.
   * @param evaluator class for evaluating the organisms.
   * @param selectionist class to select those to be included in the next
   *            generation.
   * @param best_organisms best organism(s) in newly created population.
   *            The previous content of this vector will be lost.
   * @param max_offsprings maximum number of offsprings generated.
   *            if < 0 -- number is limited only by the breeding algorithm
   *            used.
   * @param max_mutants maximum number of mutants generated.
   *            if < 0 -- number is limited only by the mutation algorithm
   *            used.
   * @param mutation_policy determines whether mutator to be applied to 
   *            children and / or to parents.
   * @param new_size the size of the new population. If < 0 its size will be
   *            equal to the size of the current population.
   * @return number of new organisms in the new generation.
   */
  virtual int createNewGeneration(GLBaseBreeder* breeder,
                                  GLBaseVariator* variator,
                                  GLBaseEvaluator* evaluator,
                                  GLBaseSelectionist *selectionist,
                                  vector<const GLBaseOrganism*>& 
                                                          best_organisms,
                                  int max_offsprings = -1,
                                  int max_mutants = -1,
                                  int mutation_policy = 
                                    GLConstants::MUTATE_CHILDREN |
                                    GLConstants::MUTATE_PARENTS,
                                  int new_size = -1);

  /**
   * Evaluates all the organisms in the population. It is up to
   * implementation to decide, whether to evaluate only new organisms or
   * all of them.
   * @param evaluator evaluator operator.
   * @return number of organism which were evaluated (for which evaluation
   *      operator was called).
   */
  virtual int evaluateAll(GLBaseEvaluator* evaluator) = 0;

  /**
   * Returns the current size of the population.
   * @return the current size of the population.
   */
  int getSize()
    { return m_main_storage->getSize(); }

  /**
   * Returns the vector of pointers to the organisms from main storage.
   * @param container vector of pointers to the organisms from storage
   *                    (output value).
   * @param fill when APPEND the new organisms are added to the container,
   *    when REPLACE -- the container is cleaned before adding the
   *    organisms.
   * @param size number of organisms from storage to be returned in vector.
   *            if < 0 or > getSize() then returns all the organisms.
   * @param copy_clones when true the pointers to the clones of the organisms
   *    will be returned, otherwise -- pointers to the original organisms.
   * @return number of elements transferred to container.
   *
   */
  virtual int
      getOrganismsVector(vector<const GLBaseOrganism*>& container,
                         GLConstants::TStorageFilling fill =
                           GLConstants::APPEND,
                         int size = -1,
                         bool copy_clones = false) const
    { return m_main_storage->
        getOrganismsVector(container, fill, size, copy_clones);
    }

  /**
   * Provides the vector of pointers to the organisms in storage starting
   * from the first one. The base version uses iterator to fill the
   * vector, however, for real implementation of storage which uses stl
   * containers more efficient approach should be possible.
   * @param container vector of pointers to the organisms from storage
   *                    (output value).
   * @param fill when APPEND the new organisms are added to the container,
   *    when REPLACE -- the container is cleaned before adding the
   *    organisms.
   * @param size number of organisms from storage to be returned in vector.
   *            if < 0 or > getSize() then returns all the organisms.
   * @param copy_clones when true the pointers to the clones of the organisms
   *    will be copied, otherwise -- pointers to the original organisms.
   * @return number of elements transferred to container.
   *
   */
  virtual int
      getOrganismsVector(vector<GLBaseOrganism*>& container,
                         GLConstants::TStorageFilling fill =
                           GLConstants::APPEND,
                         int size = -1,
                         bool copy_clones = false)
  { return m_main_storage->
      getOrganismsVector(container, fill, size, copy_clones);
  }

  /**
   * Checks, if organism is really the new one for this population.
   * @param organism organism to be checked for uniqueness.
   * @return true if no identical organisms was found everywhere within
   *    population.
   */
  virtual bool isOrganismTrulyNew(const GLBaseOrganism* organism) const = 0;

  /**
   * Forms the new generation out of existing organisms by using the
   * selectionist. The size of the new generation will be new_size or
   * the total number of organisms available, whatever is less.
   * This function normally to be called after methods 'breed', 'mutate'
   * and 'evaluateAll' were called.
   * @param selectionist class to select those to be included in the next
   *            generation.
   * @param best_organisms best organism(s) in new generation.
   *            The previous content of this vector will be lost.
   * @param new_size size of the new generation. If < 0 then the size of
   *    the new generation will be == the size of the current one.
   * @return number of NEW organisms (not present in current generation)
   *             in next generation.
   */
  virtual int keepFittest(GLBaseSelectionist *selectionist,
                          vector<const GLBaseOrganism*>& 
                                           best_organisms,
                          int new_size = -1) = 0;

  /**
   * Mutate the original population and the newborn (it is assumed, that
   * this method is called after Breed).
   * @param variator mutator operator to be applied to the organisms.
   * @param max_mutants maximum number of mutants generated.
   *            if < 0 -- number is limited only by the mutation algorithm
   *            used.
   * @param mutation_policy determines whether mutator to be applied to 
   *            children and / or to parents.
   * @return number of mutated organisms.
   */
  virtual int mutate(GLBaseVariator *variator,
                     int max_mutants = -1,
                     int mutation_policy = 
                       GLConstants::MUTATE_CHILDREN |
                       GLConstants::MUTATE_PARENTS) = 0;

  /**
   * Cleans the population and adds new organisms.
   * @param size number of new organisms to be set.
   * @param initialiser factory for organisms production.
   * @param twins when NO_TWINS no duplicates are possible
   *                    in the population.
   */
  virtual void populate(int size,
                        GLBaseOrganismInitialiser* initialiser,
                        GLConstants::TTwinsFlags twins =
                          GLConstants::NO_TWINS) = 0;

  /**
   * Last function to be called in create new generation. Deletes temporary
   * arrays etc.
   */
  virtual void prepareForNextGeneration() {}

  /**
   * Method to deal with discarded elements -- free memory, keep it in
   * 'morgue' to avoid considered organisms twice etc. After this
   * operation the vector of corpses should be empty.
   * @param corpses vector of organisms to be discarded.
   */
  virtual void processCorpses(vector<GLBaseOrganism*>& corpses) = 0;

/*------------------- DEBUG METHODS ------------------------*/
  /**
   * DEBUG METHOD. Evaluates the organisms in population and prints
   * them in ranked order. Note, that this debug method may change the
   * organisms' states by evaluating them.
   * @param out output stream.
   * @param evaluator evaluator for evaluating elements.
   * @param ranker operator to arrange the organisms according to their rank.
   */  virtual void printRankedPopulation(
                                     ostream& out,
                                     GLBaseEvaluator* evaluator,
                                     GLBaseRanker *ranker);
protected:
  ///Here core (active) population is stored. Only members from this
  ///container are considered to be members of the current population.
  GLBaseOrganismStorage* m_main_storage;
};

#endif /*GLBASEPOPULATION_H_*/
