#ifndef GLBASERANKER_H_
#define GLBASERANKER_H_
#include <vector>
using namespace std;
class GLBaseOrganism;

/**
 * Class for ranking the organisms according to their fitness or other
 * criteria. Normally it assumes, that all organisms are evaluated.
 */
class GLBaseRanker
{
public:
  /**
   * Simple constructor.
   */
  GLBaseRanker();

  /**
   * Destructor -- delete everything.
   */
  virtual ~GLBaseRanker();

  /**
   * Returns the fittest organism(s) from the vector. The criteria for
   * fittness should be the same which are used in 'rankOrganism'. May be
   * single organism for single objective optimisation or the whole Pareto
   * front for multi-objective optimisation. It is assumed,
   * that all organisms in vector are evaluated.
   * @param organisms vector of organisms.
   * @param fittest returned vector of fittest organisms. Previous content
   *    of the container will be erased.
   */
  virtual void getFittest(const vector<GLBaseOrganism*>& organisms,
                          vector<const GLBaseOrganism*>& fittest) = 0;

  /**
   * Returns the index of the first not-evaluated organisms in the array
   * or -1 if all are evaluated.
   * @param organisms vector of organisms to be checked.
   * @return true if all of the organisms are evaluated and false otherwise.
   */
  int notEvaluated(const vector<const GLBaseOrganism*>& organisms) const;

  /**
   * Takes the vector of pointers to the organisms and
   * re-arrange them according to certain criteria. For single
   * objective optimisation this is normally be by the order
   * of value of fitness function.
   * @param organisms vector of organisms to be ranked.
   * @param fittest the fittest organism(s) among just ranked. It could be
   *    single organism for single objective optimisation or the whole
   *    Pareto front in case of multi-objective optimisation. The previous
   *    content of this vector will be lost.
   */
  virtual void rankOrganisms(
        vector<GLBaseOrganism*>& organisms,
        vector<const GLBaseOrganism*>& fittest) = 0;

  /**
   * Takes the vector of pointers to the organisms and
   * re-arrange them according to certain criteria. For single
   * objective optimisation this is normally be by the order
   * of value of fitness function.
   * @param organisms vector of organisms to be ranked.
   */
  virtual void rankOrganisms(
        vector<GLBaseOrganism*>& organisms) = 0;

  /**
   * Resets the ranker (e.g. sets counter, if any, to zero etc.).
   */
  virtual void reset();
  
  /**
   * Compares currently best organisms with candidates and, if necessary, 
   * amends the current best organisms. Normally to be used for comparing 
   * all-time best organisms with the best organisms in current generation.
   * Note, that "current best" organisms are pointers to independent "clones"
   * while candidates point to current population.
   * For Single Objective ranker all the organisms in each array should have
   * the same fittness, for Multi Objective ranker it may be a matter of 
   * merging 2 Pareto fronts into one. 
   * @param candidates the vector of candidates.
   * @param current_best currently most fittest organisms.
   * @return true if vector of best organisms was amended and false otherwise.
  */
  virtual bool updateFittest(
            const vector<const GLBaseOrganism*>& candidates,
                  vector<const GLBaseOrganism*>& current_best) = 0;
};

#endif /*GLBASERANKER_H_*/
