#include "GLBaseOrganism.h"
#include "GLBaseRanker.h"
#include "GLBaseSelectionist.h"
#include "GLLogicError.h"
#include "GLRuntimeError.h"
  /**
   * Constructor. Assigns the operator which will be used for ranking
   * the organisms. Note, that the ranker will be destroyed when destructor
   * is called.
   * @param ranker operator for arranging the organisms in accordance
   * 	with their rank.
   */
GLBaseSelectionist::GLBaseSelectionist(GLBaseRanker* ranker)
{
  if (ranker == NULL)
  {
    char errormsg[] = "\nGLBaseSelectionist::GLBaseSelectionist\n"
                      "Null argument is passed as a parameter 'ranker'\n";
    throw GLRuntimeError(errormsg);
  }
  m_ranker = ranker;
}//constructor

/**
 * Destructor. Destroys everything, including the ranker.
 */
GLBaseSelectionist::~GLBaseSelectionist()
{
	if (m_ranker != NULL) delete m_ranker;
}//destructor

/**
 * Asserts the validity of the size of the new generation -- should be
 * > 0 and <= the total size of current and new generations. Probably not
 * for all selectionists this method will be necessary. If the new
 * size is not valid - throws an exception GLLogicError. 
 * @param new_size number of organisms in next generations.
 * @param current_population vector of organisms which form the 
 *    current population.
 * @param candidates vector of candidates on adding to new generation.
 */
void GLBaseSelectionist::assertNewSize(int new_size, 
                   const vector<GLBaseOrganism*>& current_population,
                   const vector<GLBaseOrganism*>& candidates)
  {
    if ((new_size < 0) ||
        ((size_t)new_size > current_population.size() + candidates.size()))
      {
        char errormsg[1000];
        sprintf(errormsg, "\nGLBaseSelectionist::assertNewSize\n"
                          "Passed parameter new_size (%i) is either < 0 or "
                          "exceeds the sum of elements in current generation"
                          " and candidates (%i and %i accordingly)\n",
                          (int)new_size, (int)current_population.size(),
                          (int)candidates.size());
        throw GLLogicError(errormsg);
      }
  }//assertNewSize

/**
 * Counts the number of survived candidates (those, which get to the new
 * generation). Note, that this is general and, therefore, not very 
 * efficient way of obtaining this number. It may well be possible, that
 * some selection algorithm will be able to get this number without calling
 * this method. Note, that following is assumed: 
 *    to_live + to_die == current_population + candidates
 * where '+' means 'union'
 * @param to_live vector of organisms which will form the next generation.
 * @param to_die vector of organisms which will be disposed of.
 * @param current_population vector of organisms which form the current
 *    generation.
 * @param candidates vector of 'candidate' organisms.
 */
int GLBaseSelectionist::countSurvivedCandidates(
            const vector<GLBaseOrganism*>& to_live,
            const vector<GLBaseOrganism*>& to_die,
            const vector<GLBaseOrganism*>& current_population,
            const vector<GLBaseOrganism*>& candidates)
   {
     //mark those, which are to live and to die
     for(size_t i = 0; i < to_live.size(); i++)
       {
         to_live[i]->setTag(1);
       }
     for(size_t i = 0; i < to_die.size(); i++)
       {
         to_die[i]->setTag(0);
       }
     //and then count how many from the new generation left to live
     int counter(0);
     for(size_t i = 0; i < candidates.size(); i++)
       {
         counter += candidates[i]->getTag();
       }
     return counter;
   }//countSurvivedCandidates

/**
 * Resets the selectionist (e.g. sets counter, if any, to zero etc.) and
 * its ranker.
 */
void GLBaseSelectionist::reset()
  {
    getRanker()->reset();
  }//reset
