#ifndef GLBASESELECTIONIST_H_
#define GLBASESELECTIONIST_H_
#include <vector>

using namespace std;
class GLBaseOrganism;
class GLBaseRanker;
/**
 * Class to choose from the vector of organism which one to carry forward
 * to the next population and which one to reject.
 */
class GLBaseSelectionist
{
public:
  /**
   * Constructor. Assigns the operator which will be used for ranking
   * the organisms. Note, that the ranker will be destroyed when destructor
   * is called.
   * @param ranker operator for arranging the organisms in accordance
   * 	with their rank.
   */
  GLBaseSelectionist(GLBaseRanker* ranker);

  /**
   * Destructor. Destroys everything, including the ranker.
   */
  virtual ~GLBaseSelectionist();

  /**
   * Choose which organism to leave and which one to die in the next
   * generation. It is assumed, that all organisms passed as input parameters
   * were evaluated already.
   * @param current_population organisms from the this generation's
   *            population.
   * @param candidates newly created organisms (e.g. mutants and/or
   *            children). It is desirable, that both arrays have not
   *            common organisms.
   * @param new_size size of the next generation.
   * @param to_live output array of organisms to live  (will be cleaned).
   * @param to_die output array of organisms to die in the next generation
   *     (will be cleaned).
   * @param best_survived best survived organism(s). May be single organism
   *     for single objective optimisation or the whole Pareto-front for
   *     multi-objective optimisation. The previous content of this
   *     array will be lost.
   * @return number of NEW organisms (not present in current generation)
   *             in next generation.
   */
  virtual int doSelection(
                      const vector<GLBaseOrganism*>& current_population,
                      const vector<GLBaseOrganism*>& candidates,
                      size_t new_size,
                      vector<GLBaseOrganism*>& to_live,
                      vector<GLBaseOrganism*>& to_die,
                      vector<const GLBaseOrganism*>& best_survived) = 0;

  /**
   * Provides access to the ranker used by selector.
   * @return pointer to ranker.
   */
  GLBaseRanker* getRanker()
  { return m_ranker;}

  /**
   * Provides access to the ranker used by selector.
   * @return pointer to ranker.
   */
  const GLBaseRanker* getRanker() const
  { return m_ranker;}

  /**
   * Resets the selectionist (e.g. sets counter, if any, to zero etc.) and
   * its ranker.
   */
  virtual void reset();
  
protected:
  /**
   * Asserts the validity of the size of the new generation -- should be
   * > 0 and <= the total size of current and new generations. Probably not
   * for all selectionists this method will be necessary. If the new
   * size is not valid - throws an exception GLLogicError.
   * @param new_size number of organisms in next generations.
   * @param current_population vector of organisms which form the
   *    current population.
   * @param candidates vector of candidates on adding to new generation.
   */
  virtual void assertNewSize(int new_size,
                     const vector<GLBaseOrganism*>& current_population,
                     const vector<GLBaseOrganism*>& candidates);

  /**
   * Counts the number of survived candidates (those, which get to the new
   * generation). Note, that this is general and, therefore, not very
   * efficient way of obtaining this number. It may well be possible, that
   * some selection algorithm will be able to get this number without calling
   * this method. Note, that following is assumed:
   *    to_live + to_die == current_population + candidates
   * where '+' means 'union'
   * @param to_live vector of organisms which will form the next generation.
   * @param to_die vector of organisms which will be disposed of.
   * @param current_population vector of organisms which form the current
   *    generation.
   * @param candidates vector of 'candidate' organisms.
   */
   virtual int countSurvivedCandidates(
              const vector<GLBaseOrganism*>& to_live,
              const vector<GLBaseOrganism*>& to_die,
              const vector<GLBaseOrganism*>& current_population,
              const vector<GLBaseOrganism*>& candidates);
private:
	//Used for ranking the organisms.
	GLBaseRanker* m_ranker;
};

#endif /*GLBASESELECTIONIST_H_*/
