#include <iostream>
#include <vector>

#include <NLstl_utils.h>

#include "GLBaseCrossover.h"
#include "GLBaseEvaluator.h"
#include "GLBaseGeneticAlgorithm.h"
#include "GLBaseOrganism.h"
#include "GLBaseOrganismStorage.h"
#include "GLBasePopulation.h"
#include "GLBaseRanker.h"
#include "GLBreederStandard.h"
#include "GLLogicError.h"
#include "GLRandomNumbersGenerator.h"
//#include "GLUtils.h"

/**
 * Constructor. Assigns the crossover which will be used for
 * breeding.
 * @param crossover crossover which will be used for breeding.
 * @param parameters structure which contains the breeding probability
 *    parameter (probability of applying the crossover
 *    operator to any considered pair, must be >= 0 and <= 1).
 * @param parent_ga genetic algorithm which owns the exemplar
 *    of this class. Will provide the means to use the evaluator and
 *    ranker from the GA.
 */
GLBreederStandard::GLBreederStandard(GLBaseCrossover *crossover,
                    GLBreederStandard::TParameters* parameters,
                    GLBaseGeneticAlgorithm *parent_ga):
                      GLBaseBreeder(crossover)
{
  //check, that the breeding probability is within the limits.
  if ((parameters->t_breeding_probability < 0) ||
      (parameters->t_breeding_probability > 1))
    {
      char errormsg[1000];
      sprintf(errormsg, "\nGLBreederStandard::GLBreederStandard\n"
                        "Parameter breeding probability (= %e) should "
                        "be in interval [0; 1]",
                        parameters->t_breeding_probability);
      throw GLLogicError(errormsg);
    }

  m_breeding_probability = parameters->t_breeding_probability;
  nlutils::assertPointer(parent_ga,
                "GLBreederStandard::GLBreederStandard, parent_ga");
  m_parent_ga = parent_ga;
}//constructor

GLBreederStandard::~GLBreederStandard()
{
}//destructor


/**
 * Breeds organisms -- produces the offsprings from parents based
 * on certain criteria using the crossover owned by this breeder.
 * @param population population to which parents belong -- mainly to
 *    be able to check the uniqueness of the new offsprings.
 * @param potential_parents organisms to be bred.
 * @param offsprings storage place to store the children.
 * @param fill when APPEND adds the organisms to the storage, when REPLACE
 *    the storage is emptied at the beginning.
 * @param twins when NO_TWINS all the offsprings will be checked for
 *    uniqueness with the parent's population before being added to the
 *    storage of offsprings.
 * @param size_limits limit number of produced offsprings. When < 0
 *    all the offsprings generated by the algorithm will be returned.
 * @return number of offsprings produced.
 */
int GLBreederStandard::breedOrganisms(GLBasePopulation* population,
                           GLBaseOrganismStorage* potential_parents,
                           GLBaseOrganismStorage* offsprings,
                           GLConstants::TStorageFilling fill,
                           GLConstants::TTwinsFlags twins,
                           int size_limits)
{
  //clean the storage of offsprings if necessary.
  if (fill == GLConstants::REPLACE)
    {
      offsprings->emptyStorage();
    }
  //get the vector of organisms from the storage of parents.
  vector<GLBaseOrganism*> parents;
  potential_parents->getOrganismsVector(parents);
  //evaluate those, which were not evaluated before
  int parents_size = parents.size();
  for(int i = 0; i < parents_size; i++)
    {
      if (!parents[i]->isEvaluated())
        {
          parents[i]->evaluate(m_parent_ga->getEvaluator());
        }
    }
  //rank them
  m_parent_ga->getRanker()->rankOrganisms(parents);
  //breed them
  int initial_size = offsprings->getSize();
  vector<GLBaseOrganism*> children; //to store children
  //loop through all parents
  for(int i = 0; i < parents_size; i++)
    {
      vector<GLBaseOrganism*> couple(2);
      couple[0] = parents[i];
      for(int j = i + 1; j < parents_size; j++)
        {
          //randomly decide if we need to breed this pair.
          if (GLRandomNumbersGenerator::getRandomUniform(0, 1)
              < m_breeding_probability)
            {
              couple[1] = parents[j];
              getCrossover()->makeOffsprings(couple, children);
              //loop through all children
              for(size_t k = 0; k < children.size(); k++)
                {
                  //check, that it is unique, if necessary
                  if ((twins == GLConstants::NO_TWINS) &&
                      !population->isOrganismTrulyNew(children[k]))
                    {
                      delete children[k];
                      continue;
                    }
                  //add it to the offsprings storage
                  offsprings->addOrganism(children[k]);
                  //check if the number of added organisms reached limits
                  if ((size_limits > 0) &&
                     (offsprings->getSize() - initial_size) == size_limits)
                    {
                      //delete the rest of the children
                      for_each(children.begin() + k + 1, children.end(),
                               nlutils::DeleteObjectSafely());
                      return size_limits;
                    }
                }
            }
        }
    }
  return offsprings->getSize() - initial_size;
}//breedOrganisms
