#ifndef GLBREEDERSTANDARD_H_
#define GLBREEDERSTANDARD_H_

#include "GLBaseBreeder.h"
#include "GLBaseParameters.h"
#include "GLLogicError.h"

class GLBaseEvaluator;
class GLBaseGeneticAlgorithm;
class GLBaseRanker;
/**
 * Standard breed algorithm -- ranks all the organisms according to
 * value of the fitness function. Then, starting from the best
 * organism, applies crossover with given probability to all of the
 * organisms ranked below (considers each possible pair and then randomly
 * decides when to apply crossover), then do the same starting with the
 * second best organism etc. until the required number of organisms was
 * generated. In this way the higher organism is ranked the more chances it
 * will have to produce offsprings.
 * This class cannot exist without parent GA, as it uses evaluator and
 * ranker from there,
 */
class GLBreederStandard: public GLBaseBreeder
{
public:
  class TParameters;
  /**
   * Constructor. Assigns the crossover which will be used for
   * breeding.
   * @param crossover crossover which will be used for breeding.
   * @param parameters structure which contains the breeding probability
   *    parameter (probability of applying the crossover
   *    operator to any considered pair, must be >= 0 and <= 1).
   * @param parent_ga genetic algorithm which owns the exemplar
   *    of this class. Will provide the means to use the evaluator and
   *    ranker from the GA.
   */
  GLBreederStandard(GLBaseCrossover *crossover,
                    GLBreederStandard::TParameters* parameters,
                    GLBaseGeneticAlgorithm *parent_ga);
  /**
   * Destructor, destroys everything, including crossover.
   */
  virtual ~GLBreederStandard();

  /**
   * Breeds organisms -- produces the offsprings from parents based
   * on certain criteria using the crossover owned by this breeder.
   * @param population population to which parents belong -- mainly to
   *    be able to check the uniqueness of the new offsprings.
   * @param potential_parents organisms to be bred.
   * @param offsprings storage place to store the children.
   * @param fill when APPEND adds the organisms to the storage, when REPLACE
   *    the storage is emptied at the beginning.
   * @param twins when NO_TWINS all the offsprings will be checked for
   *    uniqueness with the parent's population before being added to the
   *    storage of offsprings.
   * @param size_limits limit number of produced offsprings. When < 0
   *    all the offsprings generated by the algorithm will be returned.
   * @return number of offsprings produced.
   */
  virtual int breedOrganisms(GLBasePopulation* population,
                             GLBaseOrganismStorage* potential_parents,
                             GLBaseOrganismStorage* offsprings,
                             GLConstants::TStorageFilling fill =
                               GLConstants::APPEND,
                             GLConstants::TTwinsFlags twins =
                               GLConstants::NO_TWINS,
                             int size_limits = -1);
public:
  /**
   * Internal class of the additional (to base breeder's) parameters
   * necessary to initialise this class.
   */
  struct TParameters:public GLBaseParameters
  {
   /**
    * Creates new exemplar of the GLBreederStandard::TParameters class.
    * @param breeding_probability probability of applying the crossover
    *    operator to any considered pair. Must be >= 0 and <= 1.
    */
    TParameters(double breeding_probability)
    {
      //check, that the breeding probability is within the limits.
      if ((breeding_probability < 0) || (breeding_probability > 1))
        {
          char errormsg[1000];
          sprintf(errormsg, "\nGLBreederStandard::TParameters::TParameters\n"
                            "Parameter breeding_probability (= %e) should "
                            "be in interval [0; 1]",
                            breeding_probability);
          throw GLLogicError(errormsg);
        }
      t_breeding_probability = breeding_probability;
    }//constructor

    ///copying constructor
    ///@param source source of the data to copy;
    TParameters(const TParameters *source)
    {
      t_breeding_probability = source->t_breeding_probability;
    }//constructor

    /**
     * Creates the copy of the parameters.
     * @return pointer to identical copy of the current class exemplar.
     */
    virtual GLBaseParameters* copy() const
    { return new TParameters(this); }

    ///Breeding probability
    double t_breeding_probability;
  };
private:
  //probability of applying crossover to any pair
  //organisms which will be considered.
  double m_breeding_probability;
  GLBaseGeneticAlgorithm *m_parent_ga;
};

#endif /* GLBREEDERSTANDARD_H_ */
