#ifndef GLCONSTANTS_H_
#define GLCONSTANTS_H_
#include <string>
using namespace std;

/**
 * Class for defining various useful constants for the library.
 */
class GLConstants
{
public:
  /** Constants for limits on genes (elements of the organism) */
  enum TGeneBounds
  {
    BOUNDED_GENES = 0,   ///Genes can take values between min and max values.
    UNBOUNDED_GENES = 1  ///Genes can take any values.
  };

  /**
   * Constants for policy of freeing the memory allocated to the
   * organisms within some storage.
   */
  enum TMemoryPolicy
  {
    DO_NOT_FREE = 0,    //do not free memory at all
    FREE_ALL    = 1,    //free all the pointers in the storage.
    FREE_UNLOCKED = 2   //free only those, which are not locked in memory.
  };

  /**
   * Constants on policy to which organism mutation should be applied.
   */
  enum TMutationPolicy
  {
    MUTATE_PARENTS = 0x01,            //only parents will be mutated
    MUTATE_CHILDREN = 0x02           //only children will be mutated
  };
  
  /** Constants for the size property.*/
  enum TSizeLimits
  {
    FIXED = 0,          ///size of the object is fixed.
    NOT_FIXED = 1,      ///size of the object is varied (limited either by
                        ///certain given size or by max integer).
  };

  /**
   * Constants to indicate whether storage should be cleaned and filled
   * with the new data or new data should be added to storage.
   */
  enum TStorageFilling
  {
    REPLACE = 0,
    APPEND = 1
  };

  /**
   * Constants to be used with organism's tags
   */
//  enum TTagConstants
//  {
//    KEEP = 1,
//    REMOVE = 2
//  };

  /**
   * Constants for specifying if twins (duplicated organisms) are allowed
   * or not.
   */
  enum TTwinsFlags
  {
    TWINS_ALLOWED = 0,
    NO_TWINS = 1
  };

  /**
   * Flag to indicate that corpses are to be stored.
   */
  static const bool CORPSES_KEEP = true;
  /**
   * Flag to indicate, that corpses are to be disposed of.
   */
  static const bool CORPSES_DISPOSE = false;

  ///flag to indicate that 'AND' condition to be used for list of boolean
  ///operations.
  static const bool AND_OPERATOR = true;
  
  ///flag to indicate that 'OR' condition to be used for list of boolean
  ///operations.
  static const bool OR_OPERATOR  = false;
  
  /*---------------------------------------------------------------------*/
  /*|                CONSTANTS FOR GENETIC ALGORITHM STATUS             |*/
  /*---------------------------------------------------------------------*/
  ///Possible states for the genetic algorithm
  ///GA is initialised and idle
  static const int GA_STATE_INITIALISED = 101;
  ///GA is running
  static const int GA_STATE_RUNNING = 105;
  ///GA  stopped with error
  static const int GA_STATE_STOPPED_ERROR = 110;
  ///GA stopped normally
  static const int GA_STATE_STOPPED_NORMALLY = 115;

  /*---------------------------------------------------------------------*/
  /*|              CONSTANTS FOR GENETIC ALGORITHM PARAMETERS           |*/
  /*---------------------------------------------------------------------*/
  /**
   * Constants for indicating the mutator types.
   */
  ///Mutator which swaps 2 genes
  static const int MUTATOR_GENES_SWAPPING = 1001;
  ///Mutator which shuffles several (randomly chosen) genes.
  static const int MUTATOR_GENES_SHUFFLING = 1002;
  ///Mutator which moves gene into new position
  static const int MUTATOR_GENE_MOVING = 1003;
  ///Mutator which applies several mutators simultaniously to an organism or
  ///chooses randromly one from the set.
  static const int MUTATOR_COMBINED = 1004;
  
  /**
   *  Constants for indicating the crossover types.
   */
  ///Crossover which takes 2 vector-based organisms keeps coinciding genes
  ///in place and randomly shuffles the others.
  static const int CROSSOVER_VECTOR_KEEP_MATCHING_INT = 2001;
  ///One point crossover which exchanges tails of 2 parents.
  static const int CROSSOVER_ONE_POINT = 2002;
  ///Two points crossover which exchanges section of 2 parents between
  ///two points.
  static const int CROSSOVER_TWO_POINTS = 2003;
  
  /**
   * Constants for indicating the ranker type.
   */
  ///Standard ranker for single objective GA.
  static const int RANKER_STANDARD_SO = 3001;

  /**
   * Constantns for variator type.
   */
  ///Standard variator which uniformly applies mutator to all organisms in
  ///storage.
  static const int VARIATOR_STANDARD = 4001;

  /**
   * Constants for breeder type.
   */
  ///Standard breeder which applies crossover to all organisms in storage
  ///with better organisms having more chances to breed.
  static const int BREEDER_STANDARD = 5001;

  /**
   * Constants for selectionist type.
   */
  ///Elitist selectionist which mixes old and new generations and keeps
  ///the most fit organisms only.
  static const int SELECTIONIST_ELITIST = 6001;
  ///Selectionist which maintains variety by keeping only best elements
  ///of the old population and always adding elements from the new one.
  static const int SELECTIONIST_MIXER = 6002;

  /**
   * Constants for population type
   */
  ///standard population.
  static const int POPULATION_STANDARD = 7001;

  /**
   * Constants for organism initialisers.
   */
  ///Generates vectors of randomly shuffled integers.
  static const int INITIALISER_SHUFFLE_INT_VECTOR = 8001;

  /**
   * Returns string representing the value of bounds.
   * Useful for debug.
   * @param bounds constant representing bounds of the gene.
   * @return string corresponding to name of the constant.
   */
  static string toString (TGeneBounds bounds);

  /**
   * Return string representing the memory policy. Using for debug output.
   * @param memory constant for memory policy.
   * @return string corresponding to name of constant.
   */
  static string toString (TMemoryPolicy memory);

  /**
   * Returns string representing the value of limits constant.
   * Useful for debug.
   * @param limits constant representing limits type of the object.
   * @return string corresponding to name of the constant.
   */
  static string toString (TSizeLimits limits);

  /**
   * Returns string representing the value of the tag constant.
   * Useful for debug.
   * @param tag constant representing tag.
   * @return string corresponding to name of the constant.
   */
  //static string toString (TTagConstants tag);

  /**
   * Returns string representing the value of the breeder type constant.
   * Useful for debug.
   * @param breeder_type constant for breeder type.
   * @return string which is the name for the constant.
   */
  static string toStringBreeder(int breeder_type);

  /**
   * Returns string representing the value of the crossover type constant.
   * Useful for debug.
   * @param crossover_type constant for crossover type.
   * @return string which is the name for the constant.
   */
  static string toStringCrossover(int crossover_type);

  /**
   * Returns string representing the value of the current GA state.
   * Useful for debug.
   * @param ga_state constant for GA state type.
   * @return string which is the name for the constant. 
   */
  static string toStringGaState(int ga_state);
  
  /**
   * Returns string representing the value of the organism initialiser type
   * constant.
   * Useful for debug.
   * @param initialiser_type constant for organism initialiser type.
   * @return string which is the name for the constant.
   */
  static string toStringInitialiser(int initialiser_type);

  /**
   * Returns string representing the value of the mutator type constant.
   * Useful for debug.
   * @param mutator_type constant for mutator type.
   * @return string which is the name for the constant.
   */
  static string toStringMutator(int mutator_type);

  /**
   * Returns string representing the value of the population type constant.
   * Useful for debug.
   * @param population_type constant for population type.
   * @return string which is the name for the constant.
   */
  static string toStringPopulation(int population_type);

  /**
   * Returns string representing the value of the ranker type constant.
   * Useful for debug.
   * @param ranker_type constant for ranker type.
   * @return string which is the name for the constant.
   */
  static string toStringRanker(int ranker_type);

  /**
   * Returns string representing the value of the selectionist type constant.
   * Useful for debug.
   * @param selectionist_type constant for selectionist type.
   * @return string which is the name for the constant.
   */
  static string toStringSelectionist(int selectionist_type);

  /**
   * Returns string representing the value of the variator type constant.
   * Useful for debug.
   * @param variator_type constant for variator type.
   * @return string which is the name for the constant.
   */
  static string toStringVariator(int variator_type);

}; //GLConstants
#endif /*GLCONSTANTS_H_*/
