#include <algorithm>
#include <iostream>
#include "GLCrossoverVectorKeepMatching.h"
#include "GLLogicError.h"
#include "GLOrganismVector.h"
#include "GLRuntimeError.h"
#include "gl_utils.h"

using namespace std;

/**
 * Creates an offspring from the vector of parents. Takes 2 vector-based
 * organisms and creates children by keeping matching genes in their place
 * and randomly shuffling others.
 * @param parents vector of parents.
 * @param children vector of new organisms obtained from parents. Previous
 *    content will not be preserved.
 */
template<class T>
void GLCrossoverVectorKeepMatching<T>::makeOffsprings(
			const vector<GLBaseOrganism*>& parents,
			      vector<GLBaseOrganism*>& children)
{
  //Check, that 2 parents are passed.
  if (parents.size() != 2)
  {
	char errormsg[1000];
	sprintf(errormsg, 
	    "\nGLCrossoverVectorKeepMatching<T>::makeOffspring\n"
	    "Can accept 2 parents only, while vector of %i parents "
	    "was passed\n",
	    (int)parents.size());
	throw GLLogicError(errormsg);
  }
  //Cast the parents to the classes which can be processed by this
  //crossover
  const GLOrganismVector<T> *parent_1 =
              dynamic_cast<const GLOrganismVector<T>*>(parents[0]);
  const GLOrganismVector<T> *parent_2 =
	      dynamic_cast<const GLOrganismVector<T>*>(parents[1]);
  //Check, that parents were casted properly
  if ((parent_1 == NULL) || (parent_2 == NULL))
  {
    char errormsg[1000];
    sprintf(errormsg, "\nGLCrossoverVectorKeepMatching<T>::makeOffspring\n"
                      "One or both of parents are either of the wrong type\n"
                      "(should be pointer to GLVectorOrganism<T>) or NULL\n");
	throw GLRuntimeError(errormsg);
  }
  //Check, that both parents are of the same size.
  if (parent_1->getSize() != parent_2->getSize())
  {
    char errormsg[100];
    sprintf(errormsg, "\nGLCrossoverVectorKeepMatching<T>::makeOffspring\n"
                      "Both parents should have the same number of genes.\n"
                      "parent_1.getSize()= %i, parent_2.getSize() = %i\n",
                      parent_1->getSize(), parent_2->getSize());
    throw GLLogicError(errormsg);
  }

  //empty the vector with children
  children.clear();
  int size = parent_1->getSize();
  //chromosome with genes which keep the same place in both parents
  vector<T> new_chromosome(size);
  //vector of position of the genes which are different.
  vector<int> shuffled_positions;
  //vector of genes to be shuffled.
  vector<T> genes_to_shuffle;
  shuffled_positions.reserve(size);
  genes_to_shuffle.reserve(size);
  for(int i = 0; i < size; i++)
  {
    if ((*parent_1)[i] == (*parent_2)[i])
      {
	new_chromosome[i] = (*parent_1)[i];
      }
    else
      {
	shuffled_positions.push_back(i);
	genes_to_shuffle.push_back((*parent_1)[i]);
      }
  }
  //shuffle elements of array genes_to_shuffle
  gl_shuffle_vector(genes_to_shuffle);
  //fill shuffled position with the shuffled genes.
  for(size_t i = 0; i < shuffled_positions.size(); i++)
  {
    new_chromosome[shuffled_positions[i]] = genes_to_shuffle[i];
  }
  GLOrganismVector<T>* offspring = new GLOrganismVector<T>(new_chromosome);
  children.push_back(offspring);
}//makeOffspring
