#include <NLstl_utils.h>

#include "GLBaseParameters.h"
#include "GLBreederStandard.h"
#include "GLConstants.h"
#include "GLCrossoverOnePoint.h"
#include "GLCrossoverTwoPoints.h"
#include "GLCrossoverVectorKeepMatching.h"
#include "GLFactory.h"
#include "GLMutatorCombined.h"
#include "GLMutatorGeneMove.h"
#include "GLMutatorGenesSwapping.h"
#include "GLMutatorShuffling.h"
#include "GLPopulationStandard.h"
#include "GLSelectionistMixer.h"
#include "GLSelectionistStandard.h"
#include "GLRankerStandardSO.h"
#include "GLVariatorStandard.h"

#include "gl_utils.h"


  /**
   * Creates an instance of the Breeder class.
   * @param breeder_type type of the breeder to use.
   * @param crossover instance of the concrete crossover class to be passed
   *            to breeder.
   * @param breeder_parameters additional breeder parameters if necessary.
   * @param ga genetic algorithm -- owner of this breeder (and probably
   *    of the crossover as well).
   * @return pointer to the instance of Breeder class.
   */
GLBaseBreeder* GLFactory::createBreeder(
                  int breeder_type,
                  GLBaseCrossover* crossover,
                  GLBaseParameters* breeder_parameters,
                  GLBaseGeneticAlgorithm* ga)
  {
    switch(breeder_type)
    {
      case (GLConstants::BREEDER_STANDARD):
          {
            GLBreederStandard::TParameters *params =
              nlutils::transform_pointer<GLBreederStandard::TParameters,
                  GLBaseParameters>(breeder_parameters);
            return new GLBreederStandard(crossover,
                params, ga);
          }
    }
    return NULL;
  }//createBreeder

/**
 * Creates an exemplar of the crossover class.
 * @param crossover_type of the crossover to be created.
 * @param crossover_parameters additional crossover parameters.
 * @param ga genetic algorithm -- owner of this crossover.
 */
GLBaseCrossover* GLFactory::createCrossover(
       int crossover_type,
       GLBaseParameters* crossover_parameters,
       GLBaseGeneticAlgorithm* ga)
  {
    switch (crossover_type)
    {
      case (GLConstants::CROSSOVER_VECTOR_KEEP_MATCHING_INT):
        return new GLCrossoverVectorKeepMatching<int>();
      case (GLConstants::CROSSOVER_ONE_POINT):
        return new GLCrossoverOnePoint();
      case (GLConstants::CROSSOVER_TWO_POINTS):
        return new GLCrossoverTwoPoints();
    }
    return NULL;
  }//createCrossover


/**
 * Creates an exemplar of the mutator class.
 * @param mutator_type type of the mutator to be created.
 * @param mutation_rate rate of the mutation, interpretation is upto
 *    concrete implementation of the mutator. Normally one would expect
 *    to keep it in (0; 1) interval.
 * @param mutator_parameters additional mutator parameters.
 * @param ga owner of this mutator.
 */
GLBaseMutator* GLFactory::createMutator(int mutator_type,
      double mutation_rate,
      GLBaseParameters* mutator_parameters,
      GLBaseGeneticAlgorithm* ga)
  {
    switch (mutator_type)
    {
      case (GLConstants::MUTATOR_GENES_SWAPPING):
          {
            return new GLMutatorGenesSwapping(mutation_rate);
          }
      case (GLConstants::MUTATOR_GENES_SHUFFLING):
          {
            GLMutatorShuffling::TParameters *params =
              nlutils::transform_pointer<GLMutatorShuffling::TParameters,
                GLBaseParameters>(mutator_parameters,
                  "GLFactory::createMutator\n"
                  "Mutator parameters passed is NULL or not compatible with "
                  "GLMutatorShuffling::TParameters\n");
            return new GLMutatorShuffling(mutation_rate, params);
          }
      case (GLConstants::MUTATOR_GENE_MOVING):
          {
            return new GLMutatorGeneMove(mutation_rate);
          }
      case (GLConstants::MUTATOR_COMBINED):
          {
            GLMutatorCombined::TParameters *params = 
              nlutils::transform_pointer<GLMutatorCombined::TParameters,
                GLBaseParameters>(mutator_parameters,
                  "GLFactory::createMutator\n"
                  "Mutator parameters passed is NULL or not compatible with "
                  "GLMutatorCombined::TParameters\n");
            return new GLMutatorCombined(mutation_rate, params);
          }
    }
    return NULL;
  }//createMutator

/**
 * Creates an exemplar of the ranker class.
 * @param ranker_type type of the ranker to be created.
 * @param ranker_parameters additional ranker parameters.
 * @param ga owner of this ranker.
 */
GLBaseRanker* GLFactory::createRanker(int ranker_type,
    GLBaseParameters* ranker_parameters,
    GLBaseGeneticAlgorithm* ga)
  {
    switch (ranker_type)
    {
      case (GLConstants::RANKER_STANDARD_SO):
        return new GLRankerStandardSO();
    }
    return NULL;
  }//createRanker

/**
 * Creates an exemplar of the selectionist class.
 * @param selectionist_type type of the selectionist to be created.
 * @param ranker operator for arranging the organisms in accordance
 *    with their rank.
 * @param selectionist_parameters additional selectionist parameters.
 * @param ga owner of this selectionist.
 */
GLBaseSelectionist* GLFactory::createSelectionist(int selectionist_type,
     GLBaseRanker* ranker,
     GLBaseParameters* selectionist_parameters,
     GLBaseGeneticAlgorithm* ga)
   {
     switch (selectionist_type)
     {
       case (GLConstants::SELECTIONIST_ELITIST):
           {
             return new GLSelectionistStandard(ranker);
           }
       case (GLConstants::SELECTIONIST_MIXER):
           {
             GLSelectionistMixer::TParameters *params =
               nlutils::transform_pointer<GLSelectionistMixer::TParameters,
                 GLBaseParameters>(selectionist_parameters,
                   "GLFactory::createSelectionist\n"
                   "Selectionist parameters passed is NULL or not "
                   "compatible with GLSelectionist::TParameters\n");
             return new GLSelectionistMixer(ranker, params);
           }
     }
     return NULL;
   }//createSelectionist

/**
 * Creates an exemplar of the variator class.
 * @param mutator mutator operator used by this variator.
 * @param variator_type type of the variator to be created.
 * @param variator_parameters additional variator parameters.
 * @param ga owner of this variator.
 */
GLBaseVariator* GLFactory::createVariator(int variator_type,
    GLBaseMutator* mutator,
    GLBaseParameters* variator_parameters,
    GLBaseGeneticAlgorithm* ga)
  {
    switch(variator_type)
    {
      case(GLConstants::VARIATOR_STANDARD):
        return new GLVariatorStandard(mutator);
    }
    return NULL;
  }//createVariator
