#ifndef GLFACTORY_H_
#define GLFACTORY_H_

class GLBaseBreeder;
class GLBaseCrossover;
class GLBaseGeneticAlgorithm;
class GLBaseMutator;
class GLBaseParameters;
class GLBaseRanker;
class GLBaseSelectionist;
class GLBaseVariator;

/**
 * Factory for producing all the classes necessary for Genetic Algorithm.
 * Most of the methods take obligatory parameters necessary to create Base
 * classes (e.g. GLBaseMutator). Any additional parameters will be taken
 * from the exemplar of the GA.
 */
class GLFactory
{
public:
  /**
   * Simple constructor.
   */
  GLFactory() {};
  /**
   * Simple destructor
   */
  virtual ~GLFactory(){};

  /**
   * Creates an instance of the Breeder class.
   * @param breeder_type type of the breeder to use.
   * @param crossover instance of the concrete crossover class to be passed
   *            to breeder.
   * @param breeder_parameters additional breeder parameters if necessary.
   * @param ga genetic algorithm -- owner of this breeder (and probably
   *    of the crossover as well).
   * @return pointer to the instance of Breeder class.
   */
  virtual GLBaseBreeder* createBreeder(
                  int breeder_type,
                  GLBaseCrossover* crossover,
                  GLBaseParameters* breeder_parameters = NULL,
                  GLBaseGeneticAlgorithm* ga = NULL);
  /**
   * Creates an exemplar of the crossover class.
   * @param crossover_type of the crossover to be created.
   * @param ga genetic algorithm -- owner of this crossover.
   * @param crossover_parameters additional crossover parameters.
   * @return pointer to the instance of Crossover class.
   */
  virtual GLBaseCrossover* createCrossover(
       int crossover_type,
       GLBaseParameters* crossover_parameters = NULL,
       GLBaseGeneticAlgorithm* ga = NULL);
  
  /**
   * Creates an exemplar of the mutator class.
   * @param mutator_type type of the mutator to be created.
   * @param mutation_rate rate of the mutation, interpretation is upto
   *    concrete implementation of the mutator. Normally one would expect
   *    to keep it in (0; 1) interval.
   * @param mutator_parameters additional mutator parameters.
   * @param ga owner of this mutator.
   */
  virtual GLBaseMutator* createMutator(int mutator_type,
        double mutation_rate,
        GLBaseParameters* mutator_parameters = NULL,
        GLBaseGeneticAlgorithm* ga=NULL);       

  /**
   * Creates an exemplar of the ranker class.
   * @param ranker_type type of the ranker to be created.
   * @param ranker_parameters additional ranker parameters.
   * @param ga owner of this ranker.
   */
  virtual GLBaseRanker* createRanker(int ranker_type,
      GLBaseParameters* ranker_parameters = NULL,
      GLBaseGeneticAlgorithm* ga = NULL);
  
 /**
  * Creates an exemplar of the selectionist class.
  * @param selectionist_type type of the selectionist to be created.
  * @param ranker operator for arranging the organisms in accordance
  *    with their rank.
  * @param selectionist_parameters additional selectionist parameters.
  * @param ga owner of this selectionist.
  */
  virtual GLBaseSelectionist* createSelectionist(int selectionist_type,
      GLBaseRanker* ranker,
      GLBaseParameters* selectionist_parameters = NULL,
      GLBaseGeneticAlgorithm* ga = NULL);
  
  /**
   * Creates an exemplar of the variator class.
   * @param mutator mutator operator used by this variator.
   * @param variator_type type of the variator to be created.
   * @param variator_parameters additional variator parameters.
   * @param ga owner of this variator.
   */
  virtual GLBaseVariator* createVariator(int variator_type,
      GLBaseMutator* mutator,
      GLBaseParameters* variator_parameters = NULL,
      GLBaseGeneticAlgorithm* ga = NULL);
};

#endif /*GLFACTORY_H_*/
