#include <iostream>
#include "NLstl_utils.h"

#include "GLBaseRanker.h"
#include "GLGeneticAlgorithmStandard.h"
#include "GLGaStatusStandard.h"
#include "GLPopulationStandard.h"
#include "GLUtils.h"
using namespace std;

/**
 * Simple constructor.
 * @param parameters parameters of the GA. Just passed to the BaseGA.
 * @param factory factory to create GA elements (mutator, crossover etc.).
 */
GLGeneticAlgorithmStandard::GLGeneticAlgorithmStandard(
    const GLParametersGeneticAlgorithm& parameters, GLFactory* factory):
      GLBaseGeneticAlgorithm(parameters, factory)
{
  m_status = new GLGaStatusStandard();
}//constructor


/**
 * Destructor. Destroys everything.
 */
GLGeneticAlgorithmStandard::~GLGeneticAlgorithmStandard()
{
  if (m_status != NULL) delete m_status;
}//destructor

/**
 * Resets the GA after run -- resets all it elements, cleans the population
 * and the current status, frees memory occupied by organisms etc.
 * Make the GA ready to run again. The current status after the call of
 * this method is set to GA_STATE_INITIALISED.
 * Be careful not to reset the GA while it is running.
 */
void GLGeneticAlgorithmStandard::reset()
  {
    GLBaseGeneticAlgorithm::reset();
    delete m_status;
    m_status = new GLGaStatusStandard();
  }//reset

/**
 * Runs the GA.
 * @param initialiser organism initialiser -- will determine which
 *    type of organism will be used for GA. Ideally it should allow to
 *    use the same exemplar of GA for solving entirely different
 *    problems by changing only initialiser and evaluator.
 * @param evaluator class for evaluating the fitness function.
 * @param stoppers list of stoppers which determine when the GA is to stop.
 * @param bool_operator if == AND_OPERATOR then GA stops when ALL stoppers
 *    indicate that, when == OR_OPERATOR (default) then it happens when
 *    at least ONE of the stoppers is satisfied.
 * @return pointer to the structure with the GA status -- all the
 *    information necessary for post-processing the results, e.g.
 *    best ever organism in GA etc.
 */
const GLBaseGaStatus* GLGeneticAlgorithmStandard::runGA(
    GLBaseOrganismInitialiser *initialiser,
               GLBaseEvaluator* evaluator,
               const TListOfStoppers& stoppers,
                bool  bool_operator)
{
  m_status->m_timer.reset();
  //init population to start with.
#if VERBOSE > 1  
  cout << "Init population" << endl;
#endif

  nlutils::assertPointer(evaluator, 
        "\nGLGeneticAlgorithmStandard::runGA\n"
        "evaluator cannot be NULL pointer");
  m_evaluator = evaluator;
  int num_of_objectives = m_evaluator->getNumberOfObjectives();
  m_status->m_current_state = GLConstants::GA_STATE_RUNNING;

  //get the current state of evaluator
  int evaluator_initial_counter = m_evaluator->getCounter();
  GLBasePopulation *population = new GLPopulationStandard();
  population->populate(getParameters()->m_initial_population_size,
                       initialiser, GLConstants::NO_TWINS);

#if DEBUG > 2  
  cout << "Initial population" << endl;
  population->printRankedPopulation(cout, evaluator, getRanker());
#endif
  
  m_status->m_current_generation = 0;
  m_status->m_generations_since_population_changed = 0;
  m_status->m_generations_since_new_best_found = 0;
  m_status->m_generations_since_solution_improvement = 0;
  
  GLBaseFitness* current_fitness(NULL); //used to keep track on best value
                                        //of fitness function for single
                                        //objective oprimisation.
  //-----------------------------------------------------------------//
  //                            MAIN LOOP                            //
  //-----------------------------------------------------------------//
  //loop until stopping criteria is met.
#if VERBOSE==1
  cout << "Generations\n";
#endif
  while (!toFinish(stoppers, bool_operator))
    {
      if (population->createNewGeneration(
                    getBreeder(),
                    getVariator(), 
                    evaluator,
                    getSelectionist(),
                    m_status->m_best_organisms_in_population,
                    getParameters()->m_children_per_generation,
                    getParameters()->m_mutants_per_generation,
                    getParameters()->m_mutation_policy) == 0)
      {
        ++m_status->m_generations_since_population_changed;
      }
    else
      {
        m_status->m_generations_since_population_changed = 0;
      }
#if VERBOSE == 1
      cout << m_status->m_current_generation << "; ";
      if (m_status->m_current_generation % 15 == 14) cout << endl;
#endif
#if VERBOSE > 1
      cout << "Generation " << m_status->m_current_generation << endl;
#endif
#if DEBUG > 4
      population->printRankedPopulation(cout, evaluator,
                getRanker());
      cout << endl;
#endif
      //update the status of the GA.
      ++m_status->m_current_generation;
      //update the vector of best organisms of all time
      if (getSelectionist()->getRanker()->updateFittest(
                            m_status->m_best_organisms_in_population,
                            m_status->m_best_ever_organisms))
        {
          m_status->m_generations_since_new_best_found = 0;
#if VERBOSE > 1
          cout << "Best elements (total " 
               << m_status->m_best_ever_organisms.size() << ")\n";
          GLUtils::printVectorOfOrganisms(cout, 
                          m_status->m_best_ever_organisms);
#endif
        }
      else 
        {
          ++m_status->m_generations_since_new_best_found;
        }
      if (num_of_objectives > 1)
        {
          //for multiobjective GA those numbers should be the same.
          m_status->m_generations_since_solution_improvement = 
            m_status->m_generations_since_new_best_found;
        }
      else  
        {
          //for single objective we need to find, if the solution indeed
          //improved.
          if ((current_fitness == NULL) || 
              (m_status->m_best_ever_organisms.front()
                           ->getFitness()->isLess(current_fitness)))
            {
              current_fitness = m_status->m_best_ever_organisms.front()
                        ->getFitness()->copy();
              m_status->m_generations_since_solution_improvement = 0;
            }
          else
            {
              ++m_status->m_generations_since_solution_improvement;
            }
        }
      m_status->m_evaluator_calls = m_evaluator->getCounter() -
                    evaluator_initial_counter;
    }
#if VERBOSE==1
  cout << "\n";
#endif
  //-----------------------------------------------------------------//
  //                       END OF MAIN LOOP                          //
  //-----------------------------------------------------------------//
  
  //If necessary, store the last generation
  if (getParameters()->m_keep_last_generation)
    {
      population->getOrganismsVector(m_status->m_last_generation,
              GLConstants::REPLACE, -1, true);
      getRanker()->rankOrganisms(m_status->m_last_generation);
    }
  else 
    {
      m_status->m_last_generation.clear();
    }
  if (current_fitness != NULL) 
    {
      delete current_fitness;
    }
  delete population;
  m_status->m_current_state = GLConstants::GA_STATE_STOPPED_NORMALLY;
  m_status->m_time_required = m_status->m_timer.getTimeSec();
#if VERBOSE > 2
  cout << endl << *m_status;
#endif
  return m_status;
}//runGA
