/*
 * GLGeneticAlgorithmStandard.h
 *
 *  Created on: 18-Jan-2009
 *      Author: Artem_Babayan
 */

#ifndef GLGENETICALGORITHMSTANDARD_H_
#define GLGENETICALGORITHMSTANDARD_H_

#include "GLBaseGeneticAlgorithm.h"
#include "GLGaStatusStandard.h"

class GLFactory;
/**
 * Standard single objective GA.
 */
class GLGeneticAlgorithmStandard : public GLBaseGeneticAlgorithm
{
public:
  /**
   * Simple constructor.
   * @param parameters parameters of the GA. Just passed to the BaseGA.
   * @param factory factory to create GA elements (mutator, crossover etc.).
   */
  GLGeneticAlgorithmStandard(const GLParametersGeneticAlgorithm& parameters,
                             GLFactory* factory);

  /**
   * Destructor. Destroys everything.
   */
  virtual  ~GLGeneticAlgorithmStandard();

  /**
   * Returns the current status of the genetic algorithm.
   * @return pointer to the structure with the current status of the GA.
   */
  virtual const GLBaseGaStatus* getStatus() const
  { return m_status; }

  /**
   * Returns the pointer to the status of the standard GA.
   * @return pointer to the status of the standard GA.
   */
  const GLGaStatusStandard* getStatusStandard() const
  { return m_status; }
  
  /**
   * Resets the GA after run -- resets all it elements, cleans the population
   * and the current status, frees memory occupied by organisms etc.
   * Make the GA ready to run again. The current status after the call of
   * this method is set to GA_STATE_INITIALISED.
   * Be careful not to reset the GA while it is running.
   */
  virtual void reset();
  
  /**
   * Runs the GA.
   * @param initialiser organism initialiser -- will determine which
   *    type of organism will be used for GA. Ideally it should allow to
   *    use the same exemplar of GA for solving entirely different
   *    problems by changing only initialiser and evaluator.
   * @param evaluator class for evaluating the fitness function.
   * @param stoppers list of stoppers which determine when the GA is to stop.
   * @param bool_operator if == AND_OPERATOR then GA stops when ALL stoppers
   *    indicate that, when == OR_OPERATOR (default) then it happens when
   *    at least ONE of the stoppers is satisfied.
   * @return pointer to the structure with the GA status -- all the
   *    information necessary for post-processing the results, e.g.
   *    best ever organism in GA etc.
   */
  virtual const GLBaseGaStatus* runGA(
      GLBaseOrganismInitialiser *initialiser,
                 GLBaseEvaluator* evaluator,
                 const TListOfStoppers& stoppers,
                  bool  bool_operator = GLConstants::OR_OPERATOR);

private:
  GLGaStatusStandard* m_status;
};

#endif /* GLGENETICALGORITHMSTANDARD_H_ */
