#include <algorithm>
#include <NLstl_utils.h>

#include "GLBaseGaStatus.h"
#include "GLBaseGeneticAlgorithm.h"
#include "GLLogicError.h"
#include "GLMultipleGaRunner.h"
#include "GLRandomNumbersGenerator.h"

using namespace std;
/**
 * Simple constructor.
 */
GLMultipleGaRunner::GLMultipleGaRunner()
{
}//constructor

/**
 * Destructor. Destroys everything.
 */
GLMultipleGaRunner::~GLMultipleGaRunner()
{
}//destructor

/**
 * Makes several runs of Genetic Algorithm(s). May run the same or several 
 * GAs many times over the same problem.
 * @param genetic_algorithms vector of genetic algorithms to run.
 * @param number_of_runs how many times each genetic algorithm will run
 *    (number_of_runs.size() >= genetic_algorithms.size()).
 * @param initialiser is used to define the initial population of the GA
 *    (including the type of organisms).
 * @param evaluator objective function.
 * @param stoppers list of stoppers (classes which determine when GA should
 *    stop).
 * @param bool_operator if == AND_OPERATOR then GA stops when ALL stoppers
 *    indicate that, when == OR_OPERATOR (default) then it happens when
 *    at least ONE of the stoppers is satisfied.
 * @param results output vector of vectors of results. One result per GA
 *    per run. Previous content will be deleted.
 * @param same_random_seed when true, rangom number generator is 
 *    initialised with the same random seed each time new GA starts to work
 *    (e.g. if 2 GAs are scheduled to run 10 times each the random 
 *    generator will be initialised twice with the same seed). 
 */
void GLMultipleGaRunner::run( 
    const vector<GLBaseGeneticAlgorithm*>& 
                              genetic_algorithms,
           const vector<int>& number_of_runs,
   GLBaseOrganismInitialiser* initialiser,
             GLBaseEvaluator* evaluator,
       const TListOfStoppers& stoppers,
                        bool  bool_operator,
    vector< vector<GLBaseGaStatus*> >&
                              results,
                         bool same_random_seed)
  {
    //first check consistency of input data.
    if (genetic_algorithms.size() > number_of_runs.size())
      {
        char errormsg[1000];
        sprintf(errormsg, "\nGLMultipleGaRunner::Run\n"
                          "Size of array of genetic algorithms (==%i) "
                          "cannot exceed the size of number_of_runs "
                          "vector(== %i)",
                          (int)(genetic_algorithms.size()),
                          (int)(number_of_runs.size()));      
        throw GLLogicError(errormsg);
      }
    //clean the vector of results
    for(size_t i = 0; i < results.size(); ++i)
      {
        for_each(results[i].begin(), results[i].end(),
                 nlutils::DeleteObject());
      }
    results.clear();
    results.resize(genetic_algorithms.size());

    long int random_seed(0);
    //run the GAs
    for(size_t i = 0; i < genetic_algorithms.size(); ++i)
      {
        //if necessary, use the same random seed for each GA
        if (same_random_seed && (i == 0))
          {
            random_seed = GLRandomNumbersGenerator::initGenerator();
          }
        if (same_random_seed && (i > 0))
          {
            GLRandomNumbersGenerator::initGenerator(random_seed);
          }
        for(int j = 0; j < number_of_runs[i]; ++j)
          {
#if VERBOSE > 0
            cout << "\n\n---- Running batch " << i + 1 << " (out of " 
                 << genetic_algorithms.size() << "), test " << j + 1
                 << " (out of " << number_of_runs[i] << ") ----\n";
#endif
            genetic_algorithms[i]->reset();
            results[i].push_back(
                    genetic_algorithms[i]->runGA(initialiser, 
                    evaluator, stoppers, bool_operator)->copy());
          }
      }
  }//Run
