#ifndef GLCOMPARATOR_H_
#define GLCOMPARATOR_H_

#include <vector>

#include "GLBaseGaStopper.h"

using namespace std;

class GLBaseGeneticAlgorithm;
class GLBaseEvaluator;
class GLBaseOrganismInitialiser;
class GLBaseGaStatus;

/**
 * This class may run one or several Genetic Algorithms over the same 
 * problem many times to collect statistics on GA's performance or to get 
 * several solutions. 
 */
class GLMultipleGaRunner
{
public:
  /**
   * Simple constructor.
   */
  GLMultipleGaRunner();
  /**
   * Destructor. Destroys everything.
   */
  virtual ~GLMultipleGaRunner();
  
  /**
   * Makes several runs of Genetic Algorithm(s). May run the same or several 
   * GAs many times over the same problem.
   * @param genetic_algorithms vector of genetic algorithms to run.
   * @param number_of_runs how many times each genetic algorithm will run
   *    (number_of_runs.size() >= genetic_algorithms.size()).
   * @param initialiser is used to define the initial population of the GA
   *    (including the type of organisms).
   * @param evaluator objective function.
   * @param stoppers list of stoppers (classes which determine when GA should
   *    stop).
   * @param bool_operator if == AND_OPERATOR then GA stops when ALL stoppers
   *    indicate that, when == OR_OPERATOR (default) then it happens when
   *    at least ONE of the stoppers is satisfied.
   * @param results output vector of vectors of results. One result per GA
   *    per run. Previous content will be deleted.
   * @param same_random_seed when true, rangom number generator is 
   *    initialised with the same random seed each time new GA starts to work
   *    (e.g. if 2 GAs are scheduled to run 10 times each the random 
   *    generator will be initialised twice with the same seed). 
   */
  virtual void run( 
      const vector<GLBaseGeneticAlgorithm*>& 
                                genetic_algorithms,
             const vector<int>& number_of_runs,
     GLBaseOrganismInitialiser* initialiser,
               GLBaseEvaluator* evaluator,
         const TListOfStoppers& stoppers,
                          bool  bool_operator,
      vector< vector<GLBaseGaStatus*> >&
                                results,
                           bool same_random_seed = false);
};

#endif /*GLCOMPARATOR_H_*/
