#include <algorithm>
#include <functional>
#include <iostream>
#include <numeric>

#include <NLstl_utils.h>

#include "GLLogicError.h"
#include "GLMutatorCombined.h"
#include "GLRandomNumbersGenerator.h"

 /**
  * Copying constructor.
  * @param mutator source for copying.
 */
GLMutatorCombined::GLMutatorCombined(const GLMutatorCombined& mutator):
  GLBaseMutator(mutator), 
  m_apply_mutators(mutator.m_apply_mutators),
  mv_weights(mutator.mv_weights),
  mv_norm_cumul_weights(mutator.mv_norm_cumul_weights)

  {
    //create the copies of 'submutators'
    mv_mutators.resize(mutator.mv_mutators.size());
    for(size_t i = 0; i < mv_mutators.size(); i++)
        mv_mutators[i] = mutator.mv_mutators[i]->copy();
  }//constructor

/**
 * Destructor. Destroys all the 'submutators'.
 */
GLMutatorCombined::~GLMutatorCombined()
{
  //delete all mutators
  for_each(mv_mutators.begin(), mv_mutators.end(), 
           nlutils::DeleteObject());
}//destructor

/**
 * Checks that this class's parameters are set properly (e.g. if weights
 * should be supplied, they are all positive etc.) and performs some
 * initialisation. Should be called in the constructor.
 */
void GLMutatorCombined::checkAndInit()
  {
    //check that vector of mutators is not empty
    if (mv_mutators.empty())
      {
        char errormsg[1000];
        sprintf(errormsg, "\nGLMutatorCombined::GLMutatorCombined\n"
                          "Empty vector of mutators passed to the "
                          "constructor");
        throw GLLogicError(errormsg);
      }
    //check that vector of mutators does not contain NULL's
    vector<GLBaseMutator*>::const_iterator null_pointer = 
        find(mv_mutators.begin(), mv_mutators.end(), (GLBaseMutator*)NULL);
    if (null_pointer != mv_mutators.end())
      {
        char errormsg[1000];
        sprintf(errormsg, "\nGLMutatorCombined::GLMutatorCombined\n"
                          "Vector with mutators contains NULL pointer at %i\n",
                          (int)(null_pointer - mv_mutators.begin()));
      }
    //check, that, if apply_mutator==SEPARATE the weight vector is valid.
    if (m_apply_mutators == SEPARATE)
      {
        if (mv_weights.size() != mv_mutators.size())
          {
            char errormsg[1000];
            sprintf(errormsg, "\nGLMutatorCombined::GLMutatorCombined\n"
                            "Mutators usage set to SEPARATE and the "
                            "number of weights (%i) != number of mutators (%i)"
                            "passed\n",
                            (int)mv_weights.size(), (int)mv_mutators.size());
            throw GLLogicError(errormsg);
          }
        //look for non-positive weights
        vector<double>::const_iterator not_positive = 
            find_if(mv_weights.begin(), mv_weights.end(), 
                    bind2nd(less_equal<double>(), 0.0));
        if (not_positive != mv_weights.end())
          {
            char errormsg[1000];
            sprintf(errormsg, "\nGLMutatorCombined::GLMutatorCombined\n"
                            "Mutators usage set to SEPARATE and the "
                            "weights[%i] is not positive (== %e)\n",
                            int(not_positive - mv_weights.begin()), 
                            (*not_positive));
            throw GLLogicError(errormsg);
          }
        //Ok, all is good, now process the vector of weights.
        processWeights();
      }
  }//checkAndInit

/**
 * Mutate the organism by applying all the mutators one-by-one.
 * @param organism organism to be mutated.
 * @return true if organism was mutated and false otherwise.
 */
bool GLMutatorCombined::mutateCombined(GLBaseOrganism* organism)
  {
    bool result = false;
    //loop through the vector of mutators
    vector<GLBaseMutator*>::iterator it;
    for(it = mv_mutators.begin(); it != mv_mutators.end(); ++it)
      {
        bool res = (*it)->mutateOrganism(organism);
        result |= res;  //if at least one of the mutators changed organism.
      }
    return result;
  }//mutateCombined

/**
 * Performs the mutation on the organism. If passed organism
 * was changed (mutated) returns true, otherwise returns false.
 * @param organism organism to be mutated.
 * @return true if organism was mutated and false otherwise.
 */
bool GLMutatorCombined::mutateOrganism(GLBaseOrganism* organism)
  {
    bool result = false;
    //first, decide if we need to mutate at all
    if (GLRandomNumbersGenerator::getRandomUniform(0, 1) > getMutationRate())
      {
        return result;
      }
    
    //Ok, we decided to mutate
    switch(m_apply_mutators)
    {
      case SEPARATE:
        result = mutateSeparately(organism);
        break;
      case COMBINED:
        result = mutateCombined(organism);
        break;
    }
    return result;
  }//mutateOrganism

/**
 * Mutate the organism by choosing one of the mutators from
 * the vector according to the weights and applying it to
 * the organism.
 * @param organism organism to be mutated.
 * @return true if organism was mutated and false otherwise.
 */
bool GLMutatorCombined::mutateSeparately(GLBaseOrganism* organism)
  {
    bool result = false;
    //generate random number between 0 and one
    double rand = GLRandomNumbersGenerator::getRandomUniform(0, 1);
    
    //find the first element i of vector of normalised cumulative weights 
    //such that rand <= ncw[i];
    vector<double>::const_iterator it =
       find_if(mv_norm_cumul_weights.begin(),
               mv_norm_cumul_weights.end(),
               bind2nd(greater_equal<double>(), rand));
    size_t i = it - mv_norm_cumul_weights.begin();
    //this condition should never be satisfied, but just in case...
    if (i == mv_norm_cumul_weights.size()) i--;
    
    //finally, mutate the organism
    result = mv_mutators[i]->mutateOrganism(organism);
    return result;
  }//mutateSeparately
  
/**
 * Fills vector of normalized cumulative weights (ncw) based on the vector
 * of weights (w) -- last element of ncw is always 1.0. 
 * ncw[n] = sum(i=0..n, w[i])/sum(i = 0..size(w), w[i]).
 */
void GLMutatorCombined::processWeights()
  {
    mv_norm_cumul_weights.resize(mv_weights.size());
    //do nothing for empty vector of weights
    if (mv_weights.empty()) return;
    //fill it with partial sums weights
    partial_sum(mv_weights.begin(), mv_weights.end(),
                mv_norm_cumul_weights.begin());
    //take the last element of the cumulative weights
    double total = mv_norm_cumul_weights.back();
    
    //check that it is more > 0 (should be, but just in case)
    if (total <= 0)
      {
        char errormsg[1000];
        sprintf(errormsg, "\nGLMutatorCombined::processWeights\n"
                          "Total weight (%e) is <= 0, why?!\n",
                          total);
        throw GLLogicError(errormsg);
      }
    //normalise vector of partial sums
    for_each(mv_norm_cumul_weights.begin(), mv_norm_cumul_weights.end(),
             bind2nd(divides<double>(), total));
    //force the last element to 1.0
    mv_norm_cumul_weights.back() = 1.0;
  }//processWeights
  
