#ifndef GLMUTATORCOMBINED_H_
#define GLMUTATORCOMBINED_H_
#include <vector>

#include "GLBaseMutator.h"
#include "GLBaseParameters.h"

using namespace std;
/**
 * This class allows combining several mutator in one. It contains vector of
 * mutators (which will be destroyed in destructor of this class) which can
 * be used to mutate the organisms in two possible ways. 
 *
 * In SEPARATE mode organism can be mutated by one mutator only. The 
 *      probability with which each mutator will be chosen is defined
 *      by the vector of weight coefficients.
 * 
 * In COMBINED mode all the mutators will be applyed to organism. The order,
 *      in which mutators are to be applied is the same as their order in
 *      the vector of mutators. In this the vector of weights is ignored.
 * 
 * Note, that each mutator has also its own mutation rate which will be 
 * 'superimposed' in each of the mutators. So the algorithm for mutation
 * operator is this:
 *      1) Use mutator_rate of THIS class to decide if the organism will be 
 *         mutated.
 *      2) Depending on apply_mutators status one mutator will be chosen or
 *         the loop through all the mutators will be entered.
 *      3) Each (or single) mutator will decide, whether to mutate the 
 *         organism using its own mutation rate.
 * Therefore, if mutation rate for this class is set to 0.5 and mutation rates
 * for all mutators in the vector are 0.2 the total chance of the organism
 * being mutated is 0.1 in SEPARATE mode.
 */
class GLMutatorCombined : public GLBaseMutator
{
public:
  /** 
   * Determines, how the mutators from the set will be applied to the 
   * organism.
   */ 
  enum TUsage
  {
    SEPARATE = 1,   //only one mutator can be applied to the organism
                    //during mutation.
    COMBINED = 2    //all mutators can be applied to the organism during
                    //mutation.
  };
  class TParameters;
public:
  /**
   * Constructor. Sets up all the necessary data. By default always tries
   * to mutate the organism by applying to it all the 'submutators' 
   * one-by-one.
   * @param mutators vector with pointers to mutators which will be used in 
   *    mutation operator. These mutators become 'property' of this class
   *    and will be destroyed in destructor.
   * @param mutation_rate chance that any of the mutators in the vector
   *    will try to mutate the organisms. Defaut value is 1.0, so some mutator
   *    always will be applied and the probability of the organism to mutate
   *    will be determined by the mutators in the vector.
   * @param weights probability with which particular mutator will be chosen
   *    to mutate the organism. This vector is used only when apply_mutator
   *    is equal to SEPARATE. The probability for i-th mutator to be chosen
   *    is weights[i]/sum(weights), so the impact of vectors {0.5, 0.5} and 
   *    {1.0, 1.0} is equivalent. If apply_mutators == SEPARATE the size of
   *    vector 'weights' should be equal to size of 'mutators'.
   * @param apply_mutators flag which indicates how the mutators will be 
   *    applied to the organism.     
   */
  GLMutatorCombined(const vector<GLBaseMutator*>& mutators,
                    const double& mutation_rate = 0.1,
                    const vector<double>& weights = vector<double>(0),
                    TUsage apply_mutators = COMBINED):
                      GLBaseMutator(mutation_rate, true),
                      m_apply_mutators(apply_mutators),
                      mv_mutators(mutators), 
                      mv_weights(weights) 
   { checkAndInit(); }                       
  
                     
  /**
   * Constructor. Sets up all the necessary data. 
   * @param mutation_rate chance that any of the mutators in the vector
   *    will try to mutate the organisms. Defaut value is 1.0, so some mutator
   *    always will be applied and the probability of the organism to mutate
   *    will be determined by the mutators in the vector.
   * @param parameters pointer to structure which contains all the necessary 
   *    information to create this mutator.     
   */
  GLMutatorCombined(const double& mutation_rate,
                    GLMutatorCombined::TParameters *parameters):
                      GLBaseMutator(mutation_rate, true),
                      m_apply_mutators(parameters->t_apply_mutators),
                      mv_mutators(parameters->tv_mutators), 
                      mv_weights(parameters->tv_weights) 
  { checkAndInit(); }
  
  /**
   * Copying constructor. Creates copies of submutators.
   * @param mutator source for copying.
   */
  GLMutatorCombined(const GLMutatorCombined& mutator);
  
  /**
   * Destructor. Destroys all the 'submutators'.
   */
  virtual ~GLMutatorCombined();
  
  /**
   * Creates copy of the current exemplar of the mutator.
   * Should be overloaded in childrens.
   * @return copy of the current exemplar of the mutator.
   */
  virtual GLBaseMutator* copy() const
  { return new GLMutatorCombined(*this); }
  
  /**
   * Performs the mutation on the organism. If passed organism
   * was changed (mutated) returns true, otherwise returns false.
   * @param organism organism to be mutated.
   * @return true if organism was mutated and false otherwise.
   */
  virtual bool mutateOrganism(GLBaseOrganism* organism);
protected:
  /**
   * Checks that this class's parameters are set properly (e.g. if weights
   * should be supplied, they are all positive etc.) and performs some
   * initialisation. Should be called in the constructor.
   */
  void checkAndInit();
  
  /**
   * Mutate the organism by applying all the mutators one-by-one.
   * @param organism organism to be mutated.
   * @return true if organism was mutated and false otherwise.
   */
  bool mutateCombined(GLBaseOrganism* organism);
  
  /**
   * Mutate the organism by choosing one of the mutators from
   * the vector according to the weights and applying it to
   * the organism.
   * @param organism organism to be mutated.
   * @return true if organism was mutated and false otherwise.
   */
  bool mutateSeparately(GLBaseOrganism* organism);
  
  /**
   * Fills vector of normalized cumulative weights (ncw) based on the vector
   * of weights (w) -- last element of ncw is always 1.0. 
   * ncw[n] = sum(i=0..n, w[i])/sum(i = 0..size(w), w[i]).
   */
  void processWeights();
public:
  /**
   * Internal class of the additional (to base mutator's) parameters
   * necessary to initialise this class.
   */
  struct TParameters:public GLBaseParameters
  {
   /**
    * Creates new exemplar of GLMutatorCombined:: TParameters class.
    * @param mutators vector of mutators to be used in combine mutator.
    *   These 'submutators' will be destroyed in the combined mutator's
    *   destructor.
    * @param apply_mutators the way, how 'submutators' will be applyed --
    *   all of them to the single organism during the mutation (SEPARATE)
    *   or just one of them will be randomly chosen (COMBINED). In first
    *   case the vector of weights must be supplied and be of the
    *   same size as the vector of 'submutators'.
    * @param weights determine the probability with which particular 
    *   'submutator' from the set will be chosen (see description of the
    *   GLMutatorCombined constructor for more information). When supplied
    *   must be the size of 'mutators' vector. 
    */
    TParameters(const vector<GLBaseMutator*>& mutators,
                const vector<double>& weights = vector<double>(0),
                TUsage apply_mutators = COMBINED):
                  t_apply_mutators(apply_mutators),
                  tv_mutators(mutators),
                  tv_weights(weights)
                  
    {}

    ///Copying constructor. Note, that pointers in the 'mutators' vector
    ///will be copied as pointers -- no actual copies of classes will be
    ///created.
    ///@param source source of the data to copy;
    TParameters(const TParameters *source):
                  t_apply_mutators(source->t_apply_mutators),
                  tv_mutators(source->tv_mutators),
                  tv_weights(source->tv_weights)
    {}

    /**
     * Creates the copy of the parameters.
     * @return pointer to identical copy of the current class exemplar.
     */
    virtual GLBaseParameters* copy() const
    { return new TParameters(this); }
    
    ///Determines how the multiple mutators will be applied to organisim 
    ///(all together or one at a time).
    TUsage t_apply_mutators;
    ///Vector of 'basic' mutators.
    vector<GLBaseMutator*> tv_mutators;
    ///vector of weights (probability that one of the mutator will be chosen
    ///to mutate given organism, make sense only when 
    ///t_apply_mutators == SEPARATE)
    vector<double> tv_weights;
  };
  
private:
  TUsage m_apply_mutators;
  vector<GLBaseMutator*> mv_mutators;
  vector<double> mv_weights;
  vector<double> mv_norm_cumul_weights;  //vector of normalized cumulative
                                         //weights
};

#endif /*GLMUTATORCOMBINED_H_*/
