#include <iostream>
#include <math.h>

#include <NLstl_utils.h>

#include "GLBaseOrganism.h"
#include "GLInterfaceShuffleGenes.h"
#include "GLMutatorShuffling.h"
#include "GLRandomNumbersGenerator.h"

using namespace std;
/**
 * Constructor. Sets parameters.
 * @param mutation_rate probability that organism will be mutated.
 * @param parameters structure which contains maximum number of genes
 *    to shuffle and flag which determines how this number is chosen.
 */
GLMutatorShuffling::GLMutatorShuffling(double mutation_rate,
                   const GLMutatorShuffling::TParameters *const parameters):
                     GLBaseMutator(mutation_rate, true)
  {
    m_parameters = *parameters;
  }//constructur


/**
 * Performs the mutation on the organism. Chooses the number of
 * genes to shuffle, which genes to shuffle and then shuffles them.
 * If passed organism was changed (mutated) returns true, otherwise
 * returns false.
 * @param organism organism to be mutated.
 * @return true if organism was mutated and false otherwise.
 */
bool GLMutatorShuffling::mutateOrganism(GLBaseOrganism* organism)
  {
    //Get the reference to the shuffable interface.
    GLInterfaceShuffleGenes* x_organism =
        nlutils::transform_pointer<GLInterfaceShuffleGenes,
         GLBaseOrganism>(organism,
                         "GLMutatorShuffling::mutateOrganism\n"
                         "Passed pointer is either NULL or does not point "
                         "to the organism with GLInterfaceShuffleGenes");

    //See if random mutation event should occur
    if (GLRandomNumbersGenerator::getRandomUniform(0, 1) >
                getMutationRate())
      {
        return false;
      }

    //Check, that maximum number of genes to swap is >= 0 and
    //does not exceed the size of the organism.
    int organism_size = organism->getSize();
    int min_genes_to_shuffle = m_parameters.t_min_genes_to_shuffle;
    int max_genes_to_shuffle = m_parameters.t_max_genes_to_shuffle;
    if ((min_genes_to_shuffle < 0) ||
        (min_genes_to_shuffle > organism_size))
      {
        min_genes_to_shuffle = organism_size;
      }
    if ((max_genes_to_shuffle < 0) ||
        (max_genes_to_shuffle > organism_size))
      {
        max_genes_to_shuffle = organism_size;
      }
    //if number of genes to shuffle <= 1 -- exit, as there is nothing to do.
    if (max_genes_to_shuffle < 2)
      {
        ///organism did not mutated.
        return false;
      }
    int genes_to_shuffle;
    //Now, choose randomly how many genes are to shuffle
    if (min_genes_to_shuffle == max_genes_to_shuffle)
      {
        genes_to_shuffle = min_genes_to_shuffle;
      }
    if (m_parameters.t_uniform)
      genes_to_shuffle = GLRandomNumbersGenerator::getRandomInteger(
            min_genes_to_shuffle, max_genes_to_shuffle + 1);
    else
      {
        //for non-uniform case more complicated formula required.
        double probability =
            GLRandomNumbersGenerator::getRandomUniform(0, 1);
        genes_to_shuffle = (int)(-log(1. - probability) / log(2.)) + 
                          min_genes_to_shuffle;
        if (genes_to_shuffle > max_genes_to_shuffle)
          genes_to_shuffle = max_genes_to_shuffle;
      }
    //Choose randomly 'genes_to_shuffle' genes -- shuffle the array of size
    //of organism and take first few -- not very effective for large
    //organism sizes. [TODO] get better algorithm.
    vector<int> gene_numbers;
    GLRandomNumbersGenerator::shuffleIntegerVector(
              gene_numbers, 0, organism_size);
    gene_numbers.erase(gene_numbers.begin() + genes_to_shuffle,
                       gene_numbers.end());
    //shuffle genes
    x_organism->shuffleGenes(gene_numbers);
    //return indicator that genes were shuffled.
    return true;
  }//mutateOrganism

