#ifndef GLMUTATORSHUFFLING_H_
#define GLMUTATORSHUFFLING_H_
#include <iostream>
#include "GLBaseMutator.h"
#include "GLBaseParameters.h"

/**
 * This mutator randomly choses random number of genes and shuffles them
 * (yes, randomly).
 */
class GLMutatorShuffling : public GLBaseMutator
{
public:
  class TParameters;
  /**
   * Constructor. Sets parameters.
   * @param mutation_rate probability that organism will be mutated.
   * @param parameters structure which contains maximum number of genes
   *    to shuffle and flag which determines how this number is chosen.
   */
  GLMutatorShuffling(double mutation_rate,
                     const GLMutatorShuffling::TParameters * const parameters);
  
  /**
   * Copying constructor.
   * @param source where the data are copied from.
   */
  GLMutatorShuffling(const GLMutatorShuffling& source):
    GLBaseMutator(source)
    {m_parameters = source.m_parameters;}
  
  /**
   * Destructor -- destroys everything.
   */
  virtual ~GLMutatorShuffling() {};
  
  /**
   * Creates copy of the current exemplar of the mutator.
   * Should be overloaded in childrens.
   * @return copy of the current exemplar of the mutator.
   */
  virtual GLBaseMutator* copy() const
  { return new GLMutatorShuffling(*this); }
  
  /**
   * Performs the mutation on the organism. Chooses the number of 
   * genes to shuffle, which genes to shuffle and then shuffles them.
   * If passed organism was changed (mutated) returns true, otherwise 
   * returns false.
   * @param organism organism to be mutated.
   * @return true if organism was mutated and false otherwise.
   */
  virtual bool mutateOrganism(GLBaseOrganism* organism);
  
public:  
  /**
   * Internal class of the additional (to base mutator's) parameters
   * necessary to initialise this class.
   */
  struct TParameters:public GLBaseParameters
  {
   /**
    * Creates new exemplar of the GLMutatorShuffling::TParameters class.
    * @param min_genes_to_shuffle mininum number of genes this mutator
    *   will shuffle. Should be <= max_genes_to_shuffle (if not -- will be
    *   corrected -- assigned the value of max_genes_to_shuffle). 
    *   If 0 or 1 will be automatically corrected to 2. If -1  it will 
    *   shuffle all the genes in the organism.
    * @param max_genes_to_shuffle maximum number of genes this mutator
    *     can shuffle. When < 0 it can shuffle all the genes in the organism.
    *     When 0 or 1 this mutator does nothing.
    * @param uniform when true the number of genes which this mutator
    *   can shuffle is uniformly distributed random number from 2 to
    *   max_genes_to_shuffle. When false -- it will have 0.5 probability to
    *   choose 2 genes, 0.25 -- 3 genes 0.125 -- 4 genes etc.
    */
    TParameters(int min_genes_to_shuffle = -1, 
                int max_genes_to_shuffle = -1,
                bool uniform = false)
    {
      t_min_genes_to_shuffle = min_genes_to_shuffle;
      t_max_genes_to_shuffle = max_genes_to_shuffle;
      t_uniform = uniform;
      //correct parameters if necessary
      if ((t_max_genes_to_shuffle == 0) || (t_max_genes_to_shuffle == 1))
        {
          t_max_genes_to_shuffle = 2;
        }
      if ((t_min_genes_to_shuffle == 0) || (t_min_genes_to_shuffle == 1))
        {
          t_min_genes_to_shuffle = 2;
        }
      if ((t_min_genes_to_shuffle > 0) && (t_max_genes_to_shuffle > 0) &&
          (t_min_genes_to_shuffle > t_max_genes_to_shuffle))
        {
          t_min_genes_to_shuffle = t_max_genes_to_shuffle;
        }
    }//constructor

    ///copying constructor
    ///@param source source of the data to copy;
    TParameters(const TParameters *source)
    {
      t_min_genes_to_shuffle = source->t_min_genes_to_shuffle;
      t_max_genes_to_shuffle = source->t_max_genes_to_shuffle;
      t_uniform = source->t_uniform;
    }//constructor

    /**
     * Creates the copy of the parameters.
     * @return pointer to identical copy of the current class exemplar.
     */
    virtual GLBaseParameters* copy() const
    { return new TParameters(this); }

    ///Minimum number of genes the mutator will shuffle (when < 0 -- all
    ///genes).
    int t_min_genes_to_shuffle;
    ///Maximum number of genes it can shuffle (when < 0 -- all genes)
    int t_max_genes_to_shuffle;
    /// when true the number of genes which this mutator
    /// can shuffle is uniformly distributed random number from 2 to
    /// max_genes_to_shuffle. When false -- it will have 0.5 probability to
    /// choose 2 genes, 0.25 -- 3 genes 0.125 -- 4 genes etc.
    bool t_uniform; 
  };
private:
  TParameters m_parameters;
};

#endif /*GLMUTATORSHUFFLING_H_*/
