#include <NLstl_utils.h>

#include "GLLogicError.h"
#include "GLOrganismVector.h"
#include "GLRuntimeError.h"

  /**
   * Creates organism based on 1D vector representation.
   * @param init_size initial size of the organism.
   *                  Should be non-negative.
   * @param size_property determines the size variability of the
   *              organism.
   * @param max_size maximum size of the organism (used only when
   *      size_property is != FIXED, otherwise set to init_size).
   *      By default for organism of not_fixed size it will be set to
   *      init_size.
   */
template<class T>
GLOrganismVector<T>::GLOrganismVector(unsigned init_size,
              GLConstants::TSizeLimits size_property,
                      unsigned max_size):
                GLBaseOrganism(),
                m_property_bounds(GLConstants::UNBOUNDED_GENES),
                m_property_size(size_property)
{
  setSizeData(init_size, max_size);
  m_storage.resize(init_size);
}//GLOrganismVector

/**
 * Creates organism based on 1D vector representation.
 * @param chromosome chromosome -- content of the organism.
 * @param size_property determines the size variability of the
 *              organism.
 * @param max_size maximum size of the organism (used only when
 *      size_property is != FIXED, otherwise set to chromosome.size()).
 *      By default for organism of not_fixed size it will be set to
 *      chromosome.size().
 */
template<class T>
GLOrganismVector<T>::GLOrganismVector(const vector<T>& chromosome,
          GLConstants::TSizeLimits size_property,
            unsigned max_size):
              GLBaseOrganism(),
              m_property_bounds(GLConstants::UNBOUNDED_GENES),
              m_property_size(size_property),
              m_storage(chromosome)
{
  setSizeData(chromosome.size(), max_size);
}//GLOrganismVector

/**
 * Copying constructor.
 * @param source source organism.
 */
template<class T>
GLOrganismVector<T>::GLOrganismVector(const GLOrganismVector<T>& source):
  GLBaseOrganism(source)
  {
    m_gene_max          = source.m_gene_max;
    m_gene_min          = source.m_gene_min;
    m_property_bounds   = source.m_property_bounds;
    m_property_size     = source.m_property_size;
    m_max_size          = source.m_max_size;
    m_storage           = source.m_storage;
  }//GLOrganismVector

/**
 * Destructor. Destroys everything. 
 */
template<class T>
GLOrganismVector<T>::~GLOrganismVector()
  {
    
  }//destructor

/**
 * Checks if the passed pointer is compatible with the current class.
 * If not will throw GLLogicError. Normally this class should be
 * overloaded in parents.
 * @param source class to check the compatibility with the current.
 * @message some text to be used in exception. E.g. where this function
 *    was called from.
 */
template<class T>
void GLOrganismVector<T>::assertCompatibility(const GLBaseOrganism* source,
                                const string& message) const
  {
    if (dynamic_cast<const GLOrganismVector<T>*>(source) == NULL)
      {
        char errormsg[1000];
        sprintf(errormsg, "\nGLOrganismVector<T>::assertCompatibility\n"
                          "%s\n"
                          "Passed pointer is either NULL or not compatible "
                          "with this class\n",
                          message.c_str());
        throw GLLogicError(errormsg);
      }
  }//assertCompatibility

/**
 * Copies the genetic information from the source organism
 * to the current one (e.g. chromosome, type or the organism,
 * bounds etc.) leaving out fitness information and similar
 * not-internal data. Normally will reset fitness value.
 * @param source source organism.
*/
template<class T>
void GLOrganismVector<T>::copyGeneticData(const GLBaseOrganism *source)
{
  //check if organisms are compatible
  assertCompatibility(source, "From: GLOrganismVector<T>::copyGeneticData");
  const GLOrganismVector<T> *organism =
      dynamic_cast<const GLOrganismVector<T>*> (source);
  GLBaseOrganism::copyGeneticData(source);
  m_gene_max            = organism->m_gene_max;
  m_gene_min            = organism->m_gene_min;
  m_property_bounds     = organism->m_property_bounds;
  m_property_size       = organism->m_property_size;
  m_max_size            = organism->m_max_size;
  m_storage             = organism->m_storage;
}//copyGeneticData

/**
 * Exchanges 2 genes withing the organism.
 * @param first index of the first gene.
 * @param second index of the second gene.
*/
template<class T>
void GLOrganismVector<T>::exchangeGenes(int first, int second)
{
  if (!checkIndex(first) || !(checkIndex(second)))
        {
          char errormsg[1000];
          sprintf(errormsg, "\nGLOrganismVector::exhangeGenes\n"
                            "One of the indices -- first (%i) or second (%i)"
                            " is outside the organism's size [0, %i)\n",
                            first, second, getSize());
          throw GLLogicError(errormsg);
        }
  T temp = m_storage[first];
  m_storage[first] = m_storage[second];
  m_storage[second] = temp;
}//exchangeGenes

/**
 * Returns the single gene of the organism.
 * @param index gene's index.
 * @return single gene.
 */
template<class T>
T& GLOrganismVector<T>::getGene(int index)
  {
    if (!checkIndex(index))
      {
        char errormsg[1000];
        sprintf(errormsg, "\nGLOrganismVector::getGene\n"
                          "Index = %i"
                          " is outside the organism's size [0, %i)\n",
                          index, getSize());
        throw GLLogicError(errormsg);
      }
    return m_storage[index];
  }//getGene

/**
 * Returns the single gene of the organism.
 * @param index gene's index.
 * @return single gene.
 */
template<class T>
const T& GLOrganismVector<T>::getGene(int index) const
  {
    if (!checkIndex(index))
      {
        char errormsg[1000];
        sprintf(errormsg, "\nGLOrganismVector::getGene\n"
                          "Index = %i"
                          " is outside the organism's size [0, %i)\n",
                          index, getSize());
        throw GLLogicError(errormsg);
      }
    return m_storage[index];
  }//getGene

/**
 * Returns the current size of the organism (number of genes).
 * @return size of the organism.
 */
template<class T>
int GLOrganismVector<T>::getSize() const 
  { return m_storage.size(); }

/**
 * Returns true if 2 organisms are equal.
 * @param organism another organism to be compared with.
 * @param soft when true soft comparison is done -- only
 *            gene's value. When false compares all others
 *            values -- fitness fucntion etc.
 *            Tag's value are never compared.
 */
template<class T>
bool GLOrganismVector<T>::isEqual(const GLBaseOrganism *organism,
             bool soft) const
{
  /**
   * First, find out if the second organism is comparable in principle.
   */
  const GLOrganismVector<T>* org =
      dynamic_cast<const GLOrganismVector<T>*>(organism);
  //organisms are not comparable, nothing to do.
  if (org == NULL) return false;
  bool flag = (m_storage == org->m_storage);
  return flag &&   //return flag or compare further if soft == false
         (soft ||
             ((isEvaluated() == org->isEvaluated()) &&
              getFitness()->isEqual(org->getFitness()))
              );
}//isEqual

/**
 * Returns true if this organism is less the one passed as a parameter.
 * @param organism another organism to be compared with.
 * @param soft when true soft comparison is done -- based only on
 *            gene's value. When false compares all others
 *            values -- fitness fucntion etc.
 *            Tag's value are never compared.
 */
template<class T>
bool GLOrganismVector<T>::isLess(const GLBaseOrganism *organism,
             bool soft) const
  {
    const GLOrganismVector<T>* org = 
      nlutils::transform_pointer<const GLOrganismVector<T>, 
                                 const GLBaseOrganism>
        (organism, "GLOrganismVector<T>::isLess\n"
                   "Passed argument either not compatible or NULL");
    //loop through the available elements in both vectors.
    int size = min(m_storage.size(), org->m_storage.size());
    //perform the lexicographical comparison
    for (int i = 0; i < size; i++)
      {
        if (m_storage[i] != org->m_storage[i])
          return m_storage[i] < org->m_storage[i];
      }
    //ok, all elements in the comparable range are equal, then
    //return true if this organism is shorter and false otherwise.
    return m_storage.size() < org->m_storage.size();
  }//isLess

/**
 * Creates the copy of the current organism. Do not forget to delete it,
 * once not needed anymore.
 * this->isEqual(makeClone(true), true) == true;
 * this->isEqual(makeClone(true), false) == ?;
 * this->isEqual(makeClone(false), false) == true;
 * @param soft when true only genetic information (chromosome) is copied
 *            to the clone.
 * @return the copy of the current organsim.
 */
template<class T>
GLBaseOrganism* GLOrganismVector<T>::makeClone(bool soft) const
  {
    if (!soft) return new GLOrganismVector<T>(*this);
    GLOrganismVector<T> *organism = new GLOrganismVector<T>(this->getSize());
    organism->copyGeneticData(this);
    return organism;
  }//makeClone

/**
 * Moves the genes from one position to another.
 * @param old_position where the gene is moving from.
 * @param new_position where the gene is moving to.
 */
template<class T>
void GLOrganismVector<T>::moveGene(int old_position, int new_position)
  {
    if (!checkIndex(old_position) || !(checkIndex(new_position)))
          {
            char errormsg[1000];
            sprintf(errormsg, "\nGLOrganismVector::moveGene\n"
                              "One of the indices -- new_position (%i) or "
                              "old_position (%i)"
                              " is outside the organism's size [0, %i)\n",
                              old_position, new_position, getSize());
            throw GLLogicError(errormsg);
          }
    //if both positions are equal -- do nothing.
    if (old_position == new_position) return;
    
    if (old_position < new_position)
      {
        rotate(m_storage.begin() + old_position,
               m_storage.begin() + old_position + 1,
               m_storage.begin() + new_position + 1);
      }
    else
      {
        rotate(m_storage.rbegin() + getSize() - old_position - 1,
               m_storage.rbegin() + getSize() - old_position,
               m_storage.rbegin() + getSize() - new_position);
      }
        
  }//moveGene

/**
 * Sets the whole chromosome. If the organism of the fixed size the
 * chromosome must be == size of the organism. If bounds for the
 * organism are sets checks are made that those bounds are satisfied.
 * @param chromosome vector of genes to be filled by the organism.
 */
template<class T>
void GLOrganismVector<T>::setChromosome(const vector<T>& chromosome)
  {
    //first check if there chromosom satisfies limits on the size of the
    //organism
    if ((m_property_size == GLConstants::FIXED) &&
        (chromosome.size() != m_max_size))
      {
        char errormsg[1000];
        sprintf(errormsg, "\nGLOrganismVector<T>::setChromosome\n"
                          "Size of the chromosome (%i) differs from the "
                          "size of this fixed-size organism (%i)\n",
                          (int)chromosome.size(),
                          (int)m_max_size);
        throw GLLogicError(errormsg);
      }
    if ((m_property_size == GLConstants::NOT_FIXED) &&
        (chromosome.size() > m_max_size))
      {
        char errormsg[1000];
        sprintf(errormsg, "\nGLOrganismVector<T>::setChromosome\n"
                          "Size of the chromosome (%i) exceeds the maximum "
                          "on the size of the organism (%i)\n",
                          (int)chromosome.size(),
                          (int)m_max_size);
        throw GLLogicError(errormsg);
      }
    //If organism has limits on genes -- check, that the chromosome
    //satisfies them
    if (m_property_bounds == GLConstants::BOUNDED_GENES)
      {
        for(unsigned i = 0; i < chromosome.size(); i++)
          {
            if (!checkBounds(chromosome[i]))
              {
                char errormsg[1000];
                sprintf(errormsg, "\nGLOrganismVector<T>::setChromosome\n"
                              "Element %i of the chromosome is outside "
                              "the gene's bounds\n",
                              i);
                throw GLLogicError(errormsg);
              }
          }
     }
    //All ok now -- assign chromosome to storage.
    m_storage = chromosome;
  }//setChromosome

/**
 * Sets the value of the i-th gene.
 * @param index index of the gene.
 * @param gene new value of the gene.
 */
template<class T>
void GLOrganismVector<T>::setGene(int index, const T& gene)
  {
    //first check the index's bounds.
    if (!checkIndex(index))
      {
        char errormsg[1000];
        sprintf(errormsg, "\nGLOrganismVector::setGene\n"
                          "Index = %i"
                          " is outside the organism's size [0, %i)\n",
                          index, getSize());
        throw GLLogicError(errormsg);
      }
    //if new and old genes coincide -- do nothing
    if (gene == getGene(index)) return;
    //check the gene's bounds.
    if (!checkBounds(gene))
      {
        char errormsg[1000];
        sprintf(errormsg, "\nGLOrganismVector::setGene\n"
                          "Passed gene is outside allowed bounds\n");
        throw GLLogicError(errormsg);
      }
    m_storage[index] = gene;
    resetFitness();
  }//setGene

/**
 * Sets the bounds on the organism's genes. Automatically sets
 * property bounds to GLConstants::BoundedGenes.
 * @param gene_min minimum value of the gene.
 * @param gene_max maximum value of the gene.
 * @param correct_organism when true, ensures that after limits were
 *            imposed organism's genes are within the boundaries.
 * @return true if organism was corrected and false otherwise (always
 *         false if correct_organism was false).
 */
template<class T>
bool GLOrganismVector<T>:: setGeneBounds(const T& gene_min,
                   const T& gene_max,
                   bool correct_organism)
  {
    if (gene_min > gene_max)
      {
        char errormsg[1000];
        sprintf(errormsg, "\nGLOrganismVector<T>:: setGeneBounds\n"
                          "\nInconsistency in bounds: "
                          "gene_min > gene_max\n");
        throw GLLogicError(errormsg);
      }
    m_gene_min = gene_min;
    m_gene_max = gene_max;
    m_property_bounds = GLConstants::BOUNDED_GENES;
    if (!correct_organism) return false;
    bool was_corrected = false;
    //loop through all genes and correct them if they go beyound the
    //newly set bounds.
    for(int i = 0; i < getSize(); i++)
      {
        if (m_storage[i] < gene_min)
          {
            m_storage[i] = m_gene_min;
            was_corrected = true;
            continue;
          }
        if (m_storage[i] > gene_max)
          {
            m_storage[i] = m_gene_max;
            was_corrected = true;
          }
      }
    return was_corrected;
  }//setGeneBounds

/**
 * Sets the size-related data of the organism.
 * @param init_size initial size of the organism.
 * @param max_size the maximum number of elements in the organism.
 */
template<class T>
void GLOrganismVector<T>::setSizeData(unsigned init_size,
                                      unsigned max_size)
  {
    //check, that size is >= 0.
    if (init_size < 0)
      {
        char errormsg[1000];
        sprintf(errormsg, "GLOrganismVector<T>::setSizeData\n"
                          "The organism size (%i) should be >= 0\n",
                          init_size);
      }

    //set the maximum size of the organism
    switch (m_property_size)
    {
      case GLConstants::FIXED:
            m_max_size = init_size;
            break;
      case GLConstants::NOT_FIXED:
            if (max_size < init_size)
              {
                char errormsg[1000];
                sprintf(errormsg, "\nGLOrganismVector<T>::setSizeData\n"
                          "Maximum organism size (%i) set to be less than"
                          "initial size (%i)\n",
                          max_size, init_size);
                throw GLLogicError(errormsg);
              }
            m_max_size = max_size;
            break;
    }
  }//setSizeData

/**
 * Shuffles the set of genes in the organism. Genes which are
 * in the positions from the gene_numbers vector will be randomly
 * shuffled. Genes from other positions will stay where they are.
 * @param gene_numbers vector of positions of the genes to be shuffled.
 */
template<class T>
void GLOrganismVector<T>::shuffleGenes(const vector<int>& gene_numbers)
  {
    //get the vector of genes from gene_numbers
    vector<T> genes;
    genes.reserve(gene_numbers.size());
    for(size_t i = 0; i < gene_numbers.size(); i++)
      genes.push_back(getGene(gene_numbers[i]));

    //shuffle these genes.
    gl_shuffle_vector(genes);
    
    //place them back to organisms
    for(size_t i = 0; i < gene_numbers.size(); i++)
      setGene(gene_numbers[i], genes[i]);
  }//shuffleGenes

/**
 * Swaps sections with another organism. After this method is called
 * the current organism should have section of the anotherOrganism, while
 * the latter will have the section of the caller.
 * Therefore, all genes from the caller with numbers leftPos, rightPos - 1
 * inclusive will go to the anotherOrganism, while be replaced with
 * the corresponding genes from anotherOrganism in caller.
 * If rightPos == organism.getSize() the effect is equivalent to swapping
 * tails.
 * leftPos must be < rightPos.
 * @param anotherOrganism organism with which the section will be swapped.
 * @param leftPos the beginning of the section to be swapped (this gene
 *    will be swapped too).
 * @param rightPos the end of the section to be swapped (this gene will NOT
 *    be swapped).
 */
template<class T>
void GLOrganismVector<T>::swapSections(GLBaseOrganism* anotherOrganism, 
                          int leftPos, int rightPos)
  {
    //see, if another organism can be transformed to the current one
    GLOrganismVector<T>* org = nlutils::transform_pointer
        <GLOrganismVector<T>, GLBaseOrganism>(anotherOrganism, 
              "anotherOrganism from GLOrganismVector<T>::swapTails");
    
    //check, that leftPos < rightPos
    if (leftPos >= rightPos)
      {
        char errormsg[1000];
        sprintf(errormsg, "\nGLOrganismVector<T>::swapSections\n"
                          "parameter leftPos (= %i) must < rightPos (= %i)",
                          leftPos, rightPos);
        throw GLLogicError(errormsg);
      }
      
    //check, that start and end positions are within size limits of both
    //organisms (no need to check both position after the check above).
    int maxPos = min(getSize(), org->getSize());
    if ((leftPos < 0) || (rightPos > maxPos))
          {
            char errormsg[1000];
            sprintf(errormsg, "\nGLOrganismVector<T>::swapSections\n"
                    "One of the positions leftPos = %i or rightPos = %i "
                    "is outside limits for the "
                    "organisms [0, %i)",
                    leftPos, rightPos, maxPos);
            throw GLLogicError(errormsg);
          }
    //do the swap
    swap_ranges(m_storage.begin() + leftPos, m_storage.begin() + rightPos, 
         org->m_storage.begin() + leftPos);
  }//swapSections

/**
 * Swaps tails with another organism. After this method is called
 * the current organism should have tail of the anotherOrganism, while
 * the latter will have the tail of the caller.
 * @param anotherOrganisms organism with which the tails will be swapped.
 * @param pos point starting with which the tails will be swapped (including
 *      this position). 
 */
template<class T>
void GLOrganismVector<T>::swapTails(GLBaseOrganism* anotherOrganism, int pos)
  {
    //see, if another organism can be transformed to the current one
    GLOrganismVector<T>* org = nlutils::transform_pointer
        <GLOrganismVector<T>, GLBaseOrganism>(anotherOrganism, 
              "anotherOrganism from GLOrganismVector<T>::swapTails");

    //check, that pos is within size limits of the organisms
    int maxPos = min(getSize(), org->getSize());
    if ((pos < 0) || (pos >= maxPos))
          {
            char errormsg[1000];
            sprintf(errormsg, "\nGLOrganismVector<T>::swapTails\n"
                    "Position pos = %i is outside limits for this "
                    "organisms [0, %i)",
                    pos, maxPos);
          }
    //do the swap
    swap_ranges(m_storage.begin() + pos, m_storage.end(), 
         org->m_storage.begin() + pos);
  }//swapTails

/**
 * Outputs the containt of the organism to output stream.
 * @param out output stream.
*/
template<class T>
ostream& GLOrganismVector<T>::toStream(ostream& out) const
  {
    for(int i = 0; i < getSize(); i++)
      {
        out << getGene(i) << " ";
      }
    return out;
  }//toStream

/**
 * Outputs the containt of the organism to output stream in
 * debug mode.
 * @param out output stream.
 */
template<class T>
ostream& GLOrganismVector<T>::toStreamDebug(ostream& out) const
  {
    out << "Gene bounds property: "
        << GLConstants::toString(m_property_bounds) << endl
        << "Chromosome size property: "
        << GLConstants::toString(m_property_size) << endl
        << "Chromosome size: "
        << m_storage.size() << endl;
    //output the status of Evaluated flag.
    if (isEvaluated())
      {
        out << "Evaluated " << (*getFitness()) << endl;
      }
    else
      {
        out << "Not evaluated\n";
      }
    //output the genes.
    for(int i = 0; i < getSize(); i++)
      {
        out << getGene(i) << " ";
        if ((i + 1) % 20 == 0)
          {
            out << endl;
          }
      }
    if ((getSize() + 1) % 20 != 0)
      {
        out << endl;
      }
    return out;
  }//toStreamDebug
