#ifndef GLORGANISMVECTOR_H_
#define GLORGANISMVECTOR_H_

#include <limits.h>
#include <vector>

#include <NLstl_utils.h>

#include "GLBaseOrganism.h"
#include "GLConstants.h"
#include "GLInterfaceExchangeableGenes.h"
#include "GLInterfaceMoveGene.h"
#include "GLInterfaceShuffleGenes.h"
#include "GLInterfaceSwapSection.h"
#include "GLInterfaceSwapTails.h"

using namespace std;
#pragma pack(push,1)
/**
 * Class based on the 1D vector representation of the
 * organism.
 */
template <class T>
class GLOrganismVector : public GLBaseOrganism,
                         public GLInterfaceExchangeableGenes,
                         public GLInterfaceMoveGene,
                         public GLInterfaceShuffleGenes,
                         public GLInterfaceSwapSection,
                         public GLInterfaceSwapTails
{
public:
  /**
   * Creates organism based on 1D vector representation.
   * @param init_size initial size of the organism. 
   *                  Should be non-negative.
   * @param size_property determines the size variability of the
   *              organism.
   * @param max_size maximum size of the organism (used only when
   *      size_property is != FIXED, otherwise set to init_size).
   *      By default for organism of not_fixed size it will be set to 
   *      init_size. 
   */     
  GLOrganismVector(unsigned init_size,
            GLConstants::TSizeLimits size_property =
              GLConstants::FIXED,
              unsigned max_size = UINT_MAX);
  /**
   * Creates organism based on 1D vector representation.
   * @param chromosome chromosome -- content of the organism.
   * @param size_property determines the size variability of the
   *              organism.
   * @param max_size maximum size of the organism (used only when
   *      size_property is != FIXED, otherwise set to chromosome.size()).
   *      By default for organism of not_fixed size it will be set to 
   *      chromosome.size(). 
   */     
  GLOrganismVector(const vector<T>& chromosome,
            GLConstants::TSizeLimits size_property =
              GLConstants::FIXED,
              unsigned max_size = UINT_MAX);

  /**
   * Copying constructor.
   * @param source source organism.
   */
  GLOrganismVector(const GLOrganismVector<T>& source);
  
  /**
   * Destructor. Destroys everything. 
   */
  virtual ~GLOrganismVector();
  
  /**
   * Operator for accessing genes in vector-like manner.
   * @param i index of the gene to be obtained.
   * @return requested gene. 
   */
  const T& operator[](int i) const
  { return getGene(i); }

  /**
   * Operator for accessing genes in vector-like manner.
   * @param i index of the gene to be obtained.
   * @return requested gene. 
   */
  T& operator[](int i)
  { return getGene(i); }

  /**
   * Checks if the passed pointer is compatible with the current class.
   * If not will throw GLLogicError. Normally this class should be 
   * overloaded in parents.
   * @param source class to check the compatibility with the current.
   * @param message some text to be used in exception. E.g. where 
   *    this function was called from.
   */
  virtual void assertCompatibility(const GLBaseOrganism* source,
                                   const string& message) const;   
  
  /**
   * Checks, that gene is within the organism's bounds.
   * If bounds are not set will always return true.
   * @param gene gene to be checked.
   */
  bool checkBounds(const T& gene)
    {return (m_property_bounds == GLConstants::UNBOUNDED_GENES) ||
            ((m_gene_max >= gene) && (m_gene_min <= gene)); }
  
  /**
   * Checks, that index is withing the organism size's limits.
   * @param index index of the gene in the organism.
   * @return true if index >= 0 and index < getSize().
   */
  virtual bool checkIndex(int index) const
      {return (index >= 0) && (index < getSize()); };

  /**
   * Copies the genetic information from the source organism
   * to the current one (e.g. chromosome, type or the organism,
   * bounds etc.) leaving out fitness information and similar 
   * not-internal data. Normally will reset fitness value.
   * @param source source organism.
  */
  virtual void copyGeneticData(const GLBaseOrganism *source);
      
  /**
   * Exchanges 2 genes withing the organism.
   * @param first index of the first gene.
   * @param second index of the second gene.
   */
  virtual void exchangeGenes(int first, int second);
  
  /**
   * Return the chromosome of the organism -- vector with genetic 
   * information.
   * @return organism's full genetic information.
   */
  const vector<T>& getChromosome() const 
    { return m_storage; }

  /**
  * Returns the single gene of the organism.
  * @param index gene's index.
  * @return single gene.
  */
  T& getGene(int index);
  
  /**
   * Returns the single gene of the organism.
   * @param index gene's index.
   * @return single gene.
   */
  const T& getGene(int index) const;
  
  /**
   * Return the boundaries of the genes. First element in pair is minimum,
   * second is maximum. Note, that if getPropertyBounds == UNBOUNDED_GENE
   * the values in returned structure are undefined.
   * (Why I am not able to return pair<const T&, const T&> ??)
   * @return boundaries imposed on the gene's values.
   */
  pair<const T, const T> getGeneBounds() const
    { return pair<const T, const T>(m_gene_min, m_gene_max); }
	
  /**
   * Returns the property of the organism's size.
   * @return whether organism is fixed size, limited variable size etc.
   */
  GLConstants::TSizeLimits getPropertySize() const
    {return m_property_size;}
  
  /**
   * Return property of the bounds on organism's genes.
   * @return whether organism's genes have bounds or not.
   */
  GLConstants::TGeneBounds getPropertyBounds() const
    { return m_property_bounds; }
  
  /**
   * Returns the current size of the organism (number of genes).
   * @return size of the organism.
   */
  virtual int getSize() const;

  /**
   * Returns true if 2 organisms are equal.
   * @param organism another organism to be compared with.
   * @param soft when true soft comparison is done -- only
   *            gene's value. When false compares all others
   *            values -- fitness fucntion etc. 
   *            Tag's value are never compared.
   */
  virtual bool isEqual(const GLBaseOrganism *organism, 
               bool soft = true) const;
  
  /**
   * Returns true if this organism is less the one passed as a parameter.
   * @param organism another organism to be compared with.
   * @param soft when true soft comparison is done -- based only on
   *            gene's value. When false compares all others
   *            values -- fitness fucntion etc. 
   *            Tag's value are never compared.
   */
  virtual bool isLess(const GLBaseOrganism *organism, 
               bool soft = true) const;
  
  /**
   * Creates the copy of the current organism. Do not forget to delete it,
   * once not needed anymore.
   * this->isEqual(makeClone(true), true) == true;
   * this->isEqual(makeClone(true), false) == ?;
   * this->isEqual(makeClone(false), false) == true; 
   * @param soft when true only genetic information (chromosome) is copied
   *            to the clone.
   * @return the copy of the current organsim.
   */
  virtual GLBaseOrganism* makeClone(bool soft = true) const;

  /**
   * Moves the genes from one position to another.
   * @param old_position where the gene is moving from.
   * @param new_position where the gene is moving to.
   */
  virtual void moveGene(int old_position, int new_position); 
  
  /**
   * Removes bounds for organism's genes.
   */
  void removeGeneBounds()
    { m_property_bounds = GLConstants::UNBOUNDED_GENES; }

  /**
   * Sets the whole chromosome. If the organism of the fixed size the
   * chromosome must be == size of the organism. If bounds for the 
   * organism are sets checks are made that those bounds are satisfied.
   * @param chromosome vector of genes to be filled by the organism.
   */
  void setChromosome(const vector<T>& chromosome);

  /**
   * Sets the property of the organism's size.
   * @param size_limits new property of the organism's size (fixed, or not).
   */
  void setPropertySize(GLConstants::TSizeLimits size_limits)
    { m_property_size = size_limits; }
  
  /**
   * Sets the value of the i-th gene. If fitness function was evaluated --
   * reset it when gene != getGene(index)
   * @param index index of the gene.
   * @param gene new value of the gene.
   */
  void setGene(int index, const T& gene);
  
  /**
   * Sets the bounds on the organism's genes. Automatically sets
   * property bounds to GLConstants::BoundedGenes.
   * @param gene_min minimum value of the gene.
   * @param gene_max maximum value of the gene.
   * @param correct_organism when true, ensures that after limits were
   *            imposed organism's genes are within the boundaries.
   */
  bool setGeneBounds(const T& gene_min, 
                     const T& gene_max,
                     bool correct_organism = true);

  /**
   * Shuffles the set of genes in the organism. Genes which are
   * in the positions from the gene_numbers vector will be randomly
   * shuffled. Genes from other positions will stay where they are.
   * @param gene_numbers vector of positions of the genes to be shuffled.
   */
  virtual void shuffleGenes(const vector<int>& gene_numbers);

  /**
   * Swaps sections with another organism. After this method is called
   * the current organism should have section of the anotherOrganism, while
   * the latter will have the section of the caller.
   * Therefore, all genes from the caller with numbers leftPos, rightPos - 1
   * inclusive will go to the anotherOrganism, while be replaced with
   * the corresponding genes from anotherOrganism in caller.
   * If rightPos == organism.getSize() the effect is equivalent to swapping
   * tails.
   * leftPos must be < rightPos.
   * @param anotherOrganism organism with which the section will be swapped.
   * @param leftPos the beginning of the section to be swapped (this gene
   *    will be swapped too).
   * @param rightPos the end of the section to be swapped (this gene will NOT
   *    be swapped).
   */
  virtual void swapSections(GLBaseOrganism* anotherOrganism, 
                            int leftPos, int rightPos);
  
  /**
   * Swaps tails with another organism. After this method is called
   * the current organism should have tail of the anotherOrganism, while
   * the latter will have the tail of the caller.
   */
  virtual void swapTails(GLBaseOrganism* anotherOrganism, int i);
  
  /**
   * Outputs the containt of the organism to output stream.
   * @param out output stream.
   */
  virtual ostream& toStream(ostream& out) const;

  /**
   * Outputs the containt of the organism to output stream in 
   * debug mode.
   * @param out output stream.
   */
  virtual ostream& toStreamDebug(ostream& out) const;

private:
  /**
   * Sets the size-related data of the organism.
   * @param init_size initial size of the organism.
   * @param max_size the maximum number of elements in the organism.
   */
  virtual void setSizeData(unsigned init_size,
                           unsigned max_size);
private:
  T m_gene_max;   //maximum value of the gene in case of bounded genes.
  T m_gene_min;   //minimum value of the gene in case of bounded genes.
  //indicate, whether genes in the organism have limits on min and max.
  GLConstants::TGeneBounds m_property_bounds;
  //Indicate whether organism's size if fixed, 
  //non-fixed limited or unlimited. 
  GLConstants::TSizeLimits m_property_size;
  unsigned m_max_size; //maximum length of the organism.
protected:
  ///Storage for organism data are stored.
  vector<T> m_storage;  
};
#pragma pack(pop)
#endif /*GLORGANISMVECTOR_H_*/
