#include <iostream>

#include "GLBaseBreeder.h"
#include "GLBaseEvaluator.h"
#include "GLBaseRanker.h"
#include "GLBaseSelectionist.h"
#include "GLBaseVariator.h"
#include "GLPopulationStandard.h"
#include "GLStorageSet.h"

static const int INITIAL_SIZE = 200;

GLPopulationStandard::GLPopulationStandard(bool keep_corpses):
   GLBasePopulation(new GLStorageSet())
  {
    m_offsprings = new GLStorageSet();
    m_mutants = new GLStorageSet();
    m_keep_corpses = keep_corpses;
  }//constructor

GLPopulationStandard::~GLPopulationStandard()
  {
    //free memory and delete storage.
    if (m_offsprings != NULL)
      {
        m_offsprings->deleteAllOrganisms();
        delete m_offsprings;
      }
    //free memory and delete storage.
    if (m_mutants != NULL)
      {
        m_mutants->deleteAllOrganisms();
        delete m_mutants;
      }
    //clean all the corpses.
    clearMorgue();
  }//destructor

/**
 * Create new organisms in population by applying crossover.
 * @param breeder class for producing the offsprings.
 * @param max_offsprings maximum number of children generated.
 *            if <= 0 -- number is limited only by the crossover operator
 *            used.
 * @return number of newly created organisms.
 */
int GLPopulationStandard::breed(GLBaseBreeder *breeder,
                  int max_offsprings)
{
  return breeder->breedOrganisms(this, m_main_storage, m_offsprings,
                        GLConstants::REPLACE,
                        GLConstants::NO_TWINS,
                        max_offsprings);
}//breed
/**
 * Empties morgue deleting all the rejected organisms which
 * were stored there.
 */
void GLPopulationStandard::clearMorgue()
  {
    for(TGLSetOfOrganismsConst::const_iterator it = m_morgue.begin();
        it != m_morgue.end(); ++it)
      {
        delete (*it);
      }
#if DEBUG > 2    
    cout << m_morgue.size() << " was stored in morgue by the end\n";
#endif
    m_morgue.clear();
  }//clearMorgue

/**
 * Evaluates all the organisms in the population. It is up to
 * implementation to decide, whether to evaluate only new organisms or
 * all of them.
 * @param evaluator evaluator operator.
 * @return number of organism which were evaluated (for which evaluation
 *      operator was called).
 */
int GLPopulationStandard::evaluateAll(GLBaseEvaluator* evaluator)
  {
    int evaluated = evaluateOrganisms(m_main_storage, evaluator);
    evaluated += evaluateOrganisms(m_offsprings, evaluator);
    evaluated += evaluateOrganisms(m_mutants, evaluator);
    return evaluated;
  }//evaluateAll

/**
 * Evaluates all the organisms in the container.
 * @param container container with organisms.
 * @param evaluator evaluator operator.
 * @param not_evaluated_only when true evaluator will be applied to
 *            organisms which were not evaluated before.
 */
int GLPopulationStandard::evaluateOrganisms(
                              GLBaseOrganismStorage *container,
                              GLBaseEvaluator* evaluator,
                              bool not_evaluated_only)
  {
    int counter = 0; //number of evaluated organisms.
    //Loop through all organism
    GLBaseOrganismStorage::iterator *it = container->getIterator();
    for (; !(it->isEnd()); it->moveNext())
      {
        //evaluate all organisms or only those, which
        //were not evaluated (according to the flag).
        if (!not_evaluated_only ||
           (not_evaluated_only && !(it->getElement()->isEvaluated())))
          {
            it->getElement()->evaluate(evaluator);
            counter++;
          }
      }
    delete it;
    return counter;
  }//evaluateOrganisms

/**
 * Checks, if organism is really the new one -- check among the
 * current population, newly created mutants, children and in morgue
 * (where rejected organisms are stored) if not empty. Returns true if
 * no identical (with the same chromosome set) organism was found.
 * @param organism organism to be checked for uniqueness.
 * @return true if no identical organisms was found everywhere within
 *    population.
 */
bool GLPopulationStandard::isOrganismTrulyNew(const
                GLBaseOrganism* organism) const
  {
    //first, check among the current population
    if (m_main_storage->findOrganism(organism) != NULL) return false;
    //then among children
    if (m_offsprings->findOrganism(organism) != NULL) return false;
    //then among mutants
    if (m_mutants->findOrganism(organism) != NULL) return false;
    // last, among the corpses
    if (m_morgue.find(organism) != m_morgue.end()) return false;

    //Ok, it is unique
    return true;
  }//isOrganismTrulyNew

/**
 * Forms the new generation out of existing organisms by using the
 * selectionist. The size of the new generation will be new_size or
 * the total number of organisms available, whatever is less.
 * This function normally to be called after methods 'breed', 'mutate'
 * and 'evaluateAll' were called.
 * @param selectionist class to select those to be included in the next
 *            generation.
 * @param best_organisms best organism(s) in newly created generation.
 *            The previous content of this vector will be lost.
 * @param new_size size of the new generation. If < 0 then the size of
 *    the new generation will be == the size of the current one.
 * @return number of NEW organisms (not present in current generation)
 *             in next generation.
 */
int GLPopulationStandard::keepFittest(
                        GLBaseSelectionist* selectionist,
                        vector<const GLBaseOrganism*>& best_organisms,
                        int new_size)
  {
    //adjust the size parameter
    unsigned newsize = (new_size < 0) ? getSize() : new_size;
    //transfer organisms from current population the single vector.
    vector<GLBaseOrganism*> current_population;
    m_main_storage->getOrganismsVector(current_population);

    //transfer newly created organisms to the new vector.
    vector<GLBaseOrganism*> new_generation;
    m_mutants->getOrganismsVector(new_generation);
    m_offsprings->getOrganismsVector(new_generation, GLConstants::APPEND);

    //Create containers for organisms to live and to die.
    vector<GLBaseOrganism*> to_live;
    vector<GLBaseOrganism*> to_die;
    int counter = selectionist->doSelection(current_population,
                              new_generation, newsize, to_live, to_die,
                              best_organisms);

    //fill main storage with organisms to live.
    m_main_storage->fillFromVector(to_live, GLConstants::REPLACE);
    //process discarded elements
    processCorpses(to_die);

    return counter;
  }//keepFittest

/**
 * Mutate the original population and the newborn (it is assumed, that
 * this method is called after Breed).
 * @param variator mutator operator to be applied to the organisms.
 * @param apply_to_newborns when true it can also be applied to the
 *                    newly born children (after Breed function call).
 * @param max_mutants maximum number of mutants generated.
 *            if < 0 -- number is limited only by the mutation algorithm
 *            used.
 * @return number of mutated organisms.
 */
int GLPopulationStandard::mutate(GLBaseVariator *variator,
                   int max_mutants,
                   int mutation_policy)
  {
    //clean storage with mutants first
    m_mutants->emptyStorage();
    int mutated(0);
    //mutate base storage
    if (mutation_policy & GLConstants::MUTATE_PARENTS)
      {
        mutated = variator->applyMutator(this, m_main_storage, m_mutants,
                                         GLConstants::APPEND,
                                         GLConstants::NO_TWINS,
                                         max_mutants);
      }
    //adjust the number of maximum allowed number mutants to
    //take into account already created ones.
    if (max_mutants > 0)
      {
        //need to consider only case when number of mutants is limited
        //(max_mutants > 0)
        max_mutants = max_mutants > mutated ? max_mutants - mutated : 0;
      }
    if (mutation_policy & GLConstants::MUTATE_CHILDREN)
      {
        mutated += variator->applyMutator(this, m_offsprings, m_mutants,
            GLConstants::APPEND,
            GLConstants::NO_TWINS,
            max_mutants);
      }
    return mutated;
  }//mutate

/**
 * Cleans the population and adds new organisms.
 * @param size number of new organisms to be set.
 * @param initialiser factory for organisms production.
 * @param twins when NO_TWINS no duplicates are possible
 *                    in the population.
 */
void GLPopulationStandard::populate(int size,
                      GLBaseOrganismInitialiser* initialiser,
                      GLConstants::TTwinsFlags twins)
  {
    int counter = 0;
    int size_10 = size * 10; //to avoid infinite loop.
    while ((m_main_storage->getSize() != size) && (counter < size_10))
      {
        GLBaseOrganism *org = initialiser->initialiseOrganism();
        //add initialised organism to storage, delete if this was
        //unsuccessful.
        if (!addOrganism(org, twins))
          delete org;
        counter++;
      }
  }//populate

void GLPopulationStandard::prepareForNextGeneration()
  {
    //clear storage for mutants and offsprings
    m_mutants->emptyStorage();
    m_offsprings->emptyStorage();
  }//prepareForNextGeneration

/**
 * Depending on the value of flag isKeepCorpses() either
 * places them into 'morgue' or deletes the organisms and
 * empties the vector.
 * @param corpses vector of organisms to be discarded.
 */
void GLPopulationStandard::processCorpses(vector<GLBaseOrganism*>& corpses)
  {
    if (isKeepCorpses())
      {
        //add corpses one by one (cannot use insert method because of const
        //attrubute in morgue)
        for(size_t i = 0; i < corpses.size(); i++)
          {
            //delete organisms which are already in morgue.
            if (m_morgue.insert(corpses[i]).second == false)
              delete corpses[i];
          }
      }
    else
      {
        //delete all elements from the 'corpses' array
        for(size_t i = 0; i < corpses.size(); i++)
          {
            delete corpses[i];
          }
      }
    corpses.clear();
  }//processCorpses
