#ifndef GLPOPULATIONSTANDARD_H_
#define GLPOPULATIONSTANDARD_H_
#include <set>
#include "GLBaseOrganism.h"
#include "GLBasePopulation.h"

using namespace std;

/**
 * Standard population -- basic real class implementing all the
 * pure virtual functions inherited from the parent.
 */
class GLPopulationStandard : public GLBasePopulation
{
public:
  /**
   * Constructor, creates all the storage containers.
   */
  GLPopulationStandard(bool keep_corpses = GLConstants::CORPSES_KEEP);

  /**
   * Destructor, deletes all the storage containers.
   */
  virtual ~GLPopulationStandard();

  /**
   * Create new organisms in population by applying crossover.
   * @param breeder class for producing the offsprings.
   * @param max_offsprings maximum number of children generated.
   *            if < 0 -- number is limited only by the crossover operator
   *            used.
   * @return number of newly created organisms.
   */
  virtual int breed(GLBaseBreeder *breeder,
                    int max_offsprings = -1);

  /**
   * Empties morgue deleting all the rejected organisms which
   * were stored there.
   */
  void clearMorgue();

  /**
   * Evaluates all the organisms in the population. It is up to
   * implementation to decide, whether to evaluate only new organisms or
   * all of them.
   * @param evaluator evaluator operator.
   * @return number of organism which were evaluated (for which evaluation
   *      operator was called).
   */
 virtual int evaluateAll(GLBaseEvaluator* evaluator);

 /**
  * Returns the flag indicating whether unsuccesfull organisms are
  * destroyed or kept in 'morgue'.
  */
 bool isKeepCorpses() const
 { return m_keep_corpses; }

 /**
  * Checks, if organism is really the new one -- check among the
  * current population, newly created mutants, children and in morgue
  * (where rejected organisms are stored) if not empty. Returns true if
  * no identical (with the same chromosome set) organism was found.
  * @param organism organism to be checked for uniqueness.
  * @return true if no identical organisms was found everywhere within
  *    population.
  */
 virtual bool isOrganismTrulyNew(const GLBaseOrganism* organism) const;

 /**
  * Forms the new generation out of existing organisms by using the
  * selectionist. The size of the new generation will be new_size or
  * the total number of organisms available, whatever is less.
  * This function normally to be called after methods 'breed', 'mutate'
  * and 'evaluateAll' were called.
  * @param selectionist class to select those to be included in the next
  *            generation.
  * @param best_organisms best organism(s) in new generation.
  *            The previous content of this vector will be lost.
  * @param new_size size of the new generation. If < 0 then the size of
  *    the new generation will be == the size of the current one.
  * @return number of NEW organisms (not present in current generation)
  *             in next generation.
  */
  virtual int keepFittest(GLBaseSelectionist* selectionist,
                          vector<const GLBaseOrganism*>& best_organisms,
                          int new_size = -1);

  /**
   * Mutate the original population and the newborn (it is assumed, that
   * this method is called after Breed).
   * @param variator mutator operator to be applied to the organisms.
   * @param mutation_policy determines whether mutator to be applied to 
   *            children and / or to parents.
   * @param max_mutants maximum number of mutants generated.
   *            if < 0 -- number is limited only by the mutation algorithm
   *            used.
   * @return number of mutated organisms.
   */
  virtual int mutate(GLBaseVariator *variator,
                     int max_mutants = -1,
                     int mutation_policy = 
                       GLConstants::MUTATE_CHILDREN |
                       GLConstants::MUTATE_PARENTS);

  /**
   * Cleans the population and adds new organisms.
   * @param size number of new organisms to be set.
   * @param initialiser factory for organisms production.
   * @param twins when NO_TWINS no duplicates are possible
   *                    in the population.
   */
  virtual void populate(int size,
                        GLBaseOrganismInitialiser* initialiser,
                        GLConstants::TTwinsFlags twins =
                          GLConstants::NO_TWINS);

  /**
   * Last function to be called in create new generation. Deletes temporary
   * arrays etc.
   */
  virtual void prepareForNextGeneration();

  /**
   * Depending on the value of flag isKeepCorpses() either
   * places them into 'morgue' or deletes the organisms and
   * empties the vector.
   * @param corpses vector of organisms to be discarded.
   */
  virtual void processCorpses(vector<GLBaseOrganism*>& corpses);

  /**
   * Sets the policy, whether to keep expand the morgue with new arrivals
   * or not. Note, that when 'false' is passed, new corpses are not added
   * to the morgue, but it is not emptied. Use method clearMorgue for this.
   * @param new_flag new value of the flag which determines the policy
   *    regarding the corpses.
   */
  void setKeepCorpses(bool new_flag)
    { m_keep_corpses = new_flag; }
protected:
  /**
   * Evaluates all the organisms in the container.
   * @param container container with organisms.
   * @param evaluator evaluator operator.
   * @param not_evaluated_only when true evaluator will be applied to
   *            organisms which were not evaluated before.
   * @return the number of evaluated organisms.
   */
  virtual int evaluateOrganisms(GLBaseOrganismStorage *container,
                                GLBaseEvaluator* evaluator,
                                bool not_evaluated_only = true);
protected:
  ///here the children are stored.
  GLBaseOrganismStorage *m_offsprings; 
  ///here the mutants are stored.
  GLBaseOrganismStorage *m_mutants;    

  ///when true all the discarded organisms
  ///are kept there instead of to be discarded.
  bool m_keep_corpses;

  ///here all the discarded organisms are to be kept
  TGLSetOfOrganismsConst m_morgue;
};

#endif /*GLPOPULATIONSTANDARD_H_*/
