#include <iostream>
#include <time.h>
#include "GLLogicError.h"
#include "GLRandomNumbersGenerator.h"

bool GLRandomNumbersGenerator::m_is_initialised = false; 
long GLRandomNumbersGenerator::m_initial_seed = UNINITIALISED_SEED;    

  
  /**
   * Generates the random integer number. If random generator
   * was not initialised -- initialised it with timer value.
   */
  long GLRandomNumbersGenerator::getRandomInteger()
    {
      if (!m_is_initialised) initGenerator();
      return rand();
    }//getRandomInteger
  
  /**
   * Returns random number between 0 and max.
   * @param max maximal possible random value.
   * @return random num: (num >= 0) && (num < max) 
   */
  long GLRandomNumbersGenerator::getRandomInteger(
          const long& max)
    {
      //check, that max > 0;
      if (max <= 0)
        {
          char errormsg[1000];
          sprintf(errormsg, "GLRandomNumbersGenerator::getRandomInteger"
                            "Max should be > 0 while %li was provided",
                            max);
          throw GLLogicError(errormsg);
        }
      if (max > getMaxNumber())
        {
          char errormsg[1000];
          sprintf(errormsg, "GLRandomNumbersGenerator::getRandomInteger"
                            "Max should be < %li while %li was provided",
                            getMaxNumber(), max);
          throw GLLogicError(errormsg);
        }
      long int r = getRandomInteger();
      return (r % max);
    }//getRandomInteger
 
  /**
   * Returns random number between 2 values.
   * @param min minimal possible random value.
   * @param max maximal possible random value.
   * @return random num: (num >= left) && (num < right) 
   */
  long GLRandomNumbersGenerator::getRandomInteger(
              const long& min, const long& max)
    {
      //first check that left < right
      if (min >= max)
        {
          char errormsg[1000];
          sprintf(errormsg, "GLRandomNumbersGenerator::getRandomInteger"
                            "min (=%li) should be < max (= %li)\n",
                            min, max);
          throw GLLogicError(errormsg);
        }
      return getRandomInteger(max - min) + min;
    }//getRandomInteger
 
  /**
   * Creates the vector of random integer numbers within the given
   * limits. If flag mayRepeat is set to false then elements of vector
   * will be unique (not repeating). Note, that in this case max - min
   * should be >= size, otherwise there are not enough elements to fill
   * the vector. Note, that previous contain of the vector will be lost.
   * @param result resulting vector of random integer.
   * @param min minimum possible value for integer.
   * @param max maximum possible value for integer.
   * @param size size of the vector (number of random integers to 
   *    be generated).
   * @param mayRepeat when true false elements of results vector 
   *            should be unique.
   */
//  static void getRandomIntegerVector(
//            vector<int>& result, 
//            const long &min,
//            const long &max,
//            const long& size,
//            bool mayRepeat)
//    {
//     //first, check that parameters are not contradictory.
//      if (min >= max)
//        {
//          char errormsg[1000];
//          sprintf(errormsg, "GLRandomNumbersGenerator::"
//                            "getRandomIntegerVector"
//                            "min (=%li) should be < max (= %li)\n",
//                            min, max);
//          throw GLLogicError(errormsg);
//        }
//      if ((!mayRepeat) && (max-min < size))
//        {
//          char errormsg[1000];
//          sprintf(errormsg, "GLRandomNumbersGenerator::"
//                            "getRandomIntegerVector"
//                            "Number of random elements to generate (%i) "
//                            "should be < max - min (=%li)\n",
//                             size, max - min);
//          throw GLLogicError(errormsg);
//        }
//      //Ok, clean the vector
//      result.clean();
//      result.resize(size);
//      for(int i = 0; i < size; i++)
//        {
//          result[i] = getRandomInteger(min, max);
//        }
//    }//getRandomIntegerVector
  

  /**
   * Returns uniformly distributed double from the given range.
   * @param min minimal possible random value.
   * @param max maximal possible random value.
   */
  double GLRandomNumbersGenerator::getRandomUniform(
                const double& min, 
                const double& max)
    {
      //first check that left < right
      if (min >= max)
        {
          char errormsg[1000];
          sprintf(errormsg, "GLRandomNumbersGenerator::getRandomUniform"
                            "min (=%e) should be < max (= %e)\n",
                            min, max);
          throw GLLogicError(errormsg);
        }
      double length = max - min;
      return min + length * 
               double(getRandomInteger()) / double(getMaxNumber());
    }//getRandomUniform
  
  /**
   * Initialised random generator with given positive
   * value or (if UNITIALISED_SEED was passed) with the timer.
   * Returns the value which was used to initialised generator.
   * Note, that if the same value was used to start generator
   * it will produce the same sequence of random values.
   * @param seed seed to start the random generator.
   * @return the integer which was actually used to start the generator.
   */
  long GLRandomNumbersGenerator::initGenerator(long seed)
    {
      if (seed != UNINITIALISED_SEED) m_initial_seed = seed;
      else m_initial_seed = time(NULL);
      cout << "Initialising the random generator with " 
        << m_initial_seed << endl;
      srand(m_initial_seed);
      m_is_initialised = true;
      return m_initial_seed;
    }//initGenerator


 /**
  * Returns the shuffled vector of integers between minimum and maximum
  * values. The size of the resulting vector will be max-min.
  * @param result vector with the ineger data (pevious content will be lost).
  * @param minimal integer value (included).
  * @param maximal integer value (not included).
 */
void GLRandomNumbersGenerator::shuffleIntegerVector(
                vector<int>& result,
                const long &min,
                const long &max)      
  {
      //first, check that parameters are not contradictory.
      if (min >= max)
      {
        char errormsg[1000];
        sprintf(errormsg, "GLRandomNumbersGenerator::"
                         "GLRandomNumbersGenerator::shuffleIntegerVector"
                         "min (=%li) should be < max (= %li)\n",
                          min, max);
        throw GLLogicError(errormsg);
      }
      //fill the vector with integers in order
      int size = max - min;
      int size_m1 = size - 1;
      result.clear();
      result.resize(size);
      for(int i = min, j = 0; i < max; i++, j++)
        {
          result[j] = i;
        }
      //shuffle elements in vector.
      for(int i = 0; i < size_m1; i++)
        {
          //choose random integer between i and size
          //[i; size)
          int j = getRandomInteger(i, size);
          //swap i-th and j-th elements of the results vector.
          if (i == j) continue;
          int tmp = result[j];
          result[j] = result[i];
          result[i] = tmp;
        }
  }//shuffleIntegerVector
