#ifndef GLRANDOMNUMBERSGENERATOR_H_
#define GLRANDOMNUMBERSGENERATOR_H_
#include <stdlib.h>
#include <vector>
using namespace std;

/**
 * Class to generate random numbers. By default uses
 * standard stdlib rand() function. Can be initialised with particular 
 * seed or will do it using the timer value. 
 * 
 * Each time when generator initialised with the same seed it will produce 
 * the same sequence of random integer numbers.
 */
class GLRandomNumbersGenerator
{
public:
  /**
   * Constants specific for use in random generator.
   */
  enum GeneratorConstants
  {
    UNINITIALISED_SEED = -1
  };
  
  /**
   * Destructor. Actually does nothing.
   */
  virtual ~GLRandomNumbersGenerator() {};
  
  /**
   * Returns the maximum integer number which can be randomly generated.
   */
  static long getMaxNumber() { return RAND_MAX; }
  /**
   * Generates the random integer number. If random generator
   * was not initialised -- initialised it with timer value.
   */
  static long getRandomInteger();
  
  /**
   * Returns random number between 0 and max.
   * @param max maximal possible random value.
   * @return random num: (num >= 0) && (num < max) 
   */
  static long getRandomInteger(const long& max);
  
  /**
   * Returns random number between 2 values.
   * @param min minimal possible random value.
   * @param max maximal possible random value.
   * @return random num: (num >= min) && (num < max) 
   */
  static long getRandomInteger(const long& min, const long& max);

  /**
   * Creates the vector of random integer numbers within the given
   * limits. If flag mayRepeat is set to false then elements of vector
   * will be unique (not repeating). Note, that in this case max - min
   * should be >= size, otherwise there are not enough elements to fill
   * the vector. Note, that previous contain of the vector will be lost.
   * @param result resulting vector of random integer.
   * @param min minimum possible value for integer.
   * @param max maximum possible value for integer.
   * @param size size of the vector (number of random integers to 
   *    be generated).
   * @param mayRepeat when true false elements of results vector 
   *            should be unique.
   */
//  static void getRandomIntegerVector(
//            vector<int>& result, 
//            const long &min,
//            const long &max,
//            const long& size,
//            bool mayRepeat = false);

  /**
   * Returns uniformly distributed double from the given range.
   * @param left minimal possible random value.
   * @param right maximal possible random value.
   */
  static double getRandomUniform(const double& left, 
                                 const double& right);
  
  /**
   * Returns seed which was used to run the random generator.
   * @return seed which was used to start the random generator. If
   *    not initialised yet will return 
   *    GLRandomNumbersGenerator::UNITIALISED_SEED. 
   */
  static long getSeed() {return m_initial_seed; }
  
  /**
   * Initialised random generator with given positive
   * value or (if UNITIALISED_SEED was passed) with the timer.
   * Returns the value which was used to initialised generator.
   * Note, that if the same value was used to start generator
   * it will produce the same sequence of random values.
   * @param seed positive number to start the random generator or
   *    UNINITIALISED_SEED (default) to use timer.
   * @return the integer which was actually used to start the generator.
   */
  static long initGenerator(long seed = UNINITIALISED_SEED);

  /**
   * Returns true if the generator was initialised with the seed
   * and false otherwise.
   * @return the indicator if the random generator was initialised.
   */
  static int isInitialised() {return m_is_initialised; }

  /**
   * Returns the shuffled vector of integers between minimum and maximum
   * values. The size of the resulting vector will be max-min.
   * @param result vector with the integer data (pevious content will 
   *    be lost).
   * @param min minimal integer value.
   * @param max maximal integer value.
   */
  static void shuffleIntegerVector(vector<int>& result,
                const long &min,
                const long &max);      
private:
  /**
   * Hidden constructor to make it singleton class.
   */
   GLRandomNumbersGenerator(){};
private:
  ///set to true if class is initialised
  ///and to false otherwise.
  static bool m_is_initialised; 
  ///initial seed for randomizer.
  static long m_initial_seed;    
};

#endif /*GLRANDOMNUMBERSGENERATOR_H_*/
