#include <algorithm>
#include <NLstl_utils.h>
#include "GLBaseOrganism.h"
#include "GLRankerStandardSO.h"
#include "GLUtils.h"
using namespace std;
using namespace std::rel_ops;

/**
 *Simple constructor
 */
GLRankerStandardSO::GLRankerStandardSO()
{
}

/**
 * Simple destructor.
 */
GLRankerStandardSO::~GLRankerStandardSO()
{
}

/**
 * Returns the fittest organism(s) from the vector. The criteria for
 * fittness should be the same which are used in 'rankOrganism'. Will return
 * all the organisms will the same best fittness value. It is assumed,
 * that all organisms in vector are evaluated.
 * @param organisms vector of organisms.
 * @param fittest returned vector of fittest organisms. Previous content
 *    of the container will be erased.
 */
void GLRankerStandardSO::getFittest(
                        const vector<GLBaseOrganism*>& organisms,
                        vector<const GLBaseOrganism*>& fittest)
  {
    fittest.clear();

    //if empty - return immediately.
    if (organisms.empty()) return;
    vector<GLBaseOrganism*>::const_iterator it = organisms.begin();
    fittest.push_back(*it);
    ++it;  //skip the first organism, as we just pushed it in. 
    while (it != organisms.end())
      {
        //if fitter organism found -- replace all previously found
        //organisms
        if ((*it)->getFitness()->isLess(fittest.front()->getFitness()))
          {
            fittest.clear();
            fittest.push_back((*it));
          }
        //if organism of the same fittness found -- add it.
        else if (fittest.front()->getFitness()->isEqual((*it)->getFitness()))
          {
            fittest.push_back(*it);
          }
        ++it;
      }
  }//getFittest

/**
 * Takes the vector of pointers to the organisms and
 * re-arrange them in ascending order of fittness.
 * @param organisms vector of organisms to be ranked.
 * @param fittest the fittest organism(s) among just ranked. The previous
 *    content of this vector will be lost. If there are several best
 *    organisms (with equal value of the fitness function) they all will be
 *    returned in the array.
 */
void GLRankerStandardSO::rankOrganisms(
      vector<GLBaseOrganism*>& organisms,
      vector<const GLBaseOrganism*>& fittest)
  {
    rankOrganisms(organisms);
    //fill the vector of fittest organisms
    fittest.clear();
    fittest.push_back(organisms.front());
    vector<GLBaseOrganism*>::const_iterator it = organisms.begin() + 1;
    while ((it != organisms.end()) &&
           ((*it)->getFitness()->isEqual(organisms.front()->getFitness())))
      {
        fittest.push_back(*it);
        ++it;
      }
  }//rankOrganisms

/**
 * Takes the vector of pointers to the organisms and
 * re-arrange them in ascending order of fittness.
 * @param organisms vector of organisms to be ranked.
 */
void GLRankerStandardSO::rankOrganisms(vector<GLBaseOrganism*>& organisms)
  {
    sort(organisms.begin(), organisms.end(),
         GLBaseOrganism::CompareFitnessLess());
  }//rankOrganisms

/**
 * Compares currently best organisms with candidates and, if necessary, amends
 * the current best organisms. Normally to be used for comparing all-time
 * best organisms with the best organisms in current generation.
 * Note, that "current best" organisms are pointers to independent "clones"
 * while candidates point to current population.
 * For Single Objective ranker all the organisms in each array should have
 * the same fittness, so it is just a matter of comparing the first 
 * elements of each array and return the best (or merge two arrays if
 * they have the same fittness). 
 * @param candidates the vector of candidates.
 * @param current_best currently most fittest organisms.
 * @return true if vector of best organisms was amended and false otherwise.
*/
bool GLRankerStandardSO::updateFittest(
          const vector<const GLBaseOrganism*>& candidates,
                vector<const GLBaseOrganism*>& current_best)
  {
    //first check, if candidates arrays is empty
    if (candidates.empty()) return false;
    //if current best is empty or fitness of elements there is worse then 
    //fitness of candidates -- transfer elements from candidates

    if (current_best.empty() || 
        candidates[0]->getFitness()->isLess(current_best[0]->getFitness()))
      {
        for_each(current_best.begin(), current_best.end(), 
                 nlutils::DeleteObject());
        current_best.clear();
        current_best.reserve(candidates.size());
        transform (candidates.begin(), candidates.end(),
                   back_inserter(current_best),
                   GLBaseOrganism::Cloner(false));
        return true;
      }
    //if fitness of organisms in both vectors are the same we need to add
    //new elements from candidates which are not in current best.
    bool return_flag = false;
    if (candidates[0]->getFitness()->
          isEqual(current_best[0]->getFitness()))
      {
        for(vector<const GLBaseOrganism*>::
              const_iterator it = candidates.begin(); it != candidates.end();
              ++it)
          {
            //check if candidate is in the current best already
            vector<const GLBaseOrganism*>::iterator it_cur =
              find_if(current_best.begin(), current_best.end(),
                      bind2nd(GLBaseOrganism::CompareEqual(), (*it)));
            //if not found -- add it
            if (it_cur == current_best.end())
              {
                current_best.push_back((*it)->makeClone(false));
                return_flag = true;
              }
          }
      }
    return return_flag;
  }//updateFittest
