#ifndef GLRANKERSTANDARDSO_H_
#define GLRANKERSTANDARDSO_H_
#include <iostream>

#include "GLBaseRanker.h"

/**
 * Standard ranker for single objective optimization -- ranks organisms
 * in ascending order according to the fitness value.
 */

class GLRankerStandardSO : public GLBaseRanker
{
public:
  /**
   *Simple constructor
   */
  GLRankerStandardSO();

  /**
   * Simple destructor.
   */
  virtual ~GLRankerStandardSO();

  /**
   * Returns the fittest organism(s) from the vector. The criteria for
   * fittness should be the same which are used in 'rankOrganism'. Will return
   * all the organisms will the same best fittness value. It is assumed,
   * that all organisms in vector are evaluated.
   * @param organisms vector of organisms.
   * @param fittest returned vector of fittest organisms. Previous content
   *    of the container will be erased.
   */
  virtual void getFittest(const vector<GLBaseOrganism*>& organisms,
                          vector<const GLBaseOrganism*>& fittest);

  /**
   * Takes the vector of pointers to the organisms and
   * re-arrange them in ascending order of fittness.
   * @param organisms vector of organisms to be ranked.
   * @param fittest the fittest organism(s) among just ranked. The previous
   *    content of this vector will be lost. If there are several best
   *    organisms (with equal value of the fitness function) they all will be
   *    returned in the array.
   */
  virtual void rankOrganisms(
        vector<GLBaseOrganism*>& organisms,
        vector<const GLBaseOrganism*>& fittest);

  /**
   * Takes the vector of pointers to the organisms and
   * re-arrange them in ascending order of fittness.
   * @param organisms vector of organisms to be ranked.
   */
  virtual void rankOrganisms(vector<GLBaseOrganism*>& organisms);

  /**
   * Compares currently best organisms with candidates and, if necessary, 
   * amends the current best organisms. Normally to be used for comparing 
   * all-time best organisms with the best organisms in current generation.
   * Note, that "current best" organisms are pointers to independent "clones"
   * while candidates point to current population.
   * For Single Objective ranker all the organisms in each array should have
   * the same fittness, so it is just a matter of comparing the first 
   * elements of each array and return the best (or merge two arrays if
   * they have the same fittness). 
   * @param candidates the vector of candidates.
   * @param current_best currently most fittest organisms.
   * @return true if vector of best organisms was amended and false otherwise.
  */
  virtual bool updateFittest(
            const vector<const GLBaseOrganism*>& candidates,
                  vector<const GLBaseOrganism*>& current_best);

}; //GLRankerStandardSO

#endif /*GLRANKERSTANDARDSO_H_*/
