#include "GLBaseRanker.h"
#include "GLLogicError.h"
#include "GLSelectionistMixer.h"

/**
  * Constructor. Assigns the operator which will be used for ranking
  * the organisms. Note, that the ranker will be destroyed when destructor
  * is called.
  * @param ranker operator for arranging the organisms in accordance
  *    with their rank.
  * @param proportion_old number between 0 and 1 inclusive which determines
  *   how many organisms from the old generation will be passed to the new
  *   one. In any new generation of size S the S * proportion_old organism
  *   will come from the old generaion and S * (1 - proportion_old)
  *   organisms will be new.
 */
GLSelectionistMixer::GLSelectionistMixer(GLBaseRanker* ranker,
                      const double& proportion_old):
                        GLBaseSelectionist(ranker)
  {
    setProportionOld(proportion_old);
  }//constructor


/**
  * Constructor. Assigns the operator which will be used for ranking
  * the organisms. Note, that the ranker will be destroyed when destructor
  * is called.
  * @param ranker operator for arranging the organisms in accordance
  *    with their rank.
  * @param parameters parameters specific for this selectionist.
  */
GLSelectionistMixer::GLSelectionistMixer(GLBaseRanker* ranker,
                    const GLSelectionistMixer::TParameters* parameters):
                      GLBaseSelectionist(ranker)
  {
    setProportionOld(parameters->t_proportion_old);
  }//constructor

/**
 * Destructor. Destroys everything, including ranker.
 */
GLSelectionistMixer::~GLSelectionistMixer()
{
}//destructor

/**
 * Choose which organism to leave and which one to die in the next
 * generation. It is assumed, that all organisms passed as input parameters
 * were evaluated already.
 * The algorithm is as following: if the number of organism in the next
 * generation should be S then keep S*getProportionOld() best organisms
 * from the old generation and mix them with S * (1 - getProportionOld())
 * new organisms.
 * If there is not enough organisms in either set then more organisms from
 * the opposit set will be added. If number of organisms in both generation
 * is too small then all of them will be part of the new generation.
 * @param current_population organisms from the this generation's
 *            population.
 * @param candidates newly created organisms (e.g. mutants and/or
 *            children). It is desirable, that both arrays have not
 *            common organisms.
 * @param new_size size of the next generation.
 * @param to_live output array of organisms to live  (will be cleaned).
 * @param to_die output array of organisms to die in the next generation
 *     (will be cleaned).
 * @param best_survived best survived organism(s). May be single organism
 *     for single objective optimisation or the whole Pareto-front for
 *     multi-objective optimisation. The previous content of this
 *     array will be lost.
 * @return number of NEW organisms (not present in current generation)
 *             in next generation.
 */
int GLSelectionistMixer::doSelection(
                    const vector<GLBaseOrganism*>& current_population,
                    const vector<GLBaseOrganism*>& candidates,
                    size_t new_size,
                    vector<GLBaseOrganism*>& to_live,
                    vector<GLBaseOrganism*>& to_die,
                    vector<const GLBaseOrganism*>& best_survived)
  {
    //check, that new_size parameter is viable
    assertNewSize(new_size, current_population, candidates);

    //Rank old organisms.
    vector<GLBaseOrganism*> current_sorted(current_population);
    getRanker()->rankOrganisms(current_sorted);

    //Rank new organisms.
    vector<GLBaseOrganism*> candidates_sorted(candidates);
    getRanker()->rankOrganisms(candidates_sorted);

    //decide, how many old and new organisms will form the new generation
    size_t num_old = (size_t)(new_size * getProportionOld());

    //check if current generation has enough organisms.
    if (num_old > current_sorted.size())
      {
        num_old = current_sorted.size();
      }

    size_t num_new = new_size - num_old;
    //check, if there are enough candidates (note, that total number of
    //new and old must be >= new_size because of assertion made above).
    if (num_new > candidates_sorted.size())
      {
        num_new = candidates_sorted.size();
        num_old = new_size - num_new;
      }
    //split the result between to_live and to_die.
    to_live.reserve(new_size);
    to_die.reserve(current_sorted.size() +
                   candidates_sorted.size() -
                   new_size);

    to_live.assign(current_sorted.begin(),
                   current_sorted.begin() + num_old);
    to_live.insert(to_live.end(),
                   candidates_sorted.begin(),
                   candidates_sorted.begin() + num_new);

    to_die.assign(current_sorted.begin() + num_old, current_sorted.end());
    to_die.insert(to_die.end(),
                  candidates_sorted.begin() + num_new,
                  candidates_sorted.end());

    //fill best survived organism
    getRanker()->getFittest(to_live, best_survived);

    //return number of survived candidates
    return num_new;
  }//doSelection


/**
 * Sets the new value for the proportion of old organisms in the
 * new generation.
 * @param new_proportion_old new value for the proportion of old organisms
 *    in the new generation. Must be between 0 and 1.0 inclusive.
 */
void GLSelectionistMixer::setProportionOld(const double& new_proportion_old)
  {
    //check, that new value is within the acceptable limits.
    if ((new_proportion_old < 0) || (new_proportion_old > 1))
      {
        char errormsg[1000];
        sprintf(errormsg, "\nGLSelectionistMixer::setProportionOld\n"
                  "Parameter new_proportion_old (=%e) must be between "
                  "0.0 and 1.0 inclusive\n",
                  new_proportion_old);
        throw GLLogicError(errormsg);
      }
    m_proportion_old = new_proportion_old;
  }//setProportionOld
