#ifndef GLSELECTIONISTMIXER_H_
#define GLSELECTIONISTMIXER_H_

#include "GLBaseParameters.h"
#include "GLBaseSelectionist.h"

class GLBaseRanker;
/**
 * Creates the new generation of organisms by keeping the best part of the
 * current generation and mixing in the best part from the generation of
 * children.
 */
class GLSelectionistMixer : public GLBaseSelectionist
{
public:
  class TParameters;
  /**
    * Constructor. Assigns the operator which will be used for ranking
    * the organisms. Note, that the ranker will be destroyed when destructor
    * is called.
    * @param ranker operator for arranging the organisms in accordance
    *    with their rank.
    * @param proportion_old number between 0 and 1 inclusive which determines
    *   how many organisms from the old generation will be passed to the new
    *   one. In any new generation of size S the S * proportion_old organism
    *   will come from the old generaion and S * (1 - proportion_old)
    *   organisms will be new.
   */
  GLSelectionistMixer(GLBaseRanker* ranker,
                      const double& proportion_old = 0.2);

  /**
    * Constructor. Assigns the operator which will be used for ranking
    * the organisms. Note, that the ranker will be destroyed when destructor
    * is called.
    * @param ranker operator for arranging the organisms in accordance
    *    with their rank.
    * @param parameters parameters specific for this selectionist.
    */
  GLSelectionistMixer(GLBaseRanker* ranker,
                      const GLSelectionistMixer::TParameters* parameters);
  
  /**
   * Destructor. Destroys everything, including ranker.
   */
  virtual ~GLSelectionistMixer();

  /**
   * Choose which organism to leave and which one to die in the next
   * generation. It is assumed, that all organisms passed as input parameters
   * were evaluated already.
   * The algorithm is as following: if the number of organism in the next
   * generation should be S then keep S*getProportionOld() best organisms
   * from the old generation and mix them with S * (1 - getProportionOld())
   * new organisms.
   * If there is not enough organisms in either set then more organisms from
   * the opposit set will be added. If number of organisms in both generation
   * is too small then all of them will be part of the new generation.
   * @param current_population organisms from the this generation's
   *            population.
   * @param candidates newly created organisms (e.g. mutants and/or
   *            children). It is desirable, that both arrays have not
   *            common organisms.
   * @param new_size size of the next generation.
   * @param to_live output array of organisms to live  (will be cleaned).
   * @param to_die output array of organisms to die in the next generation
   *     (will be cleaned).
   * @param best_survived best survived organism(s). May be single organism
   *     for single objective optimisation or the whole Pareto-front for
   *     multi-objective optimisation. The previous content of this
   *     array will be lost.
   * @return number of NEW organisms (not present in current generation)
   *             in next generation.
   */
  virtual int doSelection(
                      const vector<GLBaseOrganism*>& current_population,
                      const vector<GLBaseOrganism*>& candidates,
                      size_t new_size,
                      vector<GLBaseOrganism*>& to_live,
                      vector<GLBaseOrganism*>& to_die,
                      vector<const GLBaseOrganism*>& best_survived);
  /**
   * Returns the current settings for the proportion of old organisms in the
   * new generation.
   * @return current value for the proportion of old organisms in the new
   *    generation.
   */
  double getProportionOld() const
  { return m_proportion_old; }

  /**
   * Sets the new value for the proportion of old organisms in the
   * new generation.
   * @param new_proportion_old new value for the proportion of old organisms
   *    in the new generation. Must be between 0 and 1.0 inclusive.
   */
  void setProportionOld(const double& new_proportion_old);
  
  /**
   * Internal class of the additional (to base breeder) parameters
   * necessary to initialise this class.
   */
  struct TParameters: public GLBaseParameters
  {
    
    /// Constructor, initialises parameters.
    /// @param proportion_old proportion of old organisims to be kept
    /// in new generation. Must be between 0 and 1 inclusive.
    TParameters(const double& proportion_old):
          t_proportion_old(proportion_old) {};
    ///Copying constructor
    TParameters(const TParameters* source):
        t_proportion_old(source->t_proportion_old){}
    
    ///Creates copy.
    ///@return pointer to identical copy of current class instance.
    virtual TParameters* copy() const
    { return new TParameters(this); }
    
    ///proportion of old organisms to be kept in new generation.
    double t_proportion_old;  
  };//TParameters
private:
  double m_proportion_old;
};

#endif /*GLSELECTIONISTMIXER_H_*/
