#include "GLBaseOrganism.h"
#include "GLBaseRanker.h"
#include "GLLogicError.h"
#include "GLSelectionistStandard.h"
using namespace std;

/**
 * Choose which organism to leave and which one to die in the next
 * generation. It is assumed, that all organisms passed as input parameters
 * were evaluated already.
 * @param current_population organisms from the this generation's
 *            population.
 * @param candidates newly created organisms (e.g. mutants and/or
 *            children). It is desirable, that both arrays have not
 *            common organisms.
 * @param new_size size of the next generation.
 * @param to_live output array of organisms to live (will be cleaned).
 * @param to_die output array of organisms to die in the next generation
 *      (will be cleaned).
 * @param best_survived best survived organism(s). May be single organism
 *     for single objective optimisation or the whole Pareto-front for
 *     multi-objective optimisation. The previous content of this
 *     array will be lost.
 * @return number of NEW organisms (not present in current generation)
 *             in next generation.
 */
int GLSelectionistStandard::doSelection(
                    const vector<GLBaseOrganism*>& current_population,
                    const vector<GLBaseOrganism*>& candidates,
                    size_t new_size,
                    vector<GLBaseOrganism*>& to_live,
                    vector<GLBaseOrganism*>& to_die,
                    vector<const GLBaseOrganism*>& best_survived)
  {
     //check, that new_size parameter is viable
     assertNewSize(new_size, current_population, candidates);
     //Merge new and old organisms to single pool.
     vector<GLBaseOrganism*> all_organisms(current_population);
     all_organisms.insert(all_organisms.end(),
               candidates.begin(), candidates.end());

     //sort the resulting vector.
     getRanker()->rankOrganisms(all_organisms, best_survived);

     //split the result between to_live and to_die.
     to_live.assign(all_organisms.begin(), all_organisms.begin() + new_size);
     to_die.assign(all_organisms.begin() + new_size, all_organisms.end());

     //if number of best survived organisms > then total number of survived
     //organisms -- reduce it.
     if (best_survived.size() > new_size)
       {
         best_survived.erase(best_survived.begin() + new_size);
       }
     return countSurvivedCandidates(to_live, to_die,
                                    current_population, candidates);
  }//doSelection
