#include <math.h>
#include <iomanip>

#include <NLstl_utils.h>

#include "GLBaseGaStatus.h"
#include "GLStatisticsSimple.h"

typedef vector<GLBaseGaStatus*> TResultVector;
/**
 * Constructor. Collects all the relevant statistics from the vector
 * of results. Vector should contains the results of multiple runs of
 * the GA with the same parameters. This class collects only GA-run related
 * statistics (e.g. number of generations per run) and not results
 * statistics (e.g. average/best fittness value etc.).
 * @param results vector of results of GA runs.
 */
GLStatisticsSimple::GLStatisticsSimple(
                        const vector<GLBaseGaStatus*>& results):
      m_number_of_runs(results.size()),
      m_minmax_evaluator_calls(0, 0),
      m_minmax_generations_to_find_best(0, 0),
      m_minmax_generations_to_find_all_bests(0, 0),
      m_minmax_number_of_generations(0, 0),
      m_minmax_time(0.0, 0.0),
      m_stat_evaluator_calls(0.0, 0.0),
      m_stat_generations_to_find_best(0.0, 0.0),
      m_stat_generations_to_find_all_bests(0.0, 0.0),
      m_stat_number_of_generations(0.0, 0.0),
      m_stat_time(0.0, 0.0)
  {
    //do nothing if number of runs is zero.
    if (results.empty()) return;

    collectGaStatistics(results);
  }//constructor

/**
 * Simple destructor.
 */
GLStatisticsSimple::~GLStatisticsSimple()
  {
  }//destructor

/**
 * Collects all the GA statistics from the vector of results.
 * @param results vector with the results of GA runs.
 */
void GLStatisticsSimple::collectGaStatistics(
          const vector<GLBaseGaStatus*>& results)
  {
    //calculate an average and deviation for all parameters
    if (results.empty()) return;
    
    //init minimum values (maximum to 0 is Ok, as all those numbers are
    //positive, but minimum should be assigned some real value as there is
    //no upper limits.
    m_minmax_evaluator_calls.first = 
              results.front()->m_evaluator_calls;
    m_minmax_generations_to_find_best.first =
              results.front()->m_current_generation -
              results.front()->m_generations_since_solution_improvement;
    m_minmax_generations_to_find_all_bests.first = 
              results.front()->m_current_generation -
              results.front()->m_generations_since_new_best_found;
    m_minmax_number_of_generations.first =
              results.front()->m_current_generation;
    m_minmax_time.first =
              results.front()->m_time_required;
    for(TResultVector::const_iterator vit(results.begin());
            vit != results.end(); ++vit)
      {
        int num_of_generations =
                    (*vit)->m_current_generation;
        long evaluator_calls = 
                    (*vit)->m_evaluator_calls;
        int generations_to_find_best =
                    num_of_generations -
                    (*vit)->m_generations_since_solution_improvement;
        int generations_since_new_best_found =
                    num_of_generations -
                    (*vit)->m_generations_since_new_best_found;
        double time_required = 
                    (*vit)->m_time_required;
        //update minimum and maximum, if necessary.
        m_minmax_evaluator_calls.first = 
                min(m_minmax_evaluator_calls.first,
                    evaluator_calls);
        m_minmax_evaluator_calls.second = 
                max(m_minmax_evaluator_calls.second,
                    evaluator_calls);
        m_minmax_generations_to_find_best.first =
                min(m_minmax_generations_to_find_best.first,
                    generations_to_find_best);        
        m_minmax_generations_to_find_best.second =
                max(m_minmax_generations_to_find_best.second,
                    generations_to_find_best);        
        m_minmax_generations_to_find_all_bests.first =
                min(m_minmax_generations_to_find_all_bests.first,
                    generations_since_new_best_found);        
        m_minmax_generations_to_find_all_bests.second =
                max(m_minmax_generations_to_find_all_bests.second,
                    generations_since_new_best_found);        
        m_minmax_number_of_generations.first =
                min(m_minmax_number_of_generations.first,
                    num_of_generations);
        m_minmax_number_of_generations.second =
                max(m_minmax_number_of_generations.second,
                    num_of_generations);
        m_minmax_time.first =
                min(m_minmax_time.first,
                    time_required);
        m_minmax_time.second =
                max(m_minmax_time.second,
                    time_required);
        //update statistics
        m_stat_evaluator_calls.first +=
                    (*vit)->m_evaluator_calls;
        m_stat_generations_to_find_best.first +=
                    generations_to_find_best;
        m_stat_generations_to_find_all_bests.first +=
                    generations_since_new_best_found;
        m_stat_number_of_generations.first +=
                    num_of_generations;
        m_stat_time.first +=
                    time_required;
        
        m_stat_evaluator_calls.second +=
                    evaluator_calls *
                    evaluator_calls /
                    (double)m_number_of_runs;
        m_stat_generations_to_find_best.second +=
                    generations_to_find_best *
                    generations_to_find_best /
                    (double)m_number_of_runs;
        m_stat_generations_to_find_all_bests.second +=
                    generations_since_new_best_found *
                    generations_since_new_best_found /
                    (double)m_number_of_runs;
        m_stat_number_of_generations.second +=
                    num_of_generations *
                    num_of_generations /
                    (double)m_number_of_runs;
        m_stat_time.second +=
                    time_required *
                    time_required /
                    (double)m_number_of_runs;
      }
    m_stat_evaluator_calls.first /= m_number_of_runs;
    m_stat_generations_to_find_best.first /= m_number_of_runs;
    m_stat_generations_to_find_all_bests.first /= m_number_of_runs;
    m_stat_number_of_generations.first /= m_number_of_runs;
    m_stat_time.first /= m_number_of_runs;

    m_stat_evaluator_calls.second -=
                m_stat_evaluator_calls.first *
                m_stat_evaluator_calls.first;
    m_stat_generations_to_find_best.second -=
                m_stat_generations_to_find_best.first *
                m_stat_generations_to_find_best.first;
    m_stat_generations_to_find_all_bests.second -=
                m_stat_generations_to_find_all_bests.first *
                m_stat_generations_to_find_all_bests.first;
    m_stat_number_of_generations.second -=
                m_stat_number_of_generations.first *
                m_stat_number_of_generations.first;
    m_stat_time.second -=
                m_stat_time.first *
                m_stat_time.first;

    m_stat_evaluator_calls.second =
                m_stat_evaluator_calls.second > 0 ?
                sqrt(m_stat_evaluator_calls.second) : 0;
    m_stat_generations_to_find_best.second =
                m_stat_generations_to_find_best.second > 0 ?
                sqrt(m_stat_generations_to_find_best.second) : 0;
    m_stat_generations_to_find_all_bests.second =
                m_stat_generations_to_find_all_bests.second > 0 ?
                sqrt(m_stat_generations_to_find_all_bests.second) : 0;
    m_stat_number_of_generations.second =
                m_stat_number_of_generations.second > 0 ?
                sqrt(m_stat_number_of_generations.second) : 0;
    m_stat_time.second =
                m_stat_time.second > 0 ?
                sqrt(m_stat_time.second) : 0;
  }//collectStatistics

/**
 * Collect and prints the statistics from the matrix of results. Each
 * row corresponds to GA with different parameters and contains the
 * results of multiple runs of the this GA. Header is the title of each
 * of the experiments (each header corresponds to row of the matrix of
 * results). Implementation in this class prints only GA-run related
 * statistics (e.g. number of generations per run) and not results
 * statistics (e.g. average of the fittness function).
 * @param results matrix of results of GA runs.
 * @param headers header for each set of results.
 */
ostream& GLStatisticsSimple::toStream(ostream& out) const
  {
    out << "Statistics is collected over " << m_number_of_runs
        << " runs\n";
    out << setfill('-') << setw(90) << "-" << setfill(' ') << endl;
    out << setw(10) << left  << "STATISTICS NAME" 
        << setw(30) << right << "AVERAGE" 
        << setw(15) << right << "ST. DEVIATION"
        << setw(15) << right << "MINIMUM"
        << setw(15) << right << "MAXIMUM"
        << endl;
    out << setfill('-') << setw(90) << "-" << setfill(' ') << endl;
    out << setw(30) << left  << "Number of generations:"
        << setw(15) << right << m_stat_number_of_generations.first
        << setw(15) << right << m_stat_number_of_generations.second 
        << setw(15) << right << m_minmax_number_of_generations.first
        << setw(15) << right << m_minmax_number_of_generations.second 
        << endl;
    out << setw(30) << left  << "Generations to find best:"
        << setw(15) << right << m_stat_generations_to_find_best.first
        << setw(15) << right << m_stat_generations_to_find_best.second
        << setw(15) << right << m_minmax_generations_to_find_best.first
        << setw(15) << right << m_minmax_generations_to_find_best.second
        << endl;
    out << setw(30) << left  << "Generations to find all bests:"
        << setw(15) << right << m_stat_generations_to_find_all_bests.first
        << setw(15) << right << m_stat_generations_to_find_all_bests.second
        << setw(15) << right << m_minmax_generations_to_find_all_bests.first
        << setw(15) << right << m_minmax_generations_to_find_all_bests.second
        << endl;
    out << setw(30) << left  << "Number of evaluator calls:"
        << setw(15) << right << m_stat_evaluator_calls.first
        << setw(15) << right << m_stat_evaluator_calls.second
        << setw(15) << right << m_minmax_evaluator_calls.first
        << setw(15) << right << m_minmax_evaluator_calls.second
        << endl;
    out << setw(30) << left  << "Time taken:"
        << setw(15) << right << m_stat_time.first
        << setw(15) << right << m_stat_time.second
        << setw(15) << right << m_minmax_time.first
        << setw(15) << right << m_minmax_time.second
        << endl;
    return out;
  }//printStatistics
