#ifndef GLSTATISTICSSIMPLE_H_
#define GLSTATISTICSSIMPLE_H_
#include <ostream>
#include <string>
#include <vector>

#include <NLInterfaceOutput.h>
using namespace std;
class GLBaseGaStatus;
/**
 * Class to calculate and print the statistics about multiple GA runs.
 * This class does not do results-related statistics (e.g. average resulting
 * value of the objective function) as it would normally depend on type
 * of objective function -- single or multiple objective etc. However, it
 * will collect and print other statistics -- average number of evaluator
 * calls, number of generations per run etc. 
 * Note, that this class may refer to data from results set (e.g. best
 * organisms etc.) so it is valid until the results set is destroyed.
 */
class GLStatisticsSimple : public nlutils::NLInterfaceOutput
{
public:
  /**
   * Constructor. Collects all the relevant statistics from the vector
   * of results. Vector should contains the results of multiple runs of 
   * the GA with the same parameters. This class collects only GA-run related
   * statistics (e.g. number of generations per run) and not results 
   * statistics (e.g. average/best fittness value etc.). 
   * @param results vector of results of GA runs.
   */
  GLStatisticsSimple(const vector<GLBaseGaStatus*>& results);
  
  /**
   * Simple destructor.
   */
  virtual ~GLStatisticsSimple();

  /**
   * Return number of runs over which statistics was collected.
   * @return number of runs over which statistics was collected.
   */
  int getNumberOfRuns()
    { return m_number_of_runs; }
  
  /**
   * Returns the minimum and maximum of number of evaluator calls over 
   * provided sample of results.
   * @return minimum (first) and maximum (second) of number of evaluator
   *    calls. 
   */
  const pair<long, long>& getMinMaxEvaluatorCalls() const
  { return m_minmax_evaluator_calls; }
  
  /**
   * Returns the minimum and maximum of number of generations required to 
   * find best solution for each run (not necessarily THE best solution for 
   * the problem or even among all results).
   * @return minimum (first) and maximum (second) of number generations
   *    required to find the best solution. 
   */
  const pair<int, int>& getMinMaxGenerationsToFindBest() const
  { return m_minmax_generations_to_find_best; }
  
  /**
   * Returns minimum and maximum (over sample of runs provided) of number
   * of generations required to find all the bests solutions for each run. 
   * E.g. for single objective oprimization -- when the set of different
   * organisms but with the same fitness function was last amended. For
   * multiple objective optimization -- when the Pareto set was last time
   * modified. 
   * @return minimum (first) and maximum (second) of number
   *    of generations to find all best solutions. 
   */
  const pair<int, int>& getMinMaxGenerationsToFindAllBests() const
  { return m_minmax_generations_to_find_all_bests; }
  
  /**
   * Returns minimum and maximum (over sample of runs provided) of number
   * of generations made during each GA run (may vary depending on
   * stoppage criteria).
   * @return minimum (first) and maximum (second) of number
   *    of generations during each GA run. 
   */
  const pair<int, int>& getMinMaxNumberOfGenerations() const
  { return m_minmax_number_of_generations; }
  
  /**
   * Returns the statistics (average and standard deviation) of number
   * of evaluator calls calculated on provided sample of results.
   * @return average (first) and standard deviation (second) of number
   *    of evaluator calls. 
   */
  const pair<double, double>& getStatEvaluatorCalls() const
  { return m_stat_evaluator_calls; }
  
  /**
   * Returns the statistics (average and standard deviation) of number
   * of generations required to find best solution for each run (not
   * necessarily THE best solution for the problem or even among all 
   * results).
   * @return average (first) and standard deviation (second) of number
   *    of evaluator calls. 
   */
  const pair<double, double>& getStatGenerationsToFindBest() const
  { return m_stat_generations_to_find_best; }
  
  /**
   * Returns the statistics (average and standard deviation) of number
   * of generations required to find all the bests solutions for each run. 
   * E.g. for single objective oprimization -- when the set of different
   * organisms but with the same fitness function was last amended. For
   * multiple objective optimization -- when the Pareto set was last time
   * modified. 
   * @return average (first) and standard deviation (second) of number
   *    of generations to find all best solutions. 
   */
  const pair<double, double>& getStatGenerationsToFindAllBests() const
  { return m_stat_generations_to_find_all_bests; }
  
  /**
   * Returns the statistics (average and standard deviation) of number
   * of generations made during each GA run (may vary depending on
   * stoppage criteria).
   * @return average (first) and standard deviation (second) of number
   *    of generations during each GA run. 
   */
  const pair<double, double>& getStatNumberOfGenerations() const
  { return m_stat_number_of_generations; }

  /**
   * Prints the statistics. 
   */
  virtual ostream& toStream(ostream& out) const;

private:
  /**
   * Collects all the GA statistics from the vector of results.
   * @param results vector with the results of GA runs.
   */
  void collectGaStatistics(const vector<GLBaseGaStatus*>& results);
private:
  //number of runs (size of vector of results).
  int m_number_of_runs;
  //Minimum and maximum values over all runs. First min, second max.
  //Number of evaluator calls
  pair<long, long> m_minmax_evaluator_calls;
  //number of generations to find best solution
  pair<int, int> m_minmax_generations_to_find_best;
  //number of generations to get all best solutions (if more than one).
  pair<int, int> m_minmax_generations_to_find_all_bests;
  //number of generation GA run.
  pair<int, int> m_minmax_number_of_generations;
  //time required for GA run (in s).
  pair<double, double> m_minmax_time;

  //Statistics (average and deviation values). First mean, second st. dev.
  //Number of evaluator calls
  pair<double, double> m_stat_evaluator_calls; 
  //number of generations to find best solution
  pair<double, double> m_stat_generations_to_find_best; 
  //number of generations to get all best solutions (if more than one).
  pair<double, double> m_stat_generations_to_find_all_bests;
  //number of generation GA run.
  pair<double, double> m_stat_number_of_generations; 
  //time required for GA run (in s).
  pair<double, double> m_stat_time;
  
};

#endif /*GLSTATISTICSSIMPLE_H_*/
