#include <math.h>
#include <iomanip>

#include "GLBaseGaStatus.h"
#include "GLFitnessSingleObjective.h"
#include "GLStatisticsSingleObjective.h"
#include "GLUtils.h"

typedef vector<GLBaseGaStatus*> TResultVector;
/**
 * Constructor. Collects all the relevant statistics from the vector
 * of results. Vector should contains the results of multiple runs of
 * the GA with the same parameters. This class collects also results
 * statistics (e.g. average/best fittness value etc.). It assumes, that
 * best organisms which are stored in the vector of results can have
 * fitness function which define methods getSum and getProduct and
 * result of those methods can be returned in
 * GLFitnessSingleObjective<double>.
 * @param results vector of results of GA runs.
 */
GLStatisticsSingleObjective::GLStatisticsSingleObjective(
                        const vector<GLBaseGaStatus*>& results):
                          GLStatisticsSimple(results),
                          m_minmax_fitness(NULL, NULL),
                          m_stat_fitness(0.0, 0.0)
  {
    //do nothing, if results vector is empty.
    if (results.empty())
      return;
    collectFitnessStatistics(results);
  }//constructor

/**
 * Simple destructor.
 */
GLStatisticsSingleObjective::~GLStatisticsSingleObjective()
  {
  }//destructor

/**
 * Collects fitness statistics from the vector of results.
 * @param results vector with the results of GA runs.
 */
void GLStatisticsSingleObjective::collectFitnessStatistics(
                        const vector<GLBaseGaStatus*>& results)
  {
    if (results.empty()) return;
    //best fitness in single run transformed to double.
    GLFitnessSingleObjective<double> best_current_fitness(0.0);
    //best fitness over all runs.
    for(TResultVector::const_iterator vit(results.begin());
            vit != results.end(); ++vit)
      {
        //take the fitness of first organism from the vector of best
        //organisms
        if ((*vit)->m_best_ever_organisms.empty()) continue;
        (*vit)->m_best_ever_organisms.front()->
            getFitness()->assignTo(best_current_fitness);

        m_stat_fitness.first  += best_current_fitness.getValue();

        m_stat_fitness.second += best_current_fitness.getValue() *
                         best_current_fitness.getValue() /
                         getNumberOfRuns();

        //count how many times the best solution was found.
        if ((m_minmax_fitness.first != NULL) &&
            (*vit)->m_best_ever_organisms.front()->getFitness()
                              ->isEqual(m_minmax_fitness.first))
          {
            ++m_times_best_found;
          }
        //look for best fitness
        if ((m_minmax_fitness.first == NULL) ||
            (*vit)->m_best_ever_organisms.front()->getFitness()
                  ->isLess(m_minmax_fitness.first))
          {
            m_minmax_fitness.first = (*vit)->m_best_ever_organisms.front()
                                                      ->getFitness();
            m_times_best_found = 1;
          }
        //look for worst fitness
        if ((m_minmax_fitness.second == NULL) ||
            m_minmax_fitness.second->isLess(
                        (*vit)->m_best_ever_organisms.front()
                              ->getFitness()))
          {
            m_minmax_fitness.second = (*vit)->m_best_ever_organisms.front()
                                                      ->getFitness();
          }
      }
    m_stat_fitness.first /= getNumberOfRuns();

    m_stat_fitness.second -= m_stat_fitness.first * m_stat_fitness.first;

    m_stat_fitness.second = m_stat_fitness.second > 0 ?
                            sqrt(m_stat_fitness.second) : 0;

    //form the set of best ever organisms
    for(TResultVector::const_iterator vit(results.begin());
            vit != results.end(); ++vit)
      {
        //take the fitness of first organism from the vector of best
        //organisms and if it is equal to the best one -- copy
        //the whole vector to the set of best organisms.
        if ((*vit)->m_best_ever_organisms.empty()) continue;
        if ((*vit)->m_best_ever_organisms.front()
            ->getFitness()->isEqual(m_minmax_fitness.first))
          {
            ms_best_org.insert((*vit)->m_best_ever_organisms.begin(),
                               (*vit)->m_best_ever_organisms.end());
          }
      }
  }//collectFitnessStatistics

/**
 * Prints the statistics.
 */
ostream& GLStatisticsSingleObjective::toStream(ostream& out) const
  {
    GLStatisticsSimple::toStream(out);
    out << setw(30) << left << "Best solution found "
        << m_times_best_found << " times\n";
    out << setw(30) << left << "Fitness value:"
        << setw(15) << right << m_stat_fitness.first
        << setw(15) << right << m_stat_fitness.second
        << setw(15) << right << (*m_minmax_fitness.first)
        << setw(15) << right << (*m_minmax_fitness.second)
        << endl;

    TGLVectorOfOrganismsConst temp_vec(ms_best_org.begin(),
                                       ms_best_org.end());
    GLUtils::printVectorOfOrganisms(out, temp_vec);
    return out;
  }//toStream
