#ifndef GLSTATISTICSSINGLEOBJECTIVE_H_
#define GLSTATISTICSSINGLEOBJECTIVE_H_
#include "GLBaseOrganism.h"
#include "GLStatisticsSimple.h"

/**
 * This class collects statistics about fittness values of single objective
 * functions. It is assumed, that fitness functions of organisms supports
 * methods getSum and getProduct and return results in 
 * GLFitnessSingleObjective<double>.
 * Note, that this class may refer to data from results set (e.g. best
 * organisms etc.) so it is valid until the results set is destroyed.
 */
class GLStatisticsSingleObjective : public GLStatisticsSimple
{
public:
  /**
   * Constructor. Collects all the relevant statistics from the vector
   * of results. Vector should contains the results of multiple runs of 
   * the GA with the same parameters. This class collects also results 
   * statistics (e.g. average/best fittness value etc.). It assumes, that
   * best organisms which are stored in the vector of results can have
   * fitness function which define methods getSum and getProduct and 
   * result of those methods can be returned in 
   * GLFitnessSingleObjective<double>.
   * @param results vector of results of GA runs.
   */
  GLStatisticsSingleObjective(const vector<GLBaseGaStatus*>& results);

  /**
   * Simple destructor.
   */
  virtual ~GLStatisticsSingleObjective();

  /**
   * Returns minimum (first) and maximum (second) of the solutions fitness
   * found over all runs.
   * @return minimum and maximum of the solution fitness found over 
   *    all runs.
   */
  const pair<const GLBaseFitness*, const GLBaseFitness*> 
      getMinMaxFitness() const
  { return m_minmax_fitness; }

  /**
   * Returns average (first) and standard deviation (second) of the solutions 
   * fitness found over all runs.
   * @return average and standard deviation of the solution fitness found 
   *    over all runs.
   */
  const pair<const GLBaseFitness*, const GLBaseFitness*> 
      getStatFitness() const
  { return m_minmax_fitness; }

  /**
   * Return in how many runs the best solution was found.
   * @return in how many runs the best solution was found.
   */
  int getTimesBestFound() const
  { return m_times_best_found; }
  
  /**
   * Prints the statistics. 
   */
  virtual ostream& toStream(ostream& out) const;

private:
  /**
   * Collects fitness statistics from the vector of results.
   * @param results vector with the results of GA runs.
   */
  void collectFitnessStatistics(const vector<GLBaseGaStatus*>& results);
private:
  //Minimum and maximum values over all runs. First min, second max.
  pair<const GLBaseFitness*, const GLBaseFitness*> m_minmax_fitness;
  //Statistics (average and deviation values). First mean, second st. dev.
  //fitness function value
  pair<double, double> m_stat_fitness;         
  int    m_times_best_found;    //how many times the best fitness was found.
  //set of best organisms obtained over all runs.
  TGLSetOfOrganismsConst ms_best_org; 
};

#endif /*GLSTATISTICSSINGLEOBJECTIVE_H_*/
