#include "GLBaseOrganism.h"
#include "GLLogicError.h"
#include "GLRuntimeError.h"
#include "GLStorageSet.h"

GLStorageSet::GLStorageSet()
{
}

GLStorageSet::~GLStorageSet()
{
}

/**
 * Adds the new organism to storage. If the 'no duplication' flag is set and
 * organism is already in the storage -- do not add it. Organisms are checked
 * in 'soft' manner.
 * @param organism new organism to be added to storage.
 * @param twins when set to NO_TWINS, check that this organism is not in
 *    the storage already.
 * @return true if organism was successfully added and false otherwise.
 */
bool GLStorageSet::addOrganism(GLBaseOrganism* organism,
                               GLConstants::TTwinsFlags twins)
  {
    //add organism if no check for duplication is necessary or this is
    //unique organism.
    return (m_storage.insert(organism)).second;
  }//addOrganism

/**
 * Removes all the organisms from the storage and frees memory
 */
void GLStorageSet::emptyStorage()
  {
    //delete all organisms depending on the memory policy
    switch(getMemoryPolicy())
    {
      case GLConstants::FREE_ALL:
            deleteAllOrganisms();
            break;
      case GLConstants::FREE_UNLOCKED:
            deleteOrganisms();
            break;
      default:
            break;
    }
    //empty the container itself.
    m_storage.clear();
  }//emptyStorage()

/**
 * Checks, if the organism is already in storage (compared not pointers,
 * but the content of the organisms).
 * @param organism organism to be found.
 * @param soft when true comparison is made in the 'soft' sense.
 * @return pointer on found organism in the storage or NULL if nothing
 *    was found.
 */
const GLBaseOrganism* GLStorageSet::findOrganism(
                                  const GLBaseOrganism* organism,
                                  bool soft) const
  {
    TGLSetOfOrganisms::const_iterator it =
          m_storage.find(const_cast<GLBaseOrganism*>(organism));
    if (it == m_storage.end())
      {
        return NULL;
      }
    return *it;
  }//findOrganism
/**
 * Returns the pointer to iterator at the first element of the storage.
 * @return pointer to iterator at the first element of the storage.
 */
GLBaseOrganismStorage::iterator* GLStorageSet::getIterator()
  {
    return new iterator(m_storage);
  }//getIterator

/**
 * Returns the pointer to constant iterator at the first element of
 * the storage.
 * @return pointer to constant iterator at the first element of
 * the storage.
 */
GLBaseOrganismStorage::const_iterator*
        GLStorageSet::getIteratorConst() const
  {
    return new const_iterator(m_storage);
  }//getIteratorConst


/**
 * Provides the vector of pointers to the organisms in storage starting
 * from the first one. The base version uses iterator to fill the
 * vector, however, for real implementation of storage which uses stl
 * containers more efficient approach should be possible.
 * @param container vector of pointers to the organisms from storage
 *                    (output value).
 * @param fill when APPEND the new organisms are added to the container,
 *    when REPLACE -- the container is cleaned before adding the
 *    organisms.
 * @param size number of organisms from storage to be returned in vector.
 *            if < 0 or > getSize() then returns all the organisms.
 * @param copy_clones when true the pointers to the clones of the organisms
 *    will be copied, otherwise -- pointers to the original organisms.
 * @return number of elements transferred to container.
 *
 */
int GLStorageSet::getOrganismsVector(
                       vector<GLBaseOrganism*>& container,
                       GLConstants::TStorageFilling fill,
                       int size,
                       bool copy_clones)
  {
    if (fill == GLConstants::REPLACE) container.clear();
    int to_transfer = size;
    if ((size == -1) || (size >= getSize()))
      {
        to_transfer = getSize();
      }
    container.reserve(container.size() + to_transfer);
    TGLSetOfOrganisms::iterator it = m_storage.begin();
    for(int i = 0; i < to_transfer; i++)
    {
      container.push_back(copy_clones ?
                          (*it)->makeClone(false) : *it);
      ++it;
    }
    return to_transfer;
  }//getOrganismsVector

/**
 * Provides the vector of pointers to the organisms in storage starting
 * from the first one.
 * @param container vector of pointers to the organisms from storage
 *                    (output value).
 * @param fill when APPEND the new organisms are added to the container,
 *    when REPLACE -- the container is cleaned before adding the
 *    organisms.
 * @param size number of organisms from storage to be returned in vector.
 *            if < 0 or > getSize() then returns all the organisms.
 * @param copy_clones when true the pointers to the clones of the organisms
 *    will be copied, otherwise -- pointers to the original organisms.
 * @return number of elements transferred to container.
 *
 */
int GLStorageSet::getOrganismsVector(
                       vector<const GLBaseOrganism*>& container,
                       GLConstants::TStorageFilling fill,
                       int size,
                       bool copy_clones) const
  {
    if (fill == GLConstants::REPLACE) container.clear();
    int to_transfer = size;
    if ((size == -1) || (size >= getSize()))
      {
        to_transfer = getSize();
      }

    int container_start = container.size();
    container.resize(container.size() + to_transfer);

    //set both iterators (for container and storage) to the start
    vector<const GLBaseOrganism*>::iterator it_cont =
            container.begin() + container_start;
    TGLSetOfOrganisms::const_iterator it =
            m_storage.begin();
    for(int i = 0; i < to_transfer; i++)
      {
        *it_cont = copy_clones ?
                    (*it)->makeClone(false) : *it;
        ++it_cont;
        ++it;
      }
    return to_transfer;
  }//getOrganismsVector

/**
 * Adds organisms from another storage to the current one.
 * @param storage another organism storage.
 * @param twins when equal to NO_TWINS, check that this organism is not in
 *    the storage already.
 */
void GLStorageSet::merge(
                    const GLBaseOrganismStorage* storage,
                    GLConstants::TTwinsFlags twins)
  {
    //first, check if the storage is mergeable
    const GLStorageSet* container =
        dynamic_cast<const GLStorageSet*>(storage);
    if (container == NULL)
      {
        char errormsg[1000];
        sprintf(errormsg, "\nGLStorageSet::merge\n"
                          "Can be merged with another GLStorageSet or "
                          "its children only");
        throw GLLogicError(errormsg);
      }
    //if no check for duplications is necessary -- do simple merge.
    m_storage.insert(container->m_storage.begin(),
                     container->m_storage.end());
  }//merge

/**
 * Decreases the size of the storage. If the size of storage was less, than
 * new size, that nothing will be done.
 * @param new_size new number of organisms in the storage.
 * @return the number of organisms in storage after operation.
 */
int GLStorageSet::trimSize(unsigned int new_size)
  {
    throw GLRuntimeError("GLStorageSet::trimSize not implemented\n");
    if (m_storage.size() <= new_size) return m_storage.size();
    //ok, we have what to trim
    //if memory policy is not 'DO_NOT_FREE' -- free memory for elements of
    //storage
//    if (getMemoryPolicy() != GLConstants::DO_NOT_FREE)
//      {
//        for(unsigned i = new_size; i < m_storage.size(); i++)
//          {
//            //free organism's memory if
//            //a) it is allocated AND
//            //  b) memory policy is FREE_ALL OR
//            //  c) memory policy is FREE_TAGGED &
//            //     d) organism is tagged for removal.
//            if ((m_storage[i] != NULL) &&
//                ((getMemoryPolicy() == GLConstants::FREE_ALL) ||
//                 ((getMemoryPolicy() == GLConstants::FREE_TAGGED) &&
//                  (m_storage[i]->getTag() == GLConstants::REMOVE))))
//              {
//                delete m_storage[i];
//              }
//          }
//      }
//    m_storage.resize(new_size);
    return new_size;
  }//trimSize
