#ifndef GLSTORAGESET_H_
#define GLSTORAGESET_H_
#include "GLBaseOrganism.h"
#include "GLBaseOrganismStorage.h"
#include "GLIteratorStorageStl.h"

/**
 * Uses set for storing the organisms. No random access to organisms
 * but provides considerably more convenient way to ensure the uniqueness of
 * organisms in the storage.
 */
class GLStorageSet : public GLBaseOrganismStorage
{
public:
  /**
   * Simple constructor
   */
  GLStorageSet();
  /**
   * Frees memory allocated to the organisms, depending on the memory
   * policy property.
   */
  virtual ~GLStorageSet();
  /**
   * Adds the new organism to storage. If the 'no duplication' flag is set and
   * organism is already in the storage -- do not add it. Organisms are checked
   * in 'soft' manner.
   * @param organism new organism to be added to storage.
   * @param twins when set to NO_TWINS, check that this organism is not in
   *    the storage already.
   * @return true if organism was successfully added and false otherwise.
   */
  virtual bool addOrganism(GLBaseOrganism* organism,
                           GLConstants::TTwinsFlags twins =
                             GLConstants::NO_TWINS);

  /**
   * Removes all the organisms from the storage and frees memory
   */
  void emptyStorage();

  /**
   * Checks, if the organism is already in storage (compared not pointers,
   * but the content of the organisms).
   * @param organism organism to be found.
   * @param soft when true comparison is made in the 'soft' sense.
   * @return pointer on found organism in the storage or NULL if nothing
   *    was found.
   */
  virtual const GLBaseOrganism* findOrganism(
                                    const GLBaseOrganism* organism,
                                    bool soft = true) const;

  /**
   * Returns the pointer to iterator at the first element of the storage.
   * @return pointer to iterator at the first element of the storage.
   */
  virtual GLBaseOrganismStorage::iterator* getIterator();

  /**
   * Returns the pointer to constant iterator at the first element of
   * the storage.
   * @return pointer to constant iterator at the first element of
   * the storage.
   */
  virtual GLBaseOrganismStorage::const_iterator* getIteratorConst() const;

  /**
   * Provides the vector of pointers to the organisms in storage starting
   * from the first one. The base version uses iterator to fill the
   * vector, however, for real implementation of storage which uses stl
   * containers more efficient approach should be possible.
   * @param container vector of pointers to the organisms from storage
   *                    (output value).
   * @param fill when APPEND the new organisms are added to the container,
   *    when REPLACE -- the container is cleaned before adding the
   *    organisms.
   * @param size number of organisms from storage to be returned in vector.
   *            if < 0 or > getSize() then returns all the organisms.
   * @param copy_clones when true the pointers to the clones of the organisms
   *    will be copied, otherwise -- pointers to the original organisms.
   * @return number of elements transferred to container.
   *
   */
  virtual int
      getOrganismsVector(vector<GLBaseOrganism*>& container,
                         GLConstants::TStorageFilling fill =
                           GLConstants::APPEND,
                         int size = -1,
                         bool copy_clones = false);

  /**
   * Provides the vector of pointers to the organisms in storage starting
   * from the first one.
   * @param container vector of pointers to the organisms from storage
   *                    (output value).
   * @param fill when APPEND the new organisms are added to the container,
   *    when REPLACE -- the container is cleaned before adding the
   *    organisms.
   * @param size number of organisms from storage to be returned in vector.
   *            if < 0 or > getSize() then returns all the organisms.
   * @param copy_clones when true the pointers to the clones of the organisms
   *    will be copied, otherwise -- pointers to the original organisms.
   * @return number of elements transferred to container.
   *
   */
  virtual int
      getOrganismsVector(vector<const GLBaseOrganism*>& container,
                         GLConstants::TStorageFilling fill =
                           GLConstants::APPEND,
                         int size = -1,
                         bool copy_clones = false) const;

  /**
   * Returns the number of elements in storage.
   * @return number of elements in storage.
   */
  virtual int getSize() const
    { return m_storage.size(); }

  /**
   * Adds organisms from another storage to the current one.
   * @param storage another organism storage.
   * @param twins when equal to NO_TWINS, check that this organism is not in
   *    the storage already.
   */
  virtual void merge(const GLBaseOrganismStorage* storage,
                     GLConstants::TTwinsFlags twins =
                       GLConstants::NO_TWINS);

  /**
   * Decreases the size of the storage. If the size of storage was less, than
   * new size, that nothing will be done.
   * @param new_size new number of organisms in the storage.
   * @return the number of organisms in storage after operation.
   */
  virtual int trimSize(unsigned int new_size);

private:

  TGLSetOfOrganisms m_storage;

/*--------------------------- INTERNAL CLASSES -----------------------*/
  typedef GLIteratorStorageStl<TGLSetOfOrganisms,
                               TGLSetOfOrganisms::iterator,
                               GLBaseOrganism*>
                               iterator;
  typedef GLIteratorStorageStl<const TGLSetOfOrganisms,
                               TGLSetOfOrganisms::const_iterator,
                               const GLBaseOrganism* const>
                               const_iterator;
};//class GLStorageSet

#endif /*GLSTORAGESET_H_*/
