#include "GLLogicError.h"
#include "GLRuntimeError.h"
#include "GLStorageVector.h"

/**
 * Constructor.
 * @param size expected size of the storage -- memory will be reserved
 * to avoid re-allocation once elements are added to storage. Note, that
 * initial size of the storage will stay 0.
*/
GLStorageVector::GLStorageVector(unsigned expected_size):
  GLBaseOrganismStorage()
{
  //reserve space for storage
  m_storage.reserve(expected_size);
}

GLStorageVector::~GLStorageVector()
{
  emptyStorage();
}

/**
 * Adds the new organism to storage. If the 'no duplication' flag is set and
 * organism is already in the storage -- do not add it. Organisms are checked
 * in 'soft' manner.
 * @param organism new organism to be added to storage.
 * @param twins when set to NO_TWINS, check that this organism is not in
 *    the storage already.
 * @return true if organism was succesfully added and false otherwise.
 */
bool GLStorageVector::addOrganism(GLBaseOrganism* organism,
                                  GLConstants::TTwinsFlags twins)
  {
    //add organism if no check for duplication is necessary or this is
    //unique organism.
    if ((twins == GLConstants::TWINS_ALLOWED) ||
        (findOrganismIndex(organism) == -1))
      {
        m_storage.push_back(organism);
        return true;
      }
    return false;
  }//addOrganism

/**
 * Removes all the organisms from the storage and frees memory
 */
void GLStorageVector::emptyStorage()
  {
    //delete all organisms depending on the memory policy
    switch(getMemoryPolicy())
    {
      case GLConstants::FREE_ALL:
            deleteAllOrganisms();
            break;
      case GLConstants::FREE_UNLOCKED:
            deleteOrganisms();
            break;
      default:
            break;
    }
    //empty the container itself.
    m_storage.clear();
  }//emptyStorage()

/**
 * Returns the index of the organism in the storage which is equal
 * to one provided or -1 if no such organism was found.
 * @param organism organism for comparison.
 * @param soft when true 'soft' comparison of organisms is conducted.
 */
int GLStorageVector::findOrganismIndex(
                      const GLBaseOrganism* organism,
                      bool soft) const
  {
    //first simple check
    if (organism == NULL) return -1;
    for(int i = 0; i < getSize(); i++)
      {
        if (m_storage[i]->isEqual(organism, soft)) return i;
      }
    return -1;
  }

/**
 * Returns the pointer to iterator at the first element of the storage.
 * @return pointer to iterator at the first element of the storage.
 */
GLBaseOrganismStorage::iterator* GLStorageVector::getIterator()
  {
    return new iterator(m_storage);
  }//getIterator

/**
 * Returns the pointer to constant iterator at the first element of
 * the storage.
 * @return pointer to constant iterator at the first element of
 * the storage.
 */
GLBaseOrganismStorage::const_iterator*
        GLStorageVector::getIteratorConst() const
  {
    return new const_iterator(m_storage);
  }//getIteratorConst

/**
 * Returns the pointer to the organism with given index.
 * @param i index of the organism to be returned.
 */
GLBaseOrganism* GLStorageVector::getOrganismPointer(unsigned i)
  {
    if ((i < 0) || (i >= m_storage.size()))
      {
        char errormsg[1000];
        sprintf(errormsg, "\nGLStorageVector::getOrganismPointer\n"
                          "Request for the organism with index %u while "
                          "only %u organisms are available\n",
                          i, (unsigned)(m_storage.size()));
        throw GLRuntimeError(errormsg);
      }
    return m_storage[i];
  }//getOrganismPointer

/**
 * Returns the pointer to the organism with given index.
 * @param i index of the organism to be returned.
 */
const GLBaseOrganism* GLStorageVector::getOrganismPointer(unsigned i) const
  {
    if ((i < 0) || (i >= m_storage.size()))
      {
        char errormsg[1000];
        sprintf(errormsg, "\nGLStorageVector::getOrganismPointer\n"
                          "Request for the organism with index %i while "
                          "only %i organisms are available\n",
                          i, (unsigned)(m_storage.size()));
        throw GLRuntimeError(errormsg);
      }
    return m_storage[i];
  }//getOrganismPointer

/**
 * Provides the vector of pointers to the organisms in storage starting
 * from the first one. The base version uses iterator to fill the
 * vector, however, for real implementation of storage which uses stl
 * containers more efficient approach should be possible.
 * @param container vector of pointers to the organisms from storage
 *                    (output value).
 * @param fill when APPEND the new organisms are added to the container,
 *    when REPLACE -- the container is cleaned before adding the
 *    organisms.
 * @param size number of organisms from storage to be returned in vector.
 *            if < 0 or > getSize() then returns all the organisms.
 * @param copy_clones when true the pointers to the clones of the organisms
 *    will be copied, otherwise -- pointers to the original organisms.
 * @return number of elements transferred to container.
 *
 */
int GLStorageVector::getOrganismsVector(
                       vector<GLBaseOrganism*>& container,
                       GLConstants::TStorageFilling fill,
                       int size,
                       bool copy_clones)
  {
    if (fill == GLConstants::REPLACE) container.clear();
    int to_transfer = size;
    if ((size == -1) || (size >= getSize()))
      {
        to_transfer = getSize();
      }

    int container_start = container.size();
    container.resize(container.size() + to_transfer);

    //set both iterators (for container and storage) to the start
    vector<GLBaseOrganism*>::iterator it_cont =
            container.begin() + container_start;
    vector<GLBaseOrganism*>::iterator it =
            m_storage.begin();
    for(int i = 0; i < to_transfer; i++)
      {
        *it_cont = copy_clones ?
            (*it)->makeClone(false) : *it;
        ++it_cont;
        ++it;
      }
    return to_transfer;
  }//getOrganismsVector

/**
 * Provides the vector of pointers to the organisms in storage starting
 * from the first one.
 * @param container vector of pointers to the organisms from storage
 *                    (output value).
 * @param fill when APPEND the new organisms are added to the container,
 *    when REPLACE -- the container is cleaned before adding the
 *    organisms.
 * @param size number of organisms from storage to be returned in vector.
 *            if < 0 or > getSize() then returns all the organisms.
 * @param copy_clones when true the pointers to the clones of the organisms
 *    will be copied, otherwise -- pointers to the original organisms.
 * @return number of elements transferred to container.
 *
 */
int GLStorageVector::getOrganismsVector(
                       vector<const GLBaseOrganism*>& container,
                       GLConstants::TStorageFilling fill,
                       int size,
                       bool copy_clones) const
  {
    if (fill == GLConstants::REPLACE) container.clear();
    int to_transfer = size;
    if ((size == -1) || (size >= getSize()))
      {
        to_transfer = getSize();
      }

    int container_start = container.size();
    container.resize(container.size() + to_transfer);

    //set both iterators (for container and storage) to the start
    vector<const GLBaseOrganism*>::iterator it_cont =
            container.begin() + container_start;
    vector<GLBaseOrganism*>::const_iterator it =
            m_storage.begin();
    for(int i = 0; i < to_transfer; i++)
      {
        *it_cont = copy_clones ?
            (*it)->makeClone(false) : *it;
        ++it_cont;
        ++it;
      }
    return to_transfer;
  }//getOrganismsVector

/**
 * Adds organisms from another storage to the current one.
 * @param storage another organism storage.
 * @param twins when equal to NO_TWINS, check that this organism is not in
 *    the storage already.
 */
void GLStorageVector::merge(
                    const GLBaseOrganismStorage* storage,
                    GLConstants::TTwinsFlags twins)
  {
    //first, check if the storage is mergeable
    const GLStorageVector* container =
        dynamic_cast<const GLStorageVector*>(storage);
    if (container == NULL)
      {
        char errormsg[1000];
        sprintf(errormsg, "\nGLStorageVector::merge\n"
                          "Can be merged with another GLStorageVector or "
                          "its children only");
        throw GLLogicError(errormsg);
      }
    //if no check for duplications is necessary -- do simple merge.
    if (twins == GLConstants::TWINS_ALLOWED)
      {
        m_storage.insert(m_storage.end(), container->m_storage.begin(),
                                container->m_storage.end());
      }
    else
      {
        //else -- add 1 by one while checking for duplicates
        for(unsigned i = 0; i < container->m_storage.size(); i++)
          addOrganism(container->m_storage[i], twins);
      }
  }//merge

/**
 * Decreases the size of the storage. If the size of storage was less, than
 * new size, that nothing will be done.
 * @param new_size new number of organisms in the storage.
 * @return the number of organisms in storage after operation.
 */
int GLStorageVector::trimSize(unsigned int new_size)
  {
    if (m_storage.size() <= new_size) return m_storage.size();
    //ok, we have what to trim
    //if memory policy is not 'DO_NOT_FREE' -- free memory for elements of
    //storage
    if (getMemoryPolicy() != GLConstants::DO_NOT_FREE)
      {
        for(unsigned i = new_size; i < m_storage.size(); i++)
          {
            //free organism's memory if
            //a) it is allocated AND
            //  b) memory policy is FREE_ALL OR
            //  c) memory policy is FREE_TAGGED &
            //     d) organism is tagged for removal.
            if ((m_storage[i] != NULL) &&
                ((getMemoryPolicy() == GLConstants::FREE_ALL) ||
                 ((getMemoryPolicy() == GLConstants::FREE_UNLOCKED) &&
                  (!m_storage[i]->isLockedInMemory()))))
              {
                delete m_storage[i];
              }
          }
      }
    m_storage.resize(new_size);
    return new_size;
  }//trimSize
