#include <iostream>
#include "GLBaseMutator.h"
#include "GLBaseOrganism.h"
#include "GLBaseOrganismStorage.h"
#include "GLBasePopulation.h"
#include "GLVariatorStandard.h"
/**
 * Constructor. Takes the pointer to mutator operator. Note, that this
 * mutator will be destroyed with this object.
 * @param mutator mutator operator.
 */
GLVariatorStandard::GLVariatorStandard(GLBaseMutator* mutator):
  GLBaseVariator(mutator)
{
}//constructor

/**
 * Destroys the mutator operator (in base destructor).
 */
GLVariatorStandard::~GLVariatorStandard()
{
}//destructor

/**
 * Applies mutator to the set container of organisms.
 * @param population population to which mutated organisms belong -- mainly
 *    to check for uniqueness of the mutants.
 * @param organisms container with organisms.
 * @param mutants where mutated organisms will be added.
 * @param fill when equal to APPEND the container with mutants will be
 *    appended with new organisms, when REPLACE - mutants will be emptied
 *    at the beginning.
 * @param twins when NO_TWINS only unique organisms will be added.
 * @param max_mutants maximum number of mutants to be created,
 *            if < 0 -- number is limited only by the mutation algorithm
 *            used.
 * @param max_mutants maximum number of mutans which can be produced by
 *    mutator. When < 0 the number of mutants produced determined only
 *    by mutation rate.
 * @return number of new organisms produced by this method.
 */
int GLVariatorStandard::applyMutator(
                 GLBasePopulation* population,
                 GLBaseOrganismStorage* organisms,
                 GLBaseOrganismStorage* mutants,
                 GLConstants::TStorageFilling fill,
                 GLConstants::TTwinsFlags twins,
                 int max_mutants)
{
  //clean the container with mutants, if necessary.
  if (fill != GLConstants::APPEND) mutants->emptyStorage();
  //see, if maximum number of mutants is 0...
  if (max_mutants == 0) return 0;
  //get the iterator for the source container
  int counter = 0;
  GLBaseOrganismStorage::const_iterator* it = organisms->getIteratorConst();
  //loop through all elements of the container
  for(; !it->isEnd(); ++(*it))
    {
      //create a clone and try to mutate it
      GLBaseOrganism *clone = it->getElement()->makeClone();
      if (m_mutator->mutateOrganism(clone))
        {
          //mutated, now check, if it is truly new organism
          //-- check if the organism is new
          if ((twins == GLConstants::TWINS_ALLOWED) ||
               population->isOrganismTrulyNew(clone))
            {
              //in some case the storage does not allow to add twins anyway,
              //so ensure, that we delete clone to avoid memory leak.
              if (mutants->addOrganism(clone, twins))
                {
                  counter++;
                }
              else 
                {
                  delete clone;
                }
              //check if we need to continue
              if ((max_mutants > 0) && (counter == max_mutants))
                {
                  break;
                }
            }
          else
            {
              delete clone;
            }
        }
      else
        {
          delete clone;
        }
    }
  delete it;
  return counter;
}//applyMutator
