#ifndef NLSTL_UTILS_
#define NLSTL_UTILS_
#include <sstream>
#include <string>
#include "NLRuntimeError.h"
namespace nlutils
{
/**
 * Various (mostly templated functions) for use with STL.
 * Bodies of these functions are defined in this header to avoid problems
 * with instantiations.
*/

 /**
  * Structure to help with deleting containers of pointers.
  * It can be used with for_each STL algorithm.
  */
  struct DeleteObject
  {
     template<typename T>
     void operator()(const T* ptr) const
     {
         delete ptr;
     }
  };//DeleteObject

 /**
  * Structure to help with deleting containers of pointers.
  * Checks that pointer is not NULL before deleting.
  * It can be used with for_each STL algorithm.
  */
  struct DeleteObjectSafely
  {
     template<typename T>
     void operator()(const T* ptr) const
     {
         if (ptr != NULL) delete ptr;
     }
  };//DeleteObjectSafely

  /**
   * Function to assert that pointer is not NULL.
   * Throws NLRuntimeError in case of NULL pointer passed.
   * @param pointer pointer to be asserted.
   * @param message useful information, e.g. name of the pointer and/or
   *            calling function.
   */
  inline void assertPointer(const void *pointer,
                     const string& message = "")
    {
      if (pointer == NULL)
        {
          char errormsg[1000];
          sprintf(errormsg, "\nnlutils::assertPointer\n"
                            "%s pointer is NULL\n",
                            message.c_str());
          throw NLRuntimeError(errormsg);
        }
    }//assertPointer

/**
 * Function to transform pointer from one class to another using
 * dynamic_cast<>. Throws the GLRuntimeError exception if result is
 * NULL pointer. After this function is called succesfully the returned
 * pointer is guaranteed to be non-NULL and of the correct type.
 * @param source to be transformed pointer to the class.
 * @param errormsg error message to be thrown (will add
 * "\ngl_utils::transform_pointer returned NULL pointer with message:\n"
 * at the beginning).
 * @return pointer to TDestinationType class.
 */
template <class TDestinationType, class TSourceType>
TDestinationType* transform_pointer(TSourceType* source,
                        const string& error_msg = "")
{
  TDestinationType* dest = dynamic_cast<TDestinationType*>(source);
  if (dest == NULL)
    {
      string errormsg = "\nnlutils::transform_pointer returned NULL "
                        "pointer with message:\n" + error_msg;
      throw NLRuntimeError(errormsg);
    }
  return dest;
}

/**
 * Converts everything printable to string.
 * @param t value to be converted to string.
 */
template <class T>
inline std::string toString (const T& t)
{
  std::stringstream ss;
  ss << t;
  return ss.str();
}//toString
}
#endif /*NLSTL_UTILS_*/
