#include <iostream>
#include <list>

#include <GLBaseGeneticAlgorithm.h>
#include <GLBreederStandard.h>
#include <GLCrossoverVectorKeepMatching.h>
#include <GLFactory.h>
#include "GLFitness.h"
#include <GLGaStopperMaxGenerations.h>
#include <GLGaStopperNoNew.h>
#include <GLGeneticAlgorithmStandard.h>
#include <GLInitialiserMutator.h>
#include <GLInitialiserShuffleIntVector.h>
#include <GLLogicError.h>
#include <GLMultipleGaRunner.h>
#include <GLMutatorCombined.h>
#include <GLMutatorGeneMove.h>
#include <GLMutatorGenesSwapping.h>
#include <GLMutatorShuffling.h>
#include <GLOrganismVector.h>
#include <GLParametersGeneticAlgorithm.h>
#include <GLSelectionistMixer.h>
#include <GLSelectionistStandard.h>
#include <GLStatisticsSingleObjective.h>
#include <GLStorageVector.h>
#include <GLRandomNumbersGenerator.h>
#include <GLRuntimeError.h>
#include <GLTestEvaluator.h>
using namespace std;

/**
 * Function for setting GA breeder parameters.
 * @param breederType depending on constant's value chooses
 *      which breeder to setup.
 * @param parameters GA parameters.
 */
void setParametersBreeder(int breederType, 
            GLParametersGeneticAlgorithm* parameters)
  {
    //set mutator type first.
    parameters->m_breeder_type = breederType;
    //if necessary -- prepare additional parameters.
    switch(breederType)
    {
      case GLConstants::BREEDER_STANDARD:
          {
            parameters->m_breeder_params =
              new GLBreederStandard::TParameters(0.7);
            break;
          }
    }
  }//setParametersBreeder

/**
 * Function for setting GA mutator parameters.
 * @param mutatorType depending on constant's value chooses
 *      which mutator to setup.
 * @param parameters GA parameters.
 */
void setParametersMutator(int mutatorType, 
            GLParametersGeneticAlgorithm* parameters)
  {
    //set mutator type first.
    parameters->m_mutator_type = mutatorType;
    //if necessary -- prepare additional parameters.
    switch (mutatorType)
    {
      case GLConstants::MUTATOR_GENES_SHUFFLING:
          {
            parameters->m_mutator_params = new 
                GLMutatorShuffling::TParameters(3, true);
            break;
          }
      case GLConstants::MUTATOR_COMBINED:
          {
            //prepare mutators
            vector<GLBaseMutator*> mutators;
            vector<double> weights;
            mutators.push_back(new GLMutatorGeneMove(1.0));
            weights.push_back(1.0);
            mutators.push_back(new GLMutatorGenesSwapping(1.0));
            weights.push_back(1.0);
            parameters->m_mutator_params = new
              GLMutatorCombined::TParameters(mutators, weights,
                        GLMutatorCombined::SEPARATE);
            break;
          }
        
    }
  }//setParametersMutator

/**
 * Function for setting GA selectionist parameters.
 * @param breederType depending on constant's value chooses
 *      which breeder to setup.
 * @param parameters GA parameters.
 */
void setParametersSelectionist(int selectionistType, 
            GLParametersGeneticAlgorithm* parameters)
  {
    //set mutator type first.
    parameters->m_selectionist_type = selectionistType;
    //if necessary -- prepare additional parameters.
    switch (selectionistType)
    {
      case GLConstants::SELECTIONIST_MIXER:
          {
            parameters->m_selectionist_params = new 
                GLSelectionistMixer::TParameters(0.1);
            break;
          }
    }
  }//setParametersSelectionist

void testFullGa()
  {
    GLFactory factory;

    // Set parameters
    cout << "Setting parameters" << endl;
    //GLRandomNumbersGenerator::initGenerator(1241186119);
    GLParametersGeneticAlgorithm *parameters = 
          new GLParametersGeneticAlgorithm();
    setParametersBreeder(GLConstants::BREEDER_STANDARD, parameters);
    setParametersMutator(GLConstants::MUTATOR_GENE_MOVING, parameters);
    setParametersSelectionist(GLConstants::SELECTIONIST_ELITIST, parameters);
    parameters->m_initial_population_size = 100;
    GLGeneticAlgorithmStandard test_ga(*parameters, &factory);
    delete parameters;
    
    cout << "GA created\n";
    
    //Set initial population
    cout << "Initialising data\n";
  GLBaseOrganismInitialiser *initialiser =
                new GLInitialiserShuffleIntVector(50);

    GLBaseEvaluator *evaluator = new GLTestEvaluator();
    TListOfStoppers stoppers;
    
    stoppers.push_back(new GLGaStopperMaxGenerations(3000));
//        stoppers.push_back(new GLGaStopperNoNew(5));
    test_ga.runGA(initialiser, evaluator, stoppers);

    cout << "Total " << evaluator->getCounter()
         << " evaluations were done\n";
    cout << *(test_ga.getStatus());
    //test_ga.getStatus() -> toStream(cout);
    delete initialiser;
    delete evaluator;
  }//testFullGa

void testMultipleGa()
  {
    cout << "Setting parameters" << endl;
    GLParametersGeneticAlgorithm *parameters = 
          new GLParametersGeneticAlgorithm();
    setParametersBreeder(GLConstants::BREEDER_STANDARD, parameters);
//    setParametersMutator(GLConstants::MUTATOR_COMBINED, parameters);
//    setParametersMutator(GLConstants::MUTATOR_GENE_MOVING, parameters);
    setParametersMutator(GLConstants::MUTATOR_GENE_MOVING, parameters);

//    setParametersSelectionist(GLConstants::SELECTIONIST_ELITIST, parameters);
    setParametersSelectionist(GLConstants::SELECTIONIST_MIXER, parameters);
    parameters->m_initial_population_size = 40;
    GLFactory factory;
    GLGeneticAlgorithmStandard ga_1(*parameters, &factory);
    
    delete parameters->m_selectionist_params;
    parameters->m_selectionist_params = new 
                GLSelectionistMixer::TParameters(0.4);

    parameters->m_initial_population_size = 40;
    GLGeneticAlgorithmStandard ga_2(*parameters, &factory);
    delete parameters;
    
    cout << "GA created\n";
    
    //Set initial population
    cout << "Initialising data\n";
    GLBaseOrganismInitialiser *initialiser =
                new GLInitialiserShuffleIntVector(40);

    GLBaseEvaluator *evaluator = new GLTestEvaluator();
    TListOfStoppers stoppers;
    
    stoppers.push_back(new GLGaStopperMaxGenerations(4000));
    
    GLMultipleGaRunner runner;
    
    vector<GLBaseGeneticAlgorithm*> algorithms;
    vector<vector<GLBaseGaStatus*> > statuses;
    algorithms.push_back(&ga_1);
    algorithms.push_back(&ga_2);
    vector<int> runs(algorithms.size(), 10);

    runner.run(algorithms, runs, initialiser, evaluator, stoppers,
               GLConstants::AND_OPERATOR, statuses);

    cout << "Total " << evaluator->getCounter()
         << " evaluations were done\n";
    for(size_t i = 0; i < statuses.size(); ++i)
      {
        GLStatisticsSingleObjective statistics(statuses[i]);
        cout << endl << statistics << endl << flush;
      }
    delete initialiser;
    delete evaluator;
  }//testMultipleGa

int main(int argc, char* argv[])
  {
    //GLRandomNumbersGenerator::initGenerator(1247821263);
    try
     {
       char symbol(0);
       
       cout << "Available tests:\n"
            << "A)TestFullGa\n"
            << "B)TestMultipleGa\n"
            << "q)Quit\n"
            << "What do you want to run? ";
 
       //if parameters passed -- use them.
       if (argc > 1) 
         {
           symbol = argv[1][0];
           cout << symbol << endl;
         }
       if (symbol == 0)
         {
           //otherwise -- read parameter from the standard input;
           cin  >> symbol;
         }
       
       switch(symbol)
       {
         case 'A' : 
           testFullGa();
           break;
         case 'B' : 
           testMultipleGa();
           break;
       }
     }
    catch (runtime_error& error)
      {
        cout << "Logic error\n"
              << error.what() << endl;
      }
    catch (logic_error& error)
      {
        cout << "Runtime error\n"
              << error.what() << endl;
      }
    catch (...)
      {
        cout << "Unidentified error\n";
      }
    cout << "Done\n";
    return 0;
  }
